﻿namespace core.Bitmaps.H2Bitmap
{
    using core;
    using core.H2;
    using core.TagInterface;
    using System;
    using System.Drawing;
    using System.Drawing.Imaging;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    public class H2Bitmap
    {
        public bitm bitmapInterface = null;
        private MetaInterface bitmapMeta = null;

        public H2Bitmap(MetaInterface MetaInterface)
        {
            this.bitmapMeta = MetaInterface;
            this.bitmapInterface = (bitm) MetaInterface.Header.GetChunk(0);
        }

        private byte[] DecodeBitmap(bitm.Bitmap_Data_Layout.BitmapFormat Format, bool Swizzled, byte[] Raw, int Height, int Width, int Depth)
        {
            byte[] buffer2;
            int num2;
            int num3;
            int num = Width;
            if ((Width % 8) != 0)
            {
                Width = (ushort) (Width + (8 - (Width % 8)));
            }
            byte[] buffer = null;
            switch (Format)
            {
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_AY8:
                    if (!Swizzled)
                    {
                        buffer = Raw;
                    }
                    else
                    {
                        buffer = H2Swizzle.Swizzle(Raw, 8, (uint) Width, (uint) Height, (uint) Depth, false);
                    }
                    buffer2 = new byte[buffer.Length * 4];
                    for (num2 = 0; num2 < buffer.Length; num2++)
                    {
                        num3 = num2 * 4;
                        buffer2[num3] = buffer[num2];
                        buffer2[num3 + 1] = buffer[num2];
                        buffer2[num3 + 2] = buffer[num2];
                        buffer2[num3 + 3] = buffer[0xff];
                    }
                    return buffer2;

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A8Y8:
                    if (!Swizzled)
                    {
                        buffer = Raw;
                    }
                    else
                    {
                        buffer = H2Swizzle.Swizzle(Raw, 0x10, (uint) Width, (uint) Height, (uint) Depth, false);
                    }
                    buffer2 = new byte[(buffer.Length / 2) * 4];
                    for (num2 = 0; num2 < (buffer.Length / 2); num2++)
                    {
                        num3 = num2 * 2;
                        buffer2[num3] = buffer[num3 + 1];
                        buffer2[num3 + 1] = buffer[num3 + 1];
                        buffer2[num3 + 2] = buffer[num3 + 1];
                        buffer2[num3 + 3] = buffer[num3];
                    }
                    return buffer2;

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_R5G6B5:
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A1R5G5B5:
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A4R4G4B4:
                    if (!Swizzled)
                    {
                        return Raw;
                    }
                    return H2Swizzle.Swizzle(Raw, 0x10, (uint) Width, (uint) Height, (uint) Depth, false);

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_X8R8G8B8:
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A8R8G8B8:
                    if (!Swizzled)
                    {
                        return Raw;
                    }
                    return H2Swizzle.Swizzle(Raw, 0x20, (uint) Width, (uint) Height, (uint) Depth, false);

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_DXT1:
                    return H2DXT.DecodeDXT1(Height, Width, Raw);

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_DXT2AND3:
                    return H2DXT.DecodeDXT23(Height, Width, Raw);

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_DXT4AND5:
                    return H2DXT.DecodeDXT45(Height, Width, Raw);

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_P8:
                    return H2Quantiziation.Dequantize_Chunk(H2Swizzle.Swizzle(Raw, 8, (uint) Width, (uint) Height, (uint) Depth, false));

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_TYPE_LIGHTMAP:
                    if (!Swizzled)
                    {
                        return Raw;
                    }
                    return H2Swizzle.Swizzle(Raw, 8, (uint) Width, (uint) Height, (uint) Depth, false);

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_TYPE_UNK:
                    if (!Swizzled)
                    {
                        buffer = Raw;
                        break;
                    }
                    buffer = H2Swizzle.Swizzle(Raw, 0x10, (uint) Width, (uint) Height, (uint) Depth, false);
                    break;

                default:
                    return null;
            }
            buffer2 = new byte[(buffer.Length / 2) * 4];
            for (num2 = 0; num2 < (buffer.Length / 2); num2++)
            {
                num3 = num2 * 2;
                buffer2[num3 * 2] = buffer[num3 + 1];
                buffer2[(num3 * 2) + 1] = buffer[num3 + 1];
                buffer2[(num3 * 2) + 2] = buffer[num3 + 1];
                buffer2[(num3 * 2) + 3] = buffer[num3];
            }
            return buffer2;
        }

        private byte[] DecodeBitmapForExtract(bitm.Bitmap_Data_Layout.BitmapFormat Format, bool Swizzled, byte[] Raw, int Height, int Width, int Depth)
        {
            int num = Width;
            if ((Width % 8) != 0)
            {
                Width = (ushort) (Width + (8 - (Width % 8)));
            }
            if ((Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A8R8G8B8) || (Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_X8R8G8B8))
            {
                return H2Swizzle.Swizzle(Raw, 0x20, (uint) Width, (uint) Height, (uint) Depth, false);
            }
            if (((Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A1R5G5B5) || (Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_R5G6B5)) || (Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A4R4G4B4))
            {
                return H2Swizzle.Swizzle(Raw, 0x10, (uint) Width, (uint) Height, (uint) Depth, true);
            }
            if (Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_P8)
            {
                return H2Quantiziation.Dequantize_Chunk(H2Swizzle.Swizzle(Raw, 8, (uint) Width, (uint) Height, (uint) Depth, false));
            }
            return Raw;
        }

        private byte[] DecodeLightmapBitmap(bool Swizzled, byte[] Raw, RGBAColor[] LightmapPalette, int Height, int Width, int Depth, int pixelOffset)
        {
            int num = Width;
            if ((Width % 8) != 0)
            {
                Width = (ushort) (Width + (8 - (Width % 8)));
            }
            byte[] buffer = null;
            buffer = H2Swizzle.Swizzle(Raw, 8, (uint) Width, (uint) Height, (uint) Depth, false);
            byte[] buffer2 = new byte[buffer.Length * 4];
            for (int i = 0; i < buffer.Length; i++)
            {
                int index = i * 4;
                buffer2[index] = LightmapPalette[buffer[i]].r;
                buffer2[index + 1] = LightmapPalette[buffer[i]].g;
                buffer2[index + 2] = LightmapPalette[buffer[i]].b;
                buffer2[index + 3] = LightmapPalette[buffer[i]].a;
            }
            return buffer2;
        }

        private byte[] EncodeBitmapForInject(bitm.Bitmap_Data_Layout.BitmapFormat Format, bool Swizzled, byte[] Raw, int Height, int Width, int Depth)
        {
            int num = Width;
            if ((Width % 8) != 0)
            {
                Width = (ushort) (Width + (8 - (Width % 8)));
            }
            if ((Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A8R8G8B8) || (Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_X8R8G8B8))
            {
                return H2Swizzle.Swizzle(Raw, 0x20, (uint) Width, (uint) Height, (uint) Depth, true);
            }
            if (((Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A1R5G5B5) || (Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_R5G6B5)) || (Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A4R4G4B4))
            {
                return H2Swizzle.Swizzle(Raw, 0x10, (uint) Width, (uint) Height, (uint) Depth, false);
            }
            if (Format == bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_P8)
            {
                return H2Swizzle.Swizzle(H2Quantiziation.Requantize_Chunk(ref Raw), 8, (uint) Width, (uint) Height, (uint) Depth, true);
            }
            return Raw;
        }

        public void ExtractImage(int Reflexive, int LOD)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = "DDS Image|*.dds";
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                bitm.Bitmap_Data_Layout chunk = (bitm.Bitmap_Data_Layout) this.bitmapInterface.Bitmap_Data.GetChunk(Reflexive);
                int height = chunk.Height.Value;
                int width = chunk.Width.Value;
                int depth = chunk.Depth.Value;
                bool swizzled = !IsBitSet2(chunk.Flags.Value, 3);
                FileStream output = new FileStream(dialog.FileName, FileMode.Create, FileAccess.Write);
                BinaryWriter bw = new BinaryWriter(output);
                H2DDS.WriteDDSHeader(ref bw, chunk, LOD);
                bw.Write(this.DecodeBitmapForExtract(chunk.Bitmap_Format, swizzled, chunk.rawdata.LOD_MS[LOD].ToArray(), height, width, depth));
                bw.Close();
                output.Close();
                MessageBox.Show("Bitmap Extracted");
            }
        }

        public MemoryStream ExtractImageMS(int Reflexive, int LOD)
        {
            bitm.Bitmap_Data_Layout chunk = (bitm.Bitmap_Data_Layout) this.bitmapInterface.Bitmap_Data.GetChunk(Reflexive);
            int height = chunk.Height.Value;
            int width = chunk.Width.Value;
            int depth = chunk.Depth.Value;
            bool swizzled = !IsBitSet2(chunk.Flags.Value, 3);
            MemoryStream output = new MemoryStream();
            BinaryWriter bw = new BinaryWriter(output);
            H2DDS.WriteDDSHeader(ref bw, chunk, LOD);
            bw.Write(this.DecodeBitmapForExtract(chunk.Bitmap_Format, swizzled, chunk.rawdata.LOD_MS[LOD].ToArray(), height, width, depth));
            bw.BaseStream.Position = 0L;
            return output;
        }

        private Bitmap GenerateBitmap(bitm.Bitmap_Data_Layout.BitmapFormat Format, byte[] DecodedRaw, int Height, int Width, int Depth)
        {
            int num = Width;
            if ((Width % 8) != 0)
            {
                Width = (ushort) (Width + (8 - (Width % 8)));
            }
            int stride = Width;
            PixelFormat undefined = PixelFormat.Undefined;
            switch (Format)
            {
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_AY8:
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_P8:
                    undefined = PixelFormat.Format32bppArgb;
                    stride *= 4;
                    break;

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A8Y8:
                    undefined = PixelFormat.Format32bppArgb;
                    stride *= 4;
                    break;

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_R5G6B5:
                    stride *= 2;
                    undefined = PixelFormat.Format16bppRgb565;
                    break;

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A1R5G5B5:
                    stride *= 2;
                    undefined = PixelFormat.Format16bppRgb555;
                    break;

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A4R4G4B4:
                    stride *= 2;
                    undefined = PixelFormat.Format16bppRgb565;
                    break;

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_X8R8G8B8:
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_A8R8G8B8:
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_DXT1:
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_DXT2AND3:
                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_FORMAT_DXT4AND5:
                    stride *= 4;
                    undefined = PixelFormat.Format32bppArgb;
                    break;

                case bitm.Bitmap_Data_Layout.BitmapFormat.BITM_TYPE_UNK:
                    stride *= 2;
                    undefined = PixelFormat.Format32bppArgb;
                    break;

                default:
                {
                    Bitmap image = new Bitmap(Width, Height);
                    Graphics graphics = Graphics.FromImage(image);
                    graphics.Clear(Color.Red);
                    string text = "No Preview Available";
                    SizeF ef = graphics.MeasureString(text, new Font(FontFamily.GenericSerif, 9f));
                    graphics.DrawString(text, new Font(FontFamily.GenericSerif, 9f), Brushes.Yellow, new PointF(Height - (ef.Width / 2f), Width - (ef.Height / 2f)));
                    return image;
                }
            }
            Marshal.FreeHGlobal(new IntPtr());
            IntPtr src = Marshal.AllocHGlobal(DecodedRaw.Length);
            RtlMoveMemory(src, DecodedRaw, DecodedRaw.Length);
            return new Bitmap(Width, Height, stride, undefined, src);
        }

        private Bitmap GenerateLightmapBitmap(byte[] DecodedRaw, int Height, int Width, int Depth)
        {
            int num = Width;
            if ((Width % 8) != 0)
            {
                Width = (ushort) (Width + (8 - (Width % 8)));
            }
            Marshal.FreeHGlobal(new IntPtr());
            IntPtr src = Marshal.AllocHGlobal(DecodedRaw.Length);
            RtlMoveMemory(src, DecodedRaw, DecodedRaw.Length);
            return new Bitmap(Width, Height, Width * 4, PixelFormat.Format32bppArgb, src);
        }

        public Bitmap GenerateLightmapPreview(int LightmapReflexive, RGBAColor[] LightmapPalette)
        {
            bitm.Bitmap_Data_Layout chunk = (bitm.Bitmap_Data_Layout) this.bitmapInterface.Bitmap_Data.GetChunk(LightmapReflexive);
            int height = chunk.Height.Value;
            int width = chunk.Width.Value;
            int depth = chunk.Depth.Value;
            int pixelOffset = chunk.PixelOffset.Value;
            bool swizzled = !IsBitSet2(chunk.Flags.Value, 3);
            byte[] decodedRaw = this.DecodeLightmapBitmap(swizzled, chunk.rawdata.LOD_MS[0].ToArray(), LightmapPalette, height, width, depth, pixelOffset);
            return this.GenerateLightmapBitmap(decodedRaw, height, width, depth);
        }

        public Bitmap GeneratePreview(int Reflexive, int LOD)
        {
            bitm.Bitmap_Data_Layout chunk = (bitm.Bitmap_Data_Layout) this.bitmapInterface.Bitmap_Data.GetChunk(Reflexive);
            int height = chunk.Height.Value;
            int width = chunk.Width.Value;
            int depth = chunk.Depth.Value;
            bool swizzled = !IsBitSet2(chunk.Flags.Value, 3);
            byte[] decodedRaw = this.DecodeBitmap(chunk.Bitmap_Format, swizzled, chunk.rawdata.LOD_MS[LOD].ToArray(), height, width, depth);
            return this.GenerateBitmap(chunk.Bitmap_Format, decodedRaw, height, width, depth);
        }

        public int GetLODCount(int BitmapReflexiveIndex)
        {
            bitm.Bitmap_Data_Layout chunk = (bitm.Bitmap_Data_Layout) this.bitmapInterface.Bitmap_Data.GetChunk(BitmapReflexiveIndex);
            return chunk.rawdata.LOD_MS.Length;
        }

        public void InjectImage(int Reflexive, int LOD)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = "DDS Image|*.dds";
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                FileStream input = new FileStream(dialog.FileName, FileMode.Open, FileAccess.Read);
                BinaryReader reader = new BinaryReader(input);
                reader.BaseStream.Position = 0x80L;
                bitm.Bitmap_Data_Layout chunk = (bitm.Bitmap_Data_Layout) this.bitmapInterface.Bitmap_Data.GetChunk(Reflexive);
                int height = chunk.Height.Value;
                int width = chunk.Width.Value;
                int depth = chunk.Depth.Value;
                bool swizzled = !IsBitSet2(chunk.Flags.Value, 3);
                BinaryWriter writer = new BinaryWriter(chunk.rawdata.LOD_MS[LOD]);
                writer.BaseStream.Position = 0L;
                writer.Write(this.EncodeBitmapForInject(chunk.Bitmap_Format, swizzled, reader.ReadBytes(((int) reader.BaseStream.Length) - 0x80), height, width, depth));
                reader.Close();
                input.Close();
                MessageBox.Show("Bitmap Injected");
            }
        }

        private static bool IsBitSet2(int bitmask, int bitpositionvalue)
        {
            while (bitpositionvalue > 1)
            {
                bitpositionvalue = bitpositionvalue >> 1;
                bitmask = bitmask >> 1;
            }
            return ((bitmask & 1) == 1);
        }

        [DllImport("kernel32.dll")]
        private static extern void RtlMoveMemory(IntPtr src, byte[] crap, int cb);
    }
}

