﻿namespace core.Bitmaps.H2Bitmap
{
    using System;

    public sealed class Swizzler
    {
        private static int Swizzle(int x, int y, int z, MaskSet masks)
        {
            return ((SwizzleAxis(x, masks.x) | SwizzleAxis(y, masks.y)) | ((z == -1) ? 0 : SwizzleAxis(z, masks.z)));
        }

        public static byte[] Swizzle(byte[] raw, int width, int height, int depth, int bitCount, bool deswizzle)
        {
            return Swizzle(raw, 0, width, height, depth, bitCount, deswizzle);
        }

        public static byte[] Swizzle(byte[] raw, int offset, int width, int height, int depth, int bitCount, bool deswizzle)
        {
            bitCount /= 8;
            int num = 0;
            int num2 = 0;
            int length = raw.Length;
            byte[] buffer = new byte[length];
            MaskSet masks = new MaskSet(width, height, depth);
            offset = 0;
            for (int i = 0; i < height; i++)
            {
                for (int j = 0; j < width; j++)
                {
                    if (deswizzle)
                    {
                        num = ((i * width) + j) * bitCount;
                        num2 = Swizzle(j, i, -1, masks) * bitCount;
                    }
                    else
                    {
                        num2 = ((i * width) + j) * bitCount;
                        num = Swizzle(j, i, -1, masks) * bitCount;
                    }
                    for (int k = offset; k < (bitCount + offset); k++)
                    {
                        buffer[num + k] = raw[num2 + k];
                    }
                }
            }
            return buffer;
        }

        private static int SwizzleAxis(int val, int mask)
        {
            int num = 1;
            int num2 = 0;
            while (num <= mask)
            {
                int num3 = mask & num;
                if (num3 != 0)
                {
                    num2 |= val & num;
                }
                else
                {
                    val = val << 1;
                }
                num = num << 1;
            }
            return num2;
        }

        private class MaskSet
        {
            public int x = 0;
            public int y = 0;
            public int z = 0;

            public MaskSet(int w, int h, int d)
            {
                int num = 1;
                int num2 = 1;
                while (((num < w) || (num < h)) || (num < d))
                {
                    if (num < w)
                    {
                        this.x |= num2;
                        num2 = num2 << 1;
                    }
                    if (num < h)
                    {
                        this.y |= num2;
                        num2 = num2 << 1;
                    }
                    if (num < d)
                    {
                        this.z |= num2;
                        num2 = num2 << 1;
                    }
                    num = num << 1;
                }
            }
        }
    }
}

