﻿namespace core
{
    using core.H2;
    using core.TagInterface;
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    public class CompiledTagInterface : MetaInterface
    {
        private int ident;
        private int magic;
        private Map map;
        private int offset;
        private int tagnumber;

        public CompiledTagInterface(Map.HaloVersionEnum type) : base(MetaInterface.InterfaceMode.CompiledTagMode)
        {
            base.type = type;
        }

        public CompiledTagInterface(int tagnumber, Map map) : base(MetaInterface.InterfaceMode.CompiledTagMode)
        {
            this.map = map;
            base.type = map.HaloVersion;
            base.tagdef = Tag_Layouts.GetTagCategory(map.HaloVersion).GetTagDefinitionByTagType(map.Meta[tagnumber].TagType);
            if (base.tagdef != null)
            {
                base.Header = new TagBlock(base.tagdef.TagType, base.tagdef.Layout, 1, map.HaloVersion);
                this.tagnumber = tagnumber;
                this.ident = map.Meta[this.Tag_Number].Ident;
                base.tagtype = map.Meta[this.Tag_Number].TagType;
                base.tagname = map.FileName[this.Tag_Number].Name;
                this.offset = map.Meta[this.Tag_Number].Offset;
                this.magic = Meta.GetMetaMagic(this.Tag_Number, ref map);
                map.io.Open();
                this.Read();
                if (base.tagtype == "snd!")
                {
                    snd_ chunk = (snd_) base.Header.GetChunk(0);
                    chunk.Sound_Resource = new Sound_Resource_Layout();
                    chunk.Sound_Resource.LoadSoundDataFromUgh(map, chunk, map.ugh_Interface);
                }
                map.io.Close();
            }
        }

        public override void Read()
        {
            base.Header.Translation.Memory_Address = this.offset + this.magic;
            base.Header.SetMapOffset(this.Offset);
            base.Header.Translation.Map_Translation_Offset = this.Offset;
            this.RecursivelyReadElements(base.Header);
        }

        private void RecursivelyReadElements(TagBlock tagblock)
        {
            Stack<TagBlock> stack = new Stack<TagBlock>();
            stack.Push(tagblock);
            while (stack.Count > 0)
            {
                TagBlock block = stack.Pop();
                Application.DoEvents();
                if (block.ChunkCount != 0)
                {
                    this.map.io.In.SeekTo(block.Translation.Map_Translation_Offset);
                    block.ReadChunks(ref this.map.io.In, base.mode, this.map.HaloVersion);
                    foreach (TagBlockLayout layout in block.chunks)
                    {
                        foreach (BaseObject obj2 in layout)
                        {
                            int num = (block.Translation.Meta_Translation_Offset + obj2.Offset) + (block.ChunkSize * layout.Chunk_Number);
                            obj2.SetMetaOffset(num);
                            obj2.SetMapOffset(obj2.MetaOffset + this.Offset);
                            if (obj2.Type == BaseObjectType.TagIdent)
                            {
                                ((TagIdent) obj2).Load(this.map);
                            }
                            else if (obj2.Type == BaseObjectType.TagReference)
                            {
                                TagReference reference = (TagReference) obj2;
                                reference.Ident.Load(this.map);
                            }
                            else if (obj2.Type == BaseObjectType.StringID)
                            {
                                core.TagInterface.StringID gid = (core.TagInterface.StringID) obj2;
                                gid.Value = this.map.StringIDs[gid.Index].Name;
                            }
                            else
                            {
                                if (obj2.Type == BaseObjectType.Raw_Tag_Block)
                                {
                                    Raw_Tag_Block block2 = (Raw_Tag_Block) obj2;
                                    block2.Tag_Ident.Load(this.map);
                                    TagBlock item = block2.Resource_Block;
                                    item.Translation.Load(this.map, this.magic);
                                    stack.Push(item);
                                    if (block2.Raw_Offset.Data_Offset == -1)
                                    {
                                        continue;
                                    }
                                    switch (block2.Raw_Offset.Map_Location)
                                    {
                                        case MapTypes.Internal:
                                            this.map.io.In.SeekTo(block2.Raw_Offset.Data_Offset);
                                            block2.Read_Raw(ref this.map.io.In);
                                            goto Label_0772;

                                        case MapTypes.MainMenu:
                                            ProgramSettings.Main_Menu_IO.In.SeekTo(block2.Raw_Offset.Data_Offset);
                                            block2.Read_Raw(ref ProgramSettings.Main_Menu_IO.In);
                                            goto Label_0772;

                                        case MapTypes.SPShared:
                                            ProgramSettings.SP_Shared_IO.In.SeekTo(block2.Raw_Offset.Data_Offset);
                                            block2.Read_Raw(ref ProgramSettings.SP_Shared_IO.In);
                                            goto Label_0772;

                                        case MapTypes.MPShared:
                                            ProgramSettings.MP_Shared_IO.In.SeekTo(block2.Raw_Offset.Data_Offset);
                                            block2.Read_Raw(ref ProgramSettings.MP_Shared_IO.In);
                                            goto Label_0772;
                                    }
                                }
                                else if (obj2.Type != BaseObjectType.Sound_Raw)
                                {
                                    if (obj2.Type == BaseObjectType.Animation_Raw)
                                    {
                                        Animation_Raw raw = (Animation_Raw) obj2;
                                        switch (raw.Raw_Offset.Map_Location)
                                        {
                                            case MapTypes.Internal:
                                                this.map.io.In.SeekTo(raw.Raw_Offset.Data_Offset);
                                                raw.Read_Raw(ref this.map.io.In);
                                                goto Label_0772;

                                            case MapTypes.MainMenu:
                                                ProgramSettings.Main_Menu_IO.In.SeekTo(raw.Raw_Offset.Data_Offset);
                                                raw.Read_Raw(ref ProgramSettings.Main_Menu_IO.In);
                                                goto Label_0772;

                                            case MapTypes.SPShared:
                                                ProgramSettings.SP_Shared_IO.In.SeekTo(raw.Raw_Offset.Data_Offset);
                                                raw.Read_Raw(ref ProgramSettings.SP_Shared_IO.In);
                                                goto Label_0772;

                                            case MapTypes.MPShared:
                                                ProgramSettings.MP_Shared_IO.In.SeekTo(raw.Raw_Offset.Data_Offset);
                                                raw.Read_Raw(ref ProgramSettings.MP_Shared_IO.In);
                                                goto Label_0772;
                                        }
                                    }
                                    else if (obj2.Type == BaseObjectType.Syntax_Block)
                                    {
                                        Syntax_Block block4 = (Syntax_Block) obj2;
                                        if (block4.Is_String_ID_Referenced)
                                        {
                                            block4.String_ID.SetMetaOffset(block4.MetaOffset + 0x10);
                                            block4.String_ID.SetMapOffset((this.offset + block4.MetaOffset) + 0x10);
                                            block4.String_ID.Value = this.map.StringIDs[block4.String_ID.Index].Name;
                                        }
                                        else if (block4.Is_Tag_Referenced)
                                        {
                                            block4.Tag_Reference.SetMetaOffset(block4.MetaOffset + 0x10);
                                            block4.Tag_Reference.SetMapOffset((this.offset + block4.MetaOffset) + 0x10);
                                            block4.Tag_Reference.Load(this.map);
                                        }
                                    }
                                    else if (obj2.Type == BaseObjectType.Unicode_Raw)
                                    {
                                        Unicode_Raw raw2 = (Unicode_Raw) obj2;
                                        raw2.Unicode_Strings.Translation.Map_Translation_Offset = raw2.Index_Offset;
                                        stack.Push(raw2.Unicode_Strings);
                                        this.map.io.In.SeekTo(raw2.Table_Offset);
                                        raw2.Read_Raw(ref this.map.io.In);
                                    }
                                    else if (obj2.Type == BaseObjectType.Bitmap_Raw)
                                    {
                                        Bitmap_Raw raw3 = (Bitmap_Raw) obj2;
                                        for (int i = 0; i < 6; i++)
                                        {
                                            if (raw3.LOD_Raw_Data_Offset[i].Data_Offset != -1)
                                            {
                                                switch (raw3.LOD_Raw_Data_Offset[i].Map_Location)
                                                {
                                                    case MapTypes.Internal:
                                                        this.map.io.In.SeekTo(raw3.LOD_Raw_Data_Offset[i].Data_Offset);
                                                        raw3.Read_LOD_Raw(ref this.map.io.In, i);
                                                        break;

                                                    case MapTypes.MainMenu:
                                                        ProgramSettings.Main_Menu_IO.In.SeekTo(raw3.LOD_Raw_Data_Offset[i].Data_Offset);
                                                        raw3.Read_LOD_Raw(ref ProgramSettings.Main_Menu_IO.In, i);
                                                        break;

                                                    case MapTypes.SPShared:
                                                        ProgramSettings.SP_Shared_IO.In.SeekTo(raw3.LOD_Raw_Data_Offset[i].Data_Offset);
                                                        raw3.Read_LOD_Raw(ref ProgramSettings.SP_Shared_IO.In, i);
                                                        break;

                                                    case MapTypes.MPShared:
                                                        ProgramSettings.MP_Shared_IO.In.SeekTo(raw3.LOD_Raw_Data_Offset[i].Data_Offset);
                                                        raw3.Read_LOD_Raw(ref ProgramSettings.MP_Shared_IO.In, i);
                                                        break;
                                                }
                                            }
                                        }
                                    }
                                    else if (obj2.Type == BaseObjectType.TagBlock)
                                    {
                                        TagBlock block5 = (TagBlock) obj2;
                                        block5.Translation.Load(this.map, this.magic);
                                        stack.Push(block5);
                                    }
                                }
                            Label_0772:;
                            }
                        }
                    }
                }
            }
        }

        public int Ident
        {
            get
            {
                return this.ident;
            }
        }

        public int MetaMagic
        {
            get
            {
                return this.magic;
            }
        }

        public int Offset
        {
            get
            {
                return this.offset;
            }
        }

        public int Tag_Number
        {
            get
            {
                return this.tagnumber;
            }
        }
    }
}

