﻿namespace core.H1
{
    using core.TagInterface;
    using System;

    public class SBSP : TagBlockLayout
    {
        public TagBlock bsp = new TagBlock("BSP Collision", BSP.Layout, Map.HaloVersionEnum.HaloCE);
        public RealRGBColor default_ambient_color = new RealRGBColor("Default Ambient Color");
        public RealRGBColor default_distant_light_one_color = new RealRGBColor("Default Distant Light 1 Color");
        public RealVector3D default_distant_light_one_direction = new RealVector3D("Default Distant Light 1 Direction");
        public RealRGBColor default_distant_light_zero_color = new RealRGBColor("Default Distant Light 0 Color");
        public RealVector3D default_distant_light_zero_direction = new RealVector3D("Default Distant Light 0 Direction");
        public RealARGBColor default_reflection_tint = new RealARGBColor("Default Reflection Tint");
        public RealRGBColor default_shadow_color = new RealRGBColor("Default Shadow Color");
        public RealVector3D default_shadow_vector = new RealVector3D("Default Shadow Vector");
        public static readonly SBSP Layout = new SBSP();
        public TagReference light_map_reference = new TagReference("Light Map Tag Reference", Map.HaloVersionEnum.HaloCE);
        public Padding temppadding1 = new Padding("TempPadding", 0x18);
        public Padding temppadding2 = new Padding("TempPadding", 0x34);
        public RealFloat vehicle_ceiling = new RealFloat("Vehicle Ceiling");
        public RealFloat vehicle_floor = new RealFloat("Vehicle Floor");

        public SBSP()
        {
            base.Add(this.temppadding1);
            base.Add(this.light_map_reference);
            base.Add(this.vehicle_floor);
            base.Add(this.vehicle_ceiling);
            base.Add(this.default_ambient_color);
            base.Add(this.default_distant_light_zero_color);
            base.Add(this.default_distant_light_zero_direction);
            base.Add(this.default_distant_light_one_color);
            base.Add(this.default_distant_light_one_direction);
            base.Add(this.default_reflection_tint);
            base.Add(this.default_shadow_vector);
            base.Add(this.default_shadow_color);
            base.Add(this.temppadding2);
            base.Add(this.bsp);
        }

        public class BSP : TagBlockLayout
        {
            public TagBlock bsp_2d_nodes = new TagBlock("BSP_2D_Nodes", BSP_2D_Node.Layout, Map.HaloVersionEnum.Halo1);
            public TagBlock bsp_2d_references = new TagBlock("BSP_2D_References", BSP_2D_Reference.Layout, Map.HaloVersionEnum.Halo1);
            public TagBlock bsp_3d_nodes = new TagBlock("BSP_3D_Nodes", BSP3DNode.Layout, Map.HaloVersionEnum.Halo1);
            public TagBlock bsp_edges = new TagBlock("BSP_Edges", Edge.Layout, Map.HaloVersionEnum.Halo1);
            public TagBlock bsp_leaves = new TagBlock("BSP_Leaves", Leaf.Layout, Map.HaloVersionEnum.Halo1);
            public TagBlock bsp_planes = new TagBlock("BSP_Planes", BSP_Plane.Layout, Map.HaloVersionEnum.Halo1);
            public TagBlock bsp_surfaces = new TagBlock("BSP_Surfaces", BSP_Surface.Layout, Map.HaloVersionEnum.Halo1);
            public TagBlock bsp_vertices = new TagBlock("BSP_Vertices", Vertice.Layout, Map.HaloVersionEnum.Halo1);
            public static readonly SBSP.BSP Layout = new SBSP.BSP();

            public BSP()
            {
                base.Add(this.bsp_3d_nodes);
                base.Add(this.bsp_planes);
                base.Add(this.bsp_leaves);
                base.Add(this.bsp_2d_references);
                base.Add(this.bsp_2d_nodes);
                base.Add(this.bsp_surfaces);
                base.Add(this.bsp_edges);
                base.Add(this.bsp_vertices);
            }

            public class BSP_2D_Node : TagBlockLayout
            {
                public static readonly SBSP.BSP.BSP_2D_Node Layout = new SBSP.BSP.BSP_2D_Node();
                public LongInteger leftchild = new LongInteger("Left Child");
                public RealPlane2D plane = new RealPlane2D("Plane");
                public LongInteger rightchild = new LongInteger("Right Child");

                public BSP_2D_Node()
                {
                    base.Add(this.plane);
                    base.Add(this.leftchild);
                    base.Add(this.rightchild);
                }
            }

            public class BSP_2D_Reference : TagBlockLayout
            {
                public LongInteger bsp_2d_node_index = new LongInteger("BSP_2D_Node Index");
                public static readonly SBSP.BSP.BSP_2D_Reference Layout = new SBSP.BSP.BSP_2D_Reference();
                public LongInteger plane_index = new LongInteger("BSP_Plane Index");

                public BSP_2D_Reference()
                {
                    base.Add(this.plane_index);
                    base.Add(this.bsp_2d_node_index);
                }
            }

            public class BSP_Plane : TagBlockLayout
            {
                public static readonly SBSP.BSP.BSP_Plane Layout = new SBSP.BSP.BSP_Plane();
                public RealPlane3D plane = new RealPlane3D("BSP_Plane");

                public BSP_Plane()
                {
                    base.Add(this.plane);
                }
            }

            public class BSP_Surface : TagBlockLayout
            {
                public ByteInteger breakablesurface_index = new ByteInteger("Breakable Surface Index");
                public ShortInteger collision_material_index = new ShortInteger("Collision Material Index");
                public LongInteger first_edge = new LongInteger("First Edge");
                public core.TagInterface.Flags flags = new core.TagInterface.Flags("Flags", core.TagInterface.Flags.BitmaskTypeEnum.Byte);
                public static readonly SBSP.BSP.BSP_Surface Layout = new SBSP.BSP.BSP_Surface();
                public LongInteger plane_index = new LongInteger("Plane Index");

                public BSP_Surface()
                {
                    base.Add(this.plane_index);
                    base.Add(this.first_edge);
                    base.Add(this.flags);
                    base.Add(this.breakablesurface_index);
                    base.Add(this.collision_material_index);
                }
            }

            public class BSP3DNode : TagBlockLayout
            {
                public ShortInteger backchild = new ShortInteger("Back Child");
                public ShortInteger frontchild = new ShortInteger("Front Child");
                public static readonly SBSP.BSP.BSP3DNode Layout = new SBSP.BSP.BSP3DNode();
                public LongInteger plane_index = new LongInteger("BSP_Plane Index");
                public ShortInteger unknown1 = new ShortInteger("unknown1");
                public ByteInteger unknown2 = new ByteInteger("unknown2");
                public ByteInteger unknownx = new ByteInteger("unknownx");

                public BSP3DNode()
                {
                    base.Add(this.plane_index);
                    base.Add(this.backchild);
                    base.Add(this.unknown1);
                    base.Add(this.frontchild);
                    base.Add(this.unknownx);
                    base.Add(this.unknown2);
                }
            }

            public class Edge : TagBlockLayout
            {
                public LongInteger EndVertex = new LongInteger("End Vertex");
                public LongInteger ForwardEdge = new LongInteger("Forward Edge");
                public static readonly SBSP.BSP.Edge Layout = new SBSP.BSP.Edge();
                public LongInteger LeftSurface = new LongInteger("Left Surface");
                public LongInteger ReverseEdge = new LongInteger("Reverse Edge");
                public LongInteger RightSurface = new LongInteger("Right Surface");
                public LongInteger StartVertex = new LongInteger("Start Vertex");

                public Edge()
                {
                    base.Add(this.StartVertex);
                    base.Add(this.EndVertex);
                    base.Add(this.ForwardEdge);
                    base.Add(this.ReverseEdge);
                    base.Add(this.LeftSurface);
                    base.Add(this.RightSurface);
                }
            }

            public class Leaf : TagBlockLayout
            {
                public ShortInteger bsp2drefcount = new ShortInteger("BSP_2D_Reference Count");
                public LongInteger bsp2drefindex = new LongInteger("BSP_2D_Reference Start Index");
                public core.TagInterface.Flags flags = new core.TagInterface.Flags("Flags", core.TagInterface.Flags.BitmaskTypeEnum.Word);
                public static readonly SBSP.BSP.Leaf Layout = new SBSP.BSP.Leaf();

                public Leaf()
                {
                    base.Add(this.flags);
                    base.Add(this.bsp2drefcount);
                    base.Add(this.bsp2drefindex);
                }
            }

            public class Vertice : TagBlockLayout
            {
                public LongInteger firstedge = new LongInteger("First Edge");
                public static readonly SBSP.BSP.Vertice Layout = new SBSP.BSP.Vertice();
                public RealPoint3D vertice = new RealPoint3D("Vertice");

                public Vertice()
                {
                    base.Add(this.vertice);
                    base.Add(this.firstedge);
                }
            }
        }
    }
}

