﻿namespace core.H2
{
    using core.TagInterface;
    using System;

    public class coll : TagBlockLayout
    {
        public static readonly coll Layout = new coll();
        public TagBlock materials = new TagBlock("Materials", Material.Layout, Map.HaloVersionEnum.Halo2);
        public TagBlock nodes = new TagBlock("Nodes", Node.Layout, Map.HaloVersionEnum.Halo2);
        public Padding padding = new Padding(20);
        public TagBlock pathfinding_spheres = new TagBlock("Pathfinding Sphere", Pathfinding_Sphere.Layout, Map.HaloVersionEnum.Halo2);
        public TagBlock regions = new TagBlock("Regions", Region.Layout, Map.HaloVersionEnum.Halo2);

        public coll()
        {
            base.Add(this.padding);
            base.Add(this.materials);
            base.Add(this.regions);
            base.Add(this.pathfinding_spheres);
            base.Add(this.nodes);
        }

        public class Material : TagBlockLayout
        {
            public static readonly coll.Material Layout = new coll.Material();
            private StringID name = new StringID("Name");

            public Material()
            {
                base.Add(this.name);
            }
        }

        public class Node : TagBlockLayout
        {
            public TagBlockIndex child_node = new TagBlockIndex("Child Node", TagBlockIndex.TagBlockIndexType.Word);
            public static readonly coll.Node Layout = new coll.Node();
            public StringID name = new StringID("Name");
            public TagBlockIndex parent_node = new TagBlockIndex("Parent Node", TagBlockIndex.TagBlockIndexType.Word);
            public TagBlockIndex region = new TagBlockIndex("Region", TagBlockIndex.TagBlockIndexType.Word);
            public TagBlockIndex sibling_node = new TagBlockIndex("Sibling Node", TagBlockIndex.TagBlockIndexType.Word);

            public Node()
            {
                base.Add(this.name);
                base.Add(this.region);
                base.Add(this.parent_node);
                base.Add(this.sibling_node);
                base.Add(this.child_node);
            }
        }

        public class Pathfinding_Sphere : TagBlockLayout
        {
            public RealPoint3D center = new RealPoint3D("Center");
            public static readonly coll.Pathfinding_Sphere Layout = new coll.Pathfinding_Sphere();
            public TagBlockIndex node = new TagBlockIndex("Node", TagBlockIndex.TagBlockIndexType.Long);
            public RealFloat radius = new RealFloat("Radius");

            public Pathfinding_Sphere()
            {
                base.Add(this.node);
                base.Add(this.center);
                base.Add(this.radius);
            }
        }

        public class Region : TagBlockLayout
        {
            public static readonly coll.Region Layout = new coll.Region();
            public StringID name = new StringID("Name");
            public TagBlock permutation = new TagBlock("Permutations", Permutation.Layout, Map.HaloVersionEnum.Halo2);

            public Region()
            {
                base.Add(this.name);
                base.Add(this.permutation);
            }

            public class Permutation : TagBlockLayout
            {
                public TagBlock bsp = new TagBlock("BSP", BSP.Layout, Map.HaloVersionEnum.Halo2);
                public static readonly coll.Region.Permutation Layout = new coll.Region.Permutation();
                public StringID name = new StringID("Name");
                public TagBlock unk1 = new TagBlock("unknown", Unknown_Tag_Block.Layout, Map.HaloVersionEnum.Halo2);

                public Permutation()
                {
                    base.Add(this.name);
                    base.Add(this.bsp);
                    base.Add(this.unk1);
                }

                public class BSP : TagBlockLayout
                {
                    public TagBlock bsp_2d_nodes = new TagBlock("BSP_2D_Nodes", BSP_2D_Node.Layout, Map.HaloVersionEnum.Halo2, 0x10);
                    public TagBlock bsp_2d_references = new TagBlock("BSP_2D_References", BSP_2D_Reference.Layout, Map.HaloVersionEnum.Halo2);
                    public TagBlock bsp_3d_nodes = new TagBlock("BSP_3D_Nodes", BSP3DNode.Layout, Map.HaloVersionEnum.Halo2, 8);
                    public TagBlock bsp_edges = new TagBlock("BSP_Edges", Edge.Layout, Map.HaloVersionEnum.Halo2);
                    public TagBlock bsp_leaves = new TagBlock("BSP_Leaves", Leaf.Layout, Map.HaloVersionEnum.Halo2, 0x10);
                    public TagBlock bsp_planes = new TagBlock("BSP_Planes", BSP_Plane.Layout, Map.HaloVersionEnum.Halo2, 0x10);
                    public TagBlock bsp_surfaces = new TagBlock("BSP_Surfaces", BSP_Surface.Layout, Map.HaloVersionEnum.Halo2, 8);
                    public TagBlock bsp_vertices = new TagBlock("BSP_Vertices", Vertice.Layout, Map.HaloVersionEnum.Halo2, 0x10);
                    public static readonly coll.Region.Permutation.BSP Layout = new coll.Region.Permutation.BSP();
                    public LongInteger unknown1 = new LongInteger("Unknown1");

                    public BSP()
                    {
                        base.Add(this.unknown1);
                        base.Add(this.bsp_3d_nodes);
                        base.Add(this.bsp_planes);
                        base.Add(this.bsp_leaves);
                        base.Add(this.bsp_2d_references);
                        base.Add(this.bsp_2d_nodes);
                        base.Add(this.bsp_surfaces);
                        base.Add(this.bsp_edges);
                        base.Add(this.bsp_vertices);
                    }

                    public class BSP_2D_Node : TagBlockLayout
                    {
                        public static readonly coll.Region.Permutation.BSP.BSP_2D_Node Layout = new coll.Region.Permutation.BSP.BSP_2D_Node();
                        public ShortInteger leftchild = new ShortInteger("Left Child");
                        public RealPlane2D plane = new RealPlane2D("Plane");
                        public ShortInteger rightchild = new ShortInteger("Right Child");

                        public BSP_2D_Node()
                        {
                            base.Add(this.plane);
                            base.Add(this.leftchild);
                            base.Add(this.rightchild);
                        }
                    }

                    public class BSP_2D_Reference : TagBlockLayout
                    {
                        public ShortInteger bsp_2d_node_index = new ShortInteger("BSP_2D_Node Index");
                        public static readonly coll.Region.Permutation.BSP.BSP_2D_Reference Layout = new coll.Region.Permutation.BSP.BSP_2D_Reference();
                        public ShortInteger plane_index = new ShortInteger("BSP_Plane Index");

                        public BSP_2D_Reference()
                        {
                            base.Add(this.plane_index);
                            base.Add(this.bsp_2d_node_index);
                        }
                    }

                    public class BSP_Plane : TagBlockLayout
                    {
                        public static readonly coll.Region.Permutation.BSP.BSP_Plane Layout = new coll.Region.Permutation.BSP.BSP_Plane();
                        public RealPlane3D plane = new RealPlane3D("BSP_Plane");

                        public BSP_Plane()
                        {
                            base.Add(this.plane);
                        }
                    }

                    public class BSP_Surface : TagBlockLayout
                    {
                        public ByteInteger breakablesurface_index = new ByteInteger("Breakable Surface Index");
                        public ShortInteger collision_material_index = new ShortInteger("Collision Material Index");
                        public ShortInteger first_edge = new ShortInteger("First Edge");
                        public core.TagInterface.Flags flags = new core.TagInterface.Flags("Flags", core.TagInterface.Flags.BitmaskTypeEnum.Byte);
                        public static readonly coll.Region.Permutation.BSP.BSP_Surface Layout = new coll.Region.Permutation.BSP.BSP_Surface();
                        public ShortInteger plane_index = new ShortInteger("Plane Index");

                        public BSP_Surface()
                        {
                            base.Add(this.plane_index);
                            base.Add(this.first_edge);
                            base.Add(this.flags);
                            base.Add(this.breakablesurface_index);
                            base.Add(this.collision_material_index);
                        }
                    }

                    public class BSP3DNode : TagBlockLayout
                    {
                        public ShortInteger backchild = new ShortInteger("Back Child");
                        public ShortInteger frontchild = new ShortInteger("Front Child");
                        public static readonly coll.Region.Permutation.BSP.BSP3DNode Layout = new coll.Region.Permutation.BSP.BSP3DNode();
                        public ShortInteger plane_index = new ShortInteger("BSP_Plane Index");
                        public ByteInteger unknown1 = new ByteInteger("Unknown 1");
                        public ByteInteger unknown2 = new ByteInteger("Unknown 2");

                        public BSP3DNode()
                        {
                            base.Add(this.plane_index);
                            base.Add(this.backchild);
                            base.Add(this.unknown1);
                            base.Add(this.frontchild);
                            base.Add(this.unknown2);
                        }
                    }

                    public class Edge : TagBlockLayout
                    {
                        public ShortInteger EndVertex = new ShortInteger("End Vertex");
                        public ShortInteger ForwardEdge = new ShortInteger("Forward Edge");
                        public static readonly coll.Region.Permutation.BSP.Edge Layout = new coll.Region.Permutation.BSP.Edge();
                        public ShortInteger LeftSurface = new ShortInteger("Left Surface");
                        public ShortInteger ReverseEdge = new ShortInteger("Reverse Edge");
                        public ShortInteger RightSurface = new ShortInteger("Right Surface");
                        public ShortInteger StartVertex = new ShortInteger("Start Vertex");

                        public Edge()
                        {
                            base.Add(this.StartVertex);
                            base.Add(this.EndVertex);
                            base.Add(this.ForwardEdge);
                            base.Add(this.ReverseEdge);
                            base.Add(this.LeftSurface);
                            base.Add(this.RightSurface);
                        }
                    }

                    public class Leaf : TagBlockLayout
                    {
                        public ByteInteger bsp2drefcount = new ByteInteger("BSP_2D_Reference Count");
                        public ShortInteger bsp2drefindex = new ShortInteger("BSP_2D_Reference Start Index");
                        public core.TagInterface.Flags flags = new core.TagInterface.Flags("Flags", core.TagInterface.Flags.BitmaskTypeEnum.Byte);
                        public static readonly coll.Region.Permutation.BSP.Leaf Layout = new coll.Region.Permutation.BSP.Leaf();

                        public Leaf()
                        {
                            base.Add(this.flags);
                            base.Add(this.bsp2drefcount);
                            base.Add(this.bsp2drefindex);
                        }
                    }

                    public class Vertice : TagBlockLayout
                    {
                        public LongInteger firstedge = new LongInteger("First Edge");
                        public static readonly coll.Region.Permutation.BSP.Vertice Layout = new coll.Region.Permutation.BSP.Vertice();
                        public RealPoint3D vertice = new RealPoint3D("Vertice");

                        public Vertice()
                        {
                            base.Add(this.vertice);
                            base.Add(this.firstedge);
                        }
                    }
                }

                public class Unknown_Tag_Block : TagBlockLayout
                {
                    public TagIdent ident = new TagIdent("ident");
                    public static readonly coll.Region.Permutation.Unknown_Tag_Block Layout = new coll.Region.Permutation.Unknown_Tag_Block();
                    public TagBlock unk1 = new TagBlock("unknown", Unknown_Tag_Block_Child.Layout, Map.HaloVersionEnum.Halo2);
                    public Padding unknown = new Padding(0x30);
                    public Padding unknown2 = new Padding(0x30);
                    public Padding unknown3 = new Padding(4);

                    public Unknown_Tag_Block()
                    {
                        base.Add(this.unknown);
                        base.Add(this.ident);
                        base.Add(this.unknown2);
                        base.Add(this.unk1);
                        base.Add(this.unknown3);
                    }

                    public class Unknown_Tag_Block_Child : TagBlockLayout
                    {
                        public static readonly coll.Region.Permutation.Unknown_Tag_Block.Unknown_Tag_Block_Child Layout = new coll.Region.Permutation.Unknown_Tag_Block.Unknown_Tag_Block_Child();
                        public ByteInteger unk1 = new ByteInteger("unknown");

                        public Unknown_Tag_Block_Child()
                        {
                            base.Add(this.unk1);
                        }
                    }
                }
            }
        }
    }
}

