﻿namespace core.IO
{
    using System;
    using System.IO;

    public class EndianIO
    {
        private EndianType endiantype;
        private string filepath;
        public EndianReader In;
        private bool isfile;
        private bool isOpen;
        public EndianWriter Out;
        private Stream s;

        public EndianIO(MemoryStream memorystream, EndianType endianstyle)
        {
            this.isfile = false;
            this.isOpen = false;
            this.endiantype = endianstyle;
            this.s = memorystream;
        }

        public EndianIO(string filelocation, EndianType endianstyle)
        {
            this.isfile = false;
            this.isOpen = false;
            this.endiantype = endianstyle;
            this.filepath = filelocation;
            this.isfile = true;
        }

        public void Close()
        {
            if (this.isOpen && this.isfile)
            {
                this.In.Close();
                this.Out.Close();
                this.isOpen = false;
            }
        }

        public void Open()
        {
            if (!this.isOpen)
            {
                if (this.isfile)
                {
                    this.s = new FileStream(this.filepath, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
                }
                switch (this.endiantype)
                {
                    case EndianType.BigEndian:
                        this.In = new EndianReader(this.s, this.endiantype);
                        this.Out = new EndianWriter(this.s, this.endiantype);
                        break;

                    case EndianType.LittleEndian:
                        this.In = new EndianReader(this.s, this.endiantype);
                        this.Out = new EndianWriter(this.s, this.endiantype);
                        break;
                }
                this.isOpen = true;
            }
        }

        public class EndianReader : BinaryReader
        {
            private EndianType endianstyle;

            public EndianReader(Stream stream, EndianType endianstyle) : base(stream)
            {
                this.endianstyle = endianstyle;
            }

            public override double ReadDouble()
            {
                double num = base.ReadDouble();
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(num);
                    Array.Reverse(bytes);
                    num = BitConverter.ToDouble(bytes, 0);
                }
                return num;
            }

            public override short ReadInt16()
            {
                short num = base.ReadInt16();
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(num);
                    Array.Reverse(bytes);
                    num = BitConverter.ToInt16(bytes, 0);
                }
                return num;
            }

            public override int ReadInt32()
            {
                int num = base.ReadInt32();
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(num);
                    Array.Reverse(bytes);
                    num = BitConverter.ToInt32(bytes, 0);
                }
                return num;
            }

            public override long ReadInt64()
            {
                long num = base.ReadInt64();
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(num);
                    Array.Reverse(bytes);
                    num = BitConverter.ToInt64(bytes, 0);
                }
                return num;
            }

            public override float ReadSingle()
            {
                float num = base.ReadSingle();
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(num);
                    Array.Reverse(bytes);
                    num = BitConverter.ToSingle(bytes, 0);
                }
                return num;
            }

            public override ushort ReadUInt16()
            {
                ushort num = base.ReadUInt16();
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(num);
                    Array.Reverse(bytes);
                    num = BitConverter.ToUInt16(bytes, 0);
                }
                return num;
            }

            public override uint ReadUInt32()
            {
                uint num = base.ReadUInt32();
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(num);
                    Array.Reverse(bytes);
                    num = BitConverter.ToUInt32(bytes, 0);
                }
                return num;
            }

            public override ulong ReadUInt64()
            {
                ulong num = base.ReadUInt64();
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(num);
                    Array.Reverse(bytes);
                    num = BitConverter.ToUInt64(bytes, 0);
                }
                return num;
            }

            public void SeekTo(int offset)
            {
                this.BaseStream.Position = offset;
            }
        }

        public class EndianWriter : BinaryWriter
        {
            private EndianType endianstyle;

            public EndianWriter(Stream stream, EndianType endianstyle) : base(stream)
            {
                this.endianstyle = endianstyle;
            }

            public void SeekTo(int offset)
            {
                this.BaseStream.Position = offset;
            }

            public override void Write(double value)
            {
                double num = value;
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(value);
                    Array.Reverse(bytes);
                    num = BitConverter.ToDouble(bytes, 0);
                }
                base.Write(num);
            }

            public override void Write(short value)
            {
                short num = value;
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(value);
                    Array.Reverse(bytes);
                    num = BitConverter.ToInt16(bytes, 0);
                }
                base.Write(num);
            }

            public override void Write(int value)
            {
                int num = value;
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(value);
                    Array.Reverse(bytes);
                    num = BitConverter.ToInt32(bytes, 0);
                }
                base.Write(num);
            }

            public override void Write(long value)
            {
                long num = value;
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(value);
                    Array.Reverse(bytes);
                    num = BitConverter.ToInt64(bytes, 0);
                }
                base.Write(num);
            }

            public override void Write(float value)
            {
                float num = value;
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(value);
                    Array.Reverse(bytes);
                    num = BitConverter.ToSingle(bytes, 0);
                }
                base.Write(num);
            }

            public override void Write(ushort value)
            {
                ushort num = value;
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(value);
                    Array.Reverse(bytes);
                    num = BitConverter.ToUInt16(bytes, 0);
                }
                base.Write(num);
            }

            public override void Write(uint value)
            {
                uint num = value;
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(value);
                    Array.Reverse(bytes);
                    num = BitConverter.ToUInt32(bytes, 0);
                }
                base.Write(num);
            }

            public override void Write(ulong value)
            {
                ulong num = value;
                if (this.endianstyle == EndianType.BigEndian)
                {
                    byte[] bytes = BitConverter.GetBytes(value);
                    Array.Reverse(bytes);
                    num = BitConverter.ToUInt64(bytes, 0);
                }
                base.Write(num);
            }
        }
    }
}

