﻿namespace core
{
    using core.H2;
    using core.IO;
    using core.TagInterface;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Windows.Forms;

    public class Map : CloneableObject
    {
        public BitmapLibraryLayout BitmapLibary;
        private BSPContainer BSP;
        private bool error;
        private FileNamesInfo fileNames;
        private string filePath;
        private HaloVersionEnum haloversion;
        private Index_HeaderInfo indexHeader;
        public EndianIO io;
        public bool isOpen;
        private Map_HeaderInfo mapHeader;
        private core.Meta matg_meta;
        private ObjectIndexInfo metaInfo;
        public MapTypes openMapType;
        internal int primarymagic;
        internal int secondarymagic;
        public StringsInfo strings;
        private core.Meta ugh_meta;

        public Map()
        {
            this.isOpen = false;
            this.error = false;
        }

        public Map(string path)
        {
            this.isOpen = false;
            this.error = false;
            this.filePath = path;
            this.io = new EndianIO(this.FilePath, EndianType.LittleEndian);
            this.io.Open();
            this.io.In.SeekTo(4);
            int num2 = this.io.In.ReadInt32();
            if (num2 == 5)
            {
                this.haloversion = HaloVersionEnum.Halo1;
            }
            else if (num2 != 8)
            {
                if (num2 != 0x261)
                {
                    MessageBox.Show("This is not a supported map type.");
                    this.error = true;
                    return;
                }
                this.haloversion = HaloVersionEnum.HaloCE;
            }
            else
            {
                this.haloversion = HaloVersionEnum.Halo2;
            }
            this.LoadMap();
            this.io.Close();
        }

        public int FindByNameAndTagType(string type, string name)
        {
            for (int i = 0; i < this.Meta.Count; i++)
            {
                if ((type == this.Meta[i].TagType) && (name == this.FileName[i].Name))
                {
                    return i;
                }
            }
            return -1;
        }

        public int FindMetaByID(int id)
        {
            object obj2 = this.Meta.identHT[id];
            if (obj2 == null)
            {
                return -1;
            }
            return (int) obj2;
        }

        public int FindMetaByOffset(int offset)
        {
            for (int i = 0; i < this.Meta.Count; i++)
            {
                if (((this.HaloVersion != HaloVersionEnum.HaloCE) || (this.Meta[i].External_Bitmap_Index == -1)) && ((offset >= this.Meta[i].Offset) && (offset < (this.Meta[i].Offset + this.Meta[i].Size))))
                {
                    return i;
                }
            }
            return -1;
        }

        private void LoadMap()
        {
            switch (this.HaloVersion)
            {
                case HaloVersionEnum.Halo2:
                {
                    this.mapHeader = new Map_HeaderInfo(ref this.io.In, this.haloversion);
                    this.indexHeader = new Index_HeaderInfo(this);
                    this.metaInfo = new ObjectIndexInfo(this);
                    this.fileNames = new FileNamesInfo(this);
                    this.strings = new StringsInfo(ref this.io.In, this);
                    this.BSP = new BSPContainer(this);
                    Map map = this;
                    this.matg_meta = new core.Meta(0, ref map);
                    matg.Sound_Layout chunk = (matg.Sound_Layout) this.globals_Interface.Sound.GetChunk(0);
                    this.ugh_meta = new core.Meta(chunk.Resource_Hack.TagNumber, ref map);
                    break;
                }
                case HaloVersionEnum.HaloCE:
                    this.BitmapLibary = new BitmapLibraryLayout(this);
                    this.mapHeader = new Map_HeaderInfo(ref this.io.In, this.HaloVersion);
                    this.indexHeader = new Index_HeaderInfo(this);
                    this.metaInfo = new ObjectIndexInfo(this);
                    this.fileNames = new FileNamesInfo(this);
                    this.BSP = new BSPContainer(this);
                    break;

                case HaloVersionEnum.Halo1:
                    this.mapHeader = new Map_HeaderInfo(ref this.io.In, this.HaloVersion);
                    this.indexHeader = new Index_HeaderInfo(this);
                    this.metaInfo = new ObjectIndexInfo(this);
                    this.fileNames = new FileNamesInfo(this);
                    this.BSP = new BSPContainer(this);
                    break;
            }
        }

        public void Sign()
        {
            if (this.HaloVersion != HaloVersionEnum.Halo2)
            {
                MessageBox.Show("Only Sign Halo 2 Maps");
            }
            else
            {
                this.io.Open();
                int num = ((int) this.io.In.BaseStream.Length) - 0x800;
                int num2 = num / 4;
                int num3 = 0;
                this.io.In.SeekTo(0x800);
                for (int i = 0; i < num2; i++)
                {
                    num3 ^= this.io.In.ReadInt32();
                }
                this.io.In.SeekTo(720);
                this.io.Out.Write(num3);
                this.mapHeader.signature = num3;
                this.io.Close();
            }
        }

        public bool ErrorLoadingMap
        {
            get
            {
                return this.error;
            }
        }

        public FileNamesInfo FileName
        {
            get
            {
                return this.fileNames;
            }
        }

        public string FilePath
        {
            get
            {
                return this.filePath;
            }
        }

        public matg globals_Interface
        {
            get
            {
                return (matg) this.matg_meta.Interface.Header.GetChunk(0);
            }
        }

        public HaloVersionEnum HaloVersion
        {
            get
            {
                return this.haloversion;
            }
        }

        public Index_HeaderInfo Index_Header
        {
            get
            {
                return this.indexHeader;
            }
        }

        public Map_HeaderInfo Map_Header
        {
            get
            {
                return this.mapHeader;
            }
        }

        public core.Meta matg_Meta
        {
            get
            {
                return this.matg_meta;
            }
            set
            {
                this.matg_meta = value;
            }
        }

        public ObjectIndexInfo Meta
        {
            get
            {
                return this.metaInfo;
            }
        }

        public int PrimaryMagic
        {
            get
            {
                return this.primarymagic;
            }
        }

        public BSPContainer SBSP
        {
            get
            {
                return this.BSP;
            }
        }

        public int SecondaryMagic
        {
            get
            {
                return this.secondarymagic;
            }
        }

        public StringsInfo StringIDs
        {
            get
            {
                return this.strings;
            }
        }

        public ugh_ ugh_Interface
        {
            get
            {
                return (ugh_) this.ugh_meta.Interface.Header.GetChunk(0);
            }
        }

        public core.Meta ugh_Meta
        {
            get
            {
                return this.ugh_meta;
            }
            set
            {
                this.ugh_meta = value;
            }
        }

        public class BitmapLibraryLayout : CloneableObject
        {
            public int BitmapCount;
            public int BitmapIndexOffset;
            public bool error;
            public int FileNameStringsOffset;
            private string filepath;
            public int FileSize;
            public EndianIO io;
            public int[] MetaOffset;
            public int[] MetaSize;
            public string[] Name;
            public int[] RawOffset;
            public int[] RawSize;
            public int[] StringLength;
            public int[] StringOffset;

            public BitmapLibraryLayout()
            {
                this.error = false;
            }

            public BitmapLibraryLayout(Map map)
            {
                this.error = false;
                if (map.HaloVersion == Map.HaloVersionEnum.HaloCE)
                {
                    this.filepath = map.filePath.Remove(map.filePath.LastIndexOf('\\'));
                    this.filepath = this.filepath + @"\bitmaps.map";
                    this.io = new EndianIO(this.filepath, EndianType.LittleEndian);
                    this.io.Open();
                    this.HaloCEBitmapLibraryLayout(ref this.io.In);
                    this.io.Close();
                }
            }

            public void HaloCEBitmapLibraryLayout(ref EndianIO.EndianReader BR)
            {
                int num;
                this.FileSize = (int) BR.BaseStream.Length;
                BR.BaseStream.Position = 4L;
                this.FileNameStringsOffset = BR.ReadInt32();
                this.BitmapIndexOffset = BR.ReadInt32();
                this.BitmapCount = BR.ReadInt32();
                this.StringLength = new int[this.BitmapCount];
                this.StringOffset = new int[this.BitmapCount];
                this.MetaOffset = new int[this.BitmapCount];
                this.MetaSize = new int[this.BitmapCount];
                this.RawOffset = new int[this.BitmapCount];
                this.RawSize = new int[this.BitmapCount];
                BR.BaseStream.Position = this.BitmapIndexOffset;
                for (num = 0; num < this.BitmapCount; num++)
                {
                    this.StringOffset[num] = BR.ReadInt32();
                    this.RawSize[num] = BR.ReadInt32();
                    this.RawOffset[num] = BR.ReadInt32();
                    if (num != 0)
                    {
                        this.StringLength[num - 1] = (this.StringOffset[num] - this.StringOffset[num - 1]) - 1;
                        if (num == (this.BitmapCount - 1))
                        {
                            this.StringLength[num] = (this.BitmapIndexOffset - this.StringOffset[num]) - 1;
                        }
                    }
                }
                this.Name = new string[this.BitmapCount];
                for (num = 0; num < this.BitmapCount; num++)
                {
                    BR.BaseStream.Position = this.FileNameStringsOffset + this.StringOffset[num];
                    this.Name[num] = new string(BR.ReadChars(this.StringLength[num]));
                }
            }
        }

        public class BSPContainer
        {
            public int bspcount;
            public Map.BSPInfo[] sbsp;

            public BSPContainer(Map map)
            {
                switch (map.HaloVersion)
                {
                    case Map.HaloVersionEnum.Halo2:
                        this.Halo2BSPContainer(map);
                        break;

                    case Map.HaloVersionEnum.HaloCE:
                        this.HaloCEContainer(map);
                        break;

                    case Map.HaloVersionEnum.Halo1:
                        this.Halo1Container(map);
                        break;
                }
            }

            public int FindBSPNumberByBSPIdent(int id)
            {
                for (int i = 0; i < this.sbsp.Length; i++)
                {
                    if (this.sbsp[i].ident == id)
                    {
                        return i;
                    }
                }
                return -1;
            }

            public int FindBSPNumberByLightMapIdent(int id)
            {
                for (int i = 0; i < this.sbsp.Length; i++)
                {
                    if (this.sbsp[i].lightmapident == id)
                    {
                        return i;
                    }
                }
                return -1;
            }

            private void Halo1Container(Map map)
            {
                map.io.In.SeekTo(map.Meta[0].Offset + 0x5a4);
                this.bspcount = map.io.In.ReadInt32();
                int num = map.io.In.ReadInt32() - map.PrimaryMagic;
                this.sbsp = new Map.BSPInfo[this.bspcount];
                for (int i = 0; i < this.bspcount; i++)
                {
                    this.sbsp[i] = new Map.BSPInfo();
                    this.sbsp[i].pointerOffset = (num + (i * 0x20)) - map.Meta[0].Offset;
                    map.io.In.SeekTo(num + (i * 0x20));
                    this.sbsp[i].offset = map.io.In.ReadInt32();
                    this.sbsp[i].size = map.io.In.ReadInt32();
                    this.sbsp[i].magic = map.io.In.ReadInt32() - this.sbsp[i].offset;
                    map.io.In.SeekTo((num + (i * 0x20)) + 0x1c);
                    this.sbsp[i].ident = map.io.In.ReadInt32();
                    this.sbsp[i].tagnumber = map.FindMetaByID(this.sbsp[i].ident);
                    map.Meta[this.sbsp[i].tagnumber].offset = this.sbsp[i].offset;
                    map.Meta[this.sbsp[i].tagnumber].size = this.sbsp[i].size;
                    map.io.In.SeekTo(this.sbsp[i].offset + 4);
                    this.sbsp[i].Halo1VerticeCount = map.io.In.ReadInt32();
                    map.io.In.SeekTo(((this.sbsp[i].offset + 0x18) + (this.sbsp[i].Halo1VerticeCount * 12)) + 4);
                    int num3 = map.io.In.ReadInt32() - this.sbsp[i].magic;
                    this.sbsp[i].lightmapoffset = num3 - this.sbsp[i].offset;
                    map.io.In.BaseStream.Position = (this.sbsp[i].offset + this.sbsp[i].lightmapoffset) + 12;
                    this.sbsp[i].lightmapident = map.io.In.ReadInt32();
                    this.sbsp[i].lightmaptagnumber = map.FindMetaByID(this.sbsp[i].lightmapident);
                }
            }

            private void Halo2BSPContainer(Map map)
            {
                map.io.In.BaseStream.Position = map.Meta[3].Offset + 0x210;
                this.bspcount = map.io.In.ReadInt32();
                int num = map.io.In.ReadInt32() - map.SecondaryMagic;
                this.sbsp = new Map.BSPInfo[this.bspcount];
                for (int i = 0; i < this.bspcount; i++)
                {
                    this.sbsp[i] = new Map.BSPInfo();
                    this.sbsp[i].pointerOffset = (num + (i * 0x44)) - map.Meta[3].Offset;
                    map.io.In.SeekTo(num + (i * 0x44));
                    this.sbsp[i].offset = map.io.In.ReadInt32();
                    Functions.ParsePointer(ref this.sbsp[i].offset, ref this.sbsp[i].location);
                    this.sbsp[i].size = map.io.In.ReadInt32();
                    this.sbsp[i].magic = map.io.In.ReadInt32() - this.sbsp[i].offset;
                    map.io.In.SeekTo((num + (i * 0x44)) + 20);
                    this.sbsp[i].ident = map.io.In.ReadInt32();
                    this.sbsp[i].tagnumber = map.FindMetaByID(this.sbsp[i].ident);
                    map.Meta[this.sbsp[i].tagnumber].offset = this.sbsp[i].offset;
                    map.Meta[this.sbsp[i].tagnumber].size = this.sbsp[i].size;
                    map.io.In.SeekTo((num + (i * 0x44)) + 0x1c);
                    this.sbsp[i].lightmapident = map.io.In.ReadInt32();
                    this.sbsp[i].lightmaptagnumber = map.FindMetaByID(this.sbsp[i].lightmapident);
                    if (this.sbsp[i].lightmaptagnumber != -1)
                    {
                        map.io.In.BaseStream.Position = this.sbsp[i].offset + 8;
                        this.sbsp[i].lightmapoffset = map.io.In.ReadInt32();
                        if (this.sbsp[i].lightmapoffset == 0)
                        {
                            this.sbsp[i].lightmapident = -1;
                            this.sbsp[i].lightmaptagnumber = -1;
                            if (MessageBox.Show("There is no lightmap for this bsp and the scenario is currently linked to a broken ID.\n Would you like Entity to fix it?", "Error", MessageBoxButtons.YesNo) == DialogResult.Yes)
                            {
                                map.io.Out.BaseStream.Position = (num + (i * 0x44)) + 0x1c;
                                map.io.Out.Write(int.Parse("FFFFFFFF", NumberStyles.HexNumber));
                            }
                        }
                        else
                        {
                            Map.BSPInfo info1 = this.sbsp[i];
                            info1.lightmapoffset += -this.sbsp[i].magic;
                            this.sbsp[i].lightmapsize = (this.sbsp[i].size + this.sbsp[i].offset) - this.sbsp[i].lightmapoffset;
                            map.Meta[this.sbsp[i].lightmaptagnumber].offset = this.sbsp[i].lightmapoffset;
                            map.Meta[this.sbsp[i].lightmaptagnumber].size = this.sbsp[i].lightmapsize;
                            map.io.In.BaseStream.Position = this.sbsp[i].lightmapoffset + 0x80;
                            int num3 = map.io.In.ReadInt32();
                            int num4 = map.io.In.ReadInt32() - this.sbsp[i].magic;
                            map.io.In.BaseStream.Position = num4 + 0x1c;
                            this.sbsp[i].LightMap_TagNumber = map.FindMetaByID(map.io.In.ReadInt32());
                            map.io.In.BaseStream.Position = num4 + 8;
                            int num5 = map.io.In.ReadInt32();
                            int num6 = map.io.In.ReadInt32() - this.sbsp[i].magic;
                            this.sbsp[i].palettesoffset = num6;
                            int index = 0;
                            while (index < num5)
                            {
                                map.io.In.BaseStream.Position = num6 + (index * 0x400);
                                Palette_Color[] item = new Palette_Color[0x100];
                                for (int j = 0; j < 0x100; j++)
                                {
                                    item[j] = new Palette_Color();
                                    item[j].r = map.io.In.ReadByte();
                                    item[j].g = map.io.In.ReadByte();
                                    item[j].b = map.io.In.ReadByte();
                                    item[j].a = map.io.In.ReadByte();
                                }
                                this.sbsp[i].LightMap_Palettes.Add(item);
                                index++;
                            }
                            map.io.In.BaseStream.Position = num4 + 40;
                            num5 = map.io.In.ReadInt32();
                            num6 = map.io.In.ReadInt32() - this.sbsp[i].magic;
                            this.sbsp[i].VisualChunk_Bitmap_Index = new int[num5];
                            this.sbsp[i].VisualChunk_LightMap_Index = new int[num5];
                            if (num5 != 0)
                            {
                                map.io.In.BaseStream.Position = num6;
                                index = 0;
                                while (index < num5)
                                {
                                    this.sbsp[i].VisualChunk_Bitmap_Index[index] = map.io.In.ReadInt16();
                                    this.sbsp[i].VisualChunk_LightMap_Index[index] = map.io.In.ReadInt16();
                                    index++;
                                }
                            }
                            map.io.In.BaseStream.Position = num4 + 0x48;
                            num5 = map.io.In.ReadInt32();
                            num6 = map.io.In.ReadInt32() - this.sbsp[i].magic;
                            this.sbsp[i].SceneryChunk_Bitmap_Index = new int[num5];
                            this.sbsp[i].SceneryChunk_LightMap_Index = new int[num5];
                            if (num5 > 0)
                            {
                                map.io.In.BaseStream.Position = num6;
                                for (index = 0; index < num5; index++)
                                {
                                    this.sbsp[i].SceneryChunk_Bitmap_Index[index] = map.io.In.ReadInt16();
                                    this.sbsp[i].SceneryChunk_LightMap_Index[index] = map.io.In.ReadInt16();
                                }
                            }
                        }
                    }
                }
            }

            private void HaloCEContainer(Map map)
            {
                map.io.In.SeekTo(map.Meta[0].Offset + 0x5a4);
                this.bspcount = map.io.In.ReadInt32();
                int num = map.io.In.ReadInt32() - map.PrimaryMagic;
                this.sbsp = new Map.BSPInfo[this.bspcount];
                for (int i = 0; i < this.bspcount; i++)
                {
                    this.sbsp[i] = new Map.BSPInfo();
                    this.sbsp[i].pointerOffset = (num + (i * 0x20)) - map.Meta[0].Offset;
                    map.io.In.SeekTo(num + (i * 0x20));
                    this.sbsp[i].offset = map.io.In.ReadInt32();
                    this.sbsp[i].size = map.io.In.ReadInt32();
                    this.sbsp[i].magic = map.io.In.ReadInt32() - this.sbsp[i].offset;
                    map.io.In.SeekTo((num + (i * 0x20)) + 0x1c);
                    this.sbsp[i].ident = map.io.In.ReadInt32();
                    this.sbsp[i].tagnumber = map.FindMetaByID(this.sbsp[i].ident);
                    map.Meta[this.sbsp[i].tagnumber].offset = this.sbsp[i].offset;
                    map.Meta[this.sbsp[i].tagnumber].size = this.sbsp[i].size;
                    map.io.In.BaseStream.Position = this.sbsp[i].offset + 4;
                    this.sbsp[i].Halo1VerticeCount = map.io.In.ReadInt32();
                    this.sbsp[i].lightmapoffset = 0x18;
                    map.io.In.BaseStream.Position = this.sbsp[i].lightmapoffset + 12;
                    this.sbsp[i].lightmapident = map.io.In.ReadInt32();
                    this.sbsp[i].lightmaptagnumber = map.FindMetaByID(this.sbsp[i].ident);
                }
            }

            public class Palette_Color
            {
                public int a;
                public int b;
                public int g;
                public int r;
            }
        }

        public class BSPInfo
        {
            public int Halo1VerticeCount;
            public int ident;
            public List<Map.BSPContainer.Palette_Color[]> LightMap_Palettes = new List<Map.BSPContainer.Palette_Color[]>();
            public int LightMap_TagNumber = -1;
            public int lightmapident;
            public int lightmapoffset;
            public int lightmapsize;
            public int lightmaptagnumber;
            public MapTypes location;
            public int magic;
            public int offset;
            public int palettesoffset;
            public int pointerOffset;
            public int[] SceneryChunk_Bitmap_Index;
            public int[] SceneryChunk_LightMap_Index;
            public int size;
            public int tagnumber;
            public int[] VisualChunk_Bitmap_Index;
            public int[] VisualChunk_LightMap_Index;

            public void WritePalettes(Map map)
            {
                map.io.Out.BaseStream.Position = this.palettesoffset;
                foreach (Map.BSPContainer.Palette_Color[] colorArray in this.LightMap_Palettes)
                {
                    for (int i = 0; i < 0x100; i++)
                    {
                        map.io.Out.Write((byte) colorArray[i].r);
                        map.io.Out.Write((byte) colorArray[i].g);
                        map.io.Out.Write((byte) colorArray[i].b);
                        map.io.Out.Write((byte) colorArray[i].a);
                    }
                }
            }
        }

        public class FileNameContainer : CloneableObject
        {
            private int length;
            private string name;
            private int offset;

            public FileNameContainer()
            {
            }

            public FileNameContainer(int offset, string name, int length)
            {
                this.offset = offset;
                this.name = name;
                this.length = length;
            }

            public int Length
            {
                get
                {
                    return this.length;
                }
            }

            public string Name
            {
                get
                {
                    return this.name;
                }
            }

            public int Offset
            {
                get
                {
                    return this.offset;
                }
            }
        }

        public class FileNamesInfo : List<Map.FileNameContainer>
        {
            public FileNamesInfo()
            {
            }

            public FileNamesInfo(Map map)
            {
                switch (map.HaloVersion)
                {
                    case Map.HaloVersionEnum.Halo2:
                        this.Halo2FileNamesInfo(map);
                        break;

                    case Map.HaloVersionEnum.HaloCE:
                    case Map.HaloVersionEnum.Halo1:
                        this.HaloCEFileNamesInfo(map);
                        break;
                }
            }

            private void Halo2FileNamesInfo(Map map)
            {
                int num;
                map.io.In.BaseStream.Position = map.Map_Header.FileIndexOffset;
                for (num = 0; num < map.Map_Header.FileNamesCount; num++)
                {
                    map.Meta[num] = new Map.MetaInfo(map.Meta[num].TagType, map.Meta[num].Offset, map.Meta[num].Size, map.Meta[num].Ident, map.mapHeader.FileNamesOffset + map.io.In.ReadInt32(), -1);
                }
                for (num = 0; num < map.Index_Header.Meta_Count; num++)
                {
                    int length;
                    string str;
                    if (num < map.Map_Header.FileNamesCount)
                    {
                        if (num != (map.Index_Header.Meta_Count - 1))
                        {
                            length = map.Meta[num + 1].String_Offset - map.Meta[num].String_Offset;
                        }
                        else
                        {
                            length = (map.mapHeader.offsetTofileNames + map.mapHeader.fileNamesSize) - map.Meta[num].String_Offset;
                        }
                        length--;
                        map.io.In.BaseStream.Position = map.Meta[num].String_Offset;
                        str = core.TagInterface.String.ReadString(ref map.io.In, length);
                        base.Add(new Map.FileNameContainer(map.Meta[num].String_Offset, str, length));
                    }
                    else
                    {
                        str = "Unknown Meta (" + ((num - map.Map_Header.FileNamesCount)).ToString() + ")";
                        length = str.Length;
                        base.Add(new Map.FileNameContainer(-1, str, length));
                    }
                }
            }

            private void HaloCEFileNamesInfo(Map map)
            {
                if (map.HaloVersion == Map.HaloVersionEnum.Halo1)
                {
                    map.mapHeader.offsetTofileNames = (map.Map_Header.IndexOffset + 0x24) + (map.Index_Header.Meta_Count * 0x20);
                    map.mapHeader.fileNamesSize = map.Index_Header.Model_Raw_Data_Offset - map.mapHeader.offsetTofileNames;
                }
                else
                {
                    map.mapHeader.offsetTofileNames = (map.Map_Header.IndexOffset + 40) + (map.Index_Header.Meta_Count * 0x20);
                    map.mapHeader.fileNamesSize = map.Meta[0].Offset - map.mapHeader.offsetTofileNames;
                }
                for (int i = 0; i < map.Index_Header.Meta_Count; i++)
                {
                    int num2;
                    if (i != (map.Index_Header.Meta_Count - 1))
                    {
                        num2 = map.Meta[i + 1].String_Offset - map.Meta[i].String_Offset;
                    }
                    else
                    {
                        num2 = (map.mapHeader.offsetTofileNames + map.mapHeader.fileNamesSize) - map.Meta[i].String_Offset;
                    }
                    num2--;
                    map.io.In.BaseStream.Position = map.Meta[i].String_Offset;
                    string name = core.TagInterface.String.ReadString(ref map.io.In, num2);
                    base.Add(new Map.FileNameContainer(map.Meta[i].String_Offset, name, num2));
                }
            }

            public string[] ToStringArray()
            {
                string[] strArray = new string[base.Count];
                for (int i = 0; i < base.Count; i++)
                {
                    strArray[i] = base[i].Name;
                }
                return strArray;
            }
        }

        public enum HaloVersionEnum
        {
            Halo2,
            HaloCE,
            Halo1
        }

        public class Index_HeaderInfo : CloneableObject
        {
            private int constant;
            internal int indicesObjectCount;
            private int metaCount;
            internal int modelIndicesOffset;
            internal int modelRawDataOffset;
            internal int modelRawDataSize;
            private int tagsOffset;
            private int tagTypeCount;
            internal int verticeObjectCount;

            public Index_HeaderInfo()
            {
            }

            public Index_HeaderInfo(Map map)
            {
                switch (map.HaloVersion)
                {
                    case Map.HaloVersionEnum.Halo2:
                        this.LoadHalo2Index_HeaderInfo(map);
                        break;

                    case Map.HaloVersionEnum.HaloCE:
                        this.LoadHaloCEIndex_HeaderInfo(map);
                        break;

                    case Map.HaloVersionEnum.Halo1:
                        this.LoadHalo1Index_HeaderInfo(map);
                        break;
                }
            }

            private void LoadHalo1Index_HeaderInfo(Map map)
            {
                map.io.In.BaseStream.Position = map.Map_Header.indexOffset;
                this.constant = map.io.In.ReadInt32();
                map.primarymagic = this.constant - (map.Map_Header.indexOffset + 0x24);
                map.io.In.ReadInt32();
                map.io.In.ReadInt32();
                this.metaCount = map.io.In.ReadInt32();
                this.verticeObjectCount = map.io.In.ReadInt32();
                this.modelRawDataOffset = map.io.In.ReadInt32() - map.PrimaryMagic;
                this.indicesObjectCount = map.io.In.ReadInt32();
                this.modelIndicesOffset = map.io.In.ReadInt32() - map.PrimaryMagic;
                this.tagsOffset = map.Map_Header.indexOffset + 0x24;
            }

            private void LoadHalo2Index_HeaderInfo(Map map)
            {
                map.io.In.BaseStream.Position = map.Map_Header.IndexOffset;
                this.constant = map.io.In.ReadInt32();
                map.primarymagic = this.constant - (map.Map_Header.IndexOffset + 0x20);
                this.tagTypeCount = map.io.In.ReadInt32();
                this.tagsOffset = map.io.In.ReadInt32() - map.PrimaryMagic;
                map.io.In.BaseStream.Position = this.tagsOffset + 8;
                map.secondarymagic = map.io.In.ReadInt32() - (map.Map_Header.IndexOffset + map.Map_Header.Index_HeaderSize);
                int num = (map.Map_Header.CombinedMetaSize - map.Map_Header.MetaSize) - map.Map_Header.Index_HeaderSize;
                map.secondarymagic = map.primarymagic + num;
                map.io.In.BaseStream.Position = map.Map_Header.IndexOffset + 0x18;
                this.metaCount = map.io.In.ReadInt32();
            }

            private void LoadHaloCEIndex_HeaderInfo(Map map)
            {
                map.io.In.BaseStream.Position = map.Map_Header.IndexOffset;
                this.constant = map.io.In.ReadInt32();
                map.primarymagic = this.constant - (map.Map_Header.IndexOffset + 40);
                map.io.In.ReadInt32();
                map.io.In.ReadInt32();
                this.metaCount = map.io.In.ReadInt32();
                this.verticeObjectCount = map.io.In.ReadInt32();
                this.modelRawDataOffset = map.io.In.ReadInt32();
                this.indicesObjectCount = map.io.In.ReadInt32();
                this.modelIndicesOffset = map.io.In.ReadInt32();
                this.modelRawDataSize = map.io.In.ReadInt32();
                this.tagsOffset = map.Map_Header.IndexOffset + 40;
            }

            public int Constant
            {
                get
                {
                    return this.constant;
                }
            }

            public int Indices_Object_Count
            {
                get
                {
                    return this.indicesObjectCount;
                }
            }

            public int Indices_Offset
            {
                get
                {
                    return this.modelIndicesOffset;
                }
            }

            public int Meta_Count
            {
                get
                {
                    return this.metaCount;
                }
            }

            public int Model_Raw_Data_Offset
            {
                get
                {
                    return this.modelRawDataOffset;
                }
            }

            public int Model_Raw_Data_Size
            {
                get
                {
                    return this.modelRawDataSize;
                }
            }

            public int Tag_Type_Count
            {
                get
                {
                    return this.tagTypeCount;
                }
            }

            public int Tags_Offset
            {
                get
                {
                    return this.tagsOffset;
                }
            }

            public int Vertice_Object_Count
            {
                get
                {
                    return this.verticeObjectCount;
                }
            }
        }

        public class Map_HeaderInfo : CloneableObject
        {
            internal int combinedSize;
            internal int fileCount;
            internal int fileNamesSize;
            internal int fileSize;
            internal int indexOffset;
            internal string mapName;
            internal MapTypes mapType;
            internal int metaSize;
            internal int metaStart;
            internal int offsetToCrazy;
            internal int offsetTofileIndex;
            internal int offsetTofileNames;
            internal int offsetToStringIndex;
            internal int offsetToStringNames1;
            internal int offsetToStringNames2;
            internal string scenarioPath;
            internal int scriptReferenceCount;
            internal int signature;
            internal int sizeOfCrazy;
            internal int sizeOfScriptReference;

            public Map_HeaderInfo()
            {
            }

            public Map_HeaderInfo(ref EndianIO.EndianReader BR, Map.HaloVersionEnum haloversion)
            {
                switch (haloversion)
                {
                    case Map.HaloVersionEnum.Halo2:
                        this.LoadHalo2Map_HeaderInfo(ref BR);
                        break;

                    case Map.HaloVersionEnum.HaloCE:
                    case Map.HaloVersionEnum.Halo1:
                        this.LoadHalo1Map_HeaderInfo(ref BR);
                        break;
                }
            }

            private void LoadHalo1Map_HeaderInfo(ref EndianIO.EndianReader BR)
            {
                BR.BaseStream.Position = 8L;
                this.fileSize = BR.ReadInt32();
                BR.BaseStream.Position = 0x10L;
                this.indexOffset = BR.ReadInt32();
                this.metaSize = BR.ReadInt32();
                BR.BaseStream.Position = 0x20L;
                this.mapName = core.TagInterface.String.ReadString(ref BR, 0x20);
            }

            private void LoadHalo2Map_HeaderInfo(ref EndianIO.EndianReader BR)
            {
                BR.BaseStream.Position = 8L;
                this.fileSize = BR.ReadInt32();
                BR.BaseStream.Position = 0x10L;
                this.indexOffset = BR.ReadInt32();
                this.metaStart = BR.ReadInt32();
                this.metaSize = BR.ReadInt32();
                this.combinedSize = BR.ReadInt32();
                BR.BaseStream.Position = 0x158L;
                this.offsetToCrazy = BR.ReadInt32();
                this.sizeOfCrazy = BR.ReadInt32();
                BR.BaseStream.Position = 0x160L;
                this.offsetToStringNames1 = BR.ReadInt32();
                this.scriptReferenceCount = BR.ReadInt32();
                this.sizeOfScriptReference = BR.ReadInt32();
                this.offsetToStringIndex = BR.ReadInt32();
                this.offsetToStringNames2 = BR.ReadInt32();
                BR.BaseStream.Position = 0x198L;
                this.mapName = core.TagInterface.String.ReadString(ref BR, 0x24);
                BR.BaseStream.Position = 0x1bcL;
                this.scenarioPath = core.TagInterface.String.ReadString(ref BR, 0x40);
                this.mapType = MapTypes.Internal;
                if (this.scenarioPath.IndexOf(@"scenarios\ui\mainmenu\mainmenu") != -1)
                {
                    this.mapType = MapTypes.MainMenu;
                }
                if (this.scenarioPath.IndexOf(@"scenarios\shared\shared") != -1)
                {
                    this.mapType = MapTypes.MPShared;
                }
                if (this.scenarioPath.IndexOf(@"scenarios\shared\single_player_shared") != -1)
                {
                    this.mapType = MapTypes.SPShared;
                }
                BR.BaseStream.Position = 0x2c0L;
                this.fileCount = BR.ReadInt32();
                this.offsetTofileNames = BR.ReadInt32();
                this.fileNamesSize = BR.ReadInt32();
                this.offsetTofileIndex = BR.ReadInt32();
                this.signature = BR.ReadInt32();
            }

            public int CombinedMetaSize
            {
                get
                {
                    return this.combinedSize;
                }
            }

            public int FileIndexOffset
            {
                get
                {
                    return this.offsetTofileIndex;
                }
            }

            public int FileNamesCount
            {
                get
                {
                    return this.fileCount;
                }
            }

            public int FileNamesOffset
            {
                get
                {
                    return this.offsetTofileNames;
                }
            }

            public int FileNamesSize
            {
                get
                {
                    return this.fileNamesSize;
                }
            }

            public int GeometryCacheOffset
            {
                get
                {
                    return this.offsetToCrazy;
                }
            }

            public int GeometryCacheSize
            {
                get
                {
                    return this.sizeOfCrazy;
                }
            }

            public int Index_HeaderSize
            {
                get
                {
                    return this.metaStart;
                }
            }

            public int IndexOffset
            {
                get
                {
                    return this.indexOffset;
                }
            }

            public string MapName
            {
                get
                {
                    return this.mapName;
                }
            }

            public int MapSize
            {
                get
                {
                    return this.fileSize;
                }
            }

            public MapTypes MapType
            {
                get
                {
                    return this.mapType;
                }
            }

            public int MetaSize
            {
                get
                {
                    return this.metaSize;
                }
            }

            public string ScenarioPath
            {
                get
                {
                    return this.scenarioPath;
                }
            }

            public int StringCount
            {
                get
                {
                    return this.scriptReferenceCount;
                }
            }

            public int StringIndexOffset
            {
                get
                {
                    return this.offsetToStringIndex;
                }
            }

            public int StringNames1Offset
            {
                get
                {
                    return this.offsetToStringNames1;
                }
            }

            public int StringNames1Size
            {
                get
                {
                    return (this.StringCount * 0x80);
                }
            }

            public int StringNames2Offset
            {
                get
                {
                    return this.offsetToStringNames2;
                }
            }

            public int StringNames2Size
            {
                get
                {
                    return this.sizeOfScriptReference;
                }
            }
        }

        public class MetaInfo : CloneableObject
        {
            internal int bitmap_index;
            internal int ident;
            internal int offset;
            internal int size;
            internal int stringoffset;
            internal string tagType;

            public MetaInfo()
            {
                this.bitmap_index = -1;
            }

            public MetaInfo(string tagtype, int offset, int size, ref Map.ObjectIndexInfo objectarray)
            {
                this.bitmap_index = -1;
                this.tagType = tagtype;
                this.size = size;
                this.offset = offset;
                this.ident = objectarray[objectarray.Count - 1].Ident + 0x10001;
            }

            public MetaInfo(string tagtype, int offset, int size, int ident)
            {
                this.bitmap_index = -1;
                this.tagType = tagtype;
                this.size = size;
                this.offset = offset;
                this.ident = ident;
            }

            public MetaInfo(string tagtype, int offset, int size, int ident, int stringoffset, int bitmapindex)
            {
                this.bitmap_index = -1;
                this.tagType = tagtype;
                this.size = size;
                this.offset = offset;
                this.ident = ident;
                this.stringoffset = stringoffset;
                this.bitmap_index = bitmapindex;
            }

            public int External_Bitmap_Index
            {
                get
                {
                    return this.bitmap_index;
                }
            }

            public int Ident
            {
                get
                {
                    return this.ident;
                }
            }

            public bool IsExternal
            {
                get
                {
                    return (this.bitmap_index != -1);
                }
            }

            public int Offset
            {
                get
                {
                    return this.offset;
                }
            }

            public int Size
            {
                get
                {
                    return this.size;
                }
            }

            public int String_Offset
            {
                get
                {
                    return this.stringoffset;
                }
            }

            public string TagType
            {
                get
                {
                    return this.tagType;
                }
            }
        }

        public class ObjectIndexInfo : List<Map.MetaInfo>
        {
            public Hashtable identHT = new Hashtable();
            public List<string> TagTypes = new List<string>();

            public ObjectIndexInfo(Map map)
            {
                switch (map.HaloVersion)
                {
                    case Map.HaloVersionEnum.Halo2:
                        this.LoadHalo2ObjectIndexInfo(map);
                        break;

                    case Map.HaloVersionEnum.HaloCE:
                        this.LoadHaloCEObjectIndexInfo(map);
                        break;

                    case Map.HaloVersionEnum.Halo1:
                        this.LoadHalo1ObjectIndexInfo(map);
                        break;
                }
                this.TagTypes.Sort();
            }

            private void LoadHalo1ObjectIndexInfo(Map map)
            {
                map.io.In.BaseStream.Position = map.Index_Header.Tags_Offset;
                for (int i = 0; i < map.Index_Header.Meta_Count; i++)
                {
                    string item = core.TagInterface.String.ReverseString(core.TagInterface.String.ReadString(ref map.io.In, 4));
                    string str2 = core.TagInterface.String.ReverseString(core.TagInterface.String.ReadString(ref map.io.In, 4));
                    string str3 = core.TagInterface.String.ReverseString(core.TagInterface.String.ReadString(ref map.io.In, 4));
                    if (this.TagTypes.IndexOf(item) == -1)
                    {
                        this.TagTypes.Add(item);
                    }
                    int stringoffset = map.io.In.ReadInt32() - map.PrimaryMagic;
                    int offset = map.io.In.ReadInt32() - map.PrimaryMagic;
                    int key = map.io.In.ReadInt32();
                    this.identHT.Add(key, i);
                    Stream baseStream = map.io.In.BaseStream;
                    baseStream.Position += 8L;
                    base.Add(new Map.MetaInfo(item, offset, 0, key, stringoffset, 0));
                }
                foreach (Map.MetaInfo info in this)
                {
                    int mapSize = map.Map_Header.MapSize;
                    for (int j = 0; j < base.Count; j++)
                    {
                        if (((base[j].Offset < mapSize) && (base[j].Offset > info.Offset)) && (base[j].Offset > 0))
                        {
                            mapSize = base[j].Offset;
                        }
                    }
                    info.size = mapSize - info.Offset;
                    if (info.Size < 0)
                    {
                        info.size = 0;
                    }
                }
            }

            private void LoadHalo2ObjectIndexInfo(Map map)
            {
                map.io.In.BaseStream.Position = map.Index_Header.Tags_Offset;
                for (int i = 0; i < map.Index_Header.Meta_Count; i++)
                {
                    string item = core.TagInterface.String.ReverseString(core.TagInterface.String.ReadString(ref map.io.In, 4));
                    if (this.TagTypes.IndexOf(item) == -1)
                    {
                        this.TagTypes.Add(item);
                    }
                    int key = map.io.In.ReadInt32();
                    this.identHT.Add(key, i);
                    int offset = map.io.In.ReadInt32() - map.SecondaryMagic;
                    int size = map.io.In.ReadInt32();
                    base.Add(new Map.MetaInfo(item, offset, size, key));
                }
            }

            private void LoadHaloCEObjectIndexInfo(Map map)
            {
                map.io.In.BaseStream.Position = map.Index_Header.Tags_Offset;
                for (int i = 0; i < map.Index_Header.Meta_Count; i++)
                {
                    string item = core.TagInterface.String.ReverseString(core.TagInterface.String.ReadString(ref map.io.In, 4));
                    string str2 = core.TagInterface.String.ReverseString(core.TagInterface.String.ReadString(ref map.io.In, 4));
                    string str3 = core.TagInterface.String.ReverseString(core.TagInterface.String.ReadString(ref map.io.In, 4));
                    if (this.TagTypes.IndexOf(item) == -1)
                    {
                        this.TagTypes.Add(item);
                    }
                    int key = map.io.In.ReadInt32();
                    this.identHT.Add(key, i);
                    int stringoffset = map.io.In.ReadInt32() - map.PrimaryMagic;
                    int offset = map.io.In.ReadInt32();
                    int index = -1;
                    int size = 0;
                    if ((item == "bitm") && !map.BitmapLibary.error)
                    {
                        if (offset < map.PrimaryMagic)
                        {
                            index = offset;
                            size = map.BitmapLibary.RawSize[index];
                            offset = map.BitmapLibary.RawOffset[index];
                        }
                        else
                        {
                            offset -= map.PrimaryMagic;
                        }
                    }
                    else if ((item == "bitm") && map.BitmapLibary.error)
                    {
                        if (offset < map.PrimaryMagic)
                        {
                            size = 0;
                            offset = 0;
                        }
                        else
                        {
                            offset -= map.PrimaryMagic;
                        }
                    }
                    else
                    {
                        offset -= map.PrimaryMagic;
                    }
                    Stream baseStream = map.io.In.BaseStream;
                    baseStream.Position += 8L;
                    base.Add(new Map.MetaInfo(item, offset, size, key, stringoffset, index));
                }
                foreach (Map.MetaInfo info in this)
                {
                    if (!info.IsExternal)
                    {
                        int mapSize = map.Map_Header.MapSize;
                        for (int j = 0; j < base.Count; j++)
                        {
                            if (((base[j].Offset < mapSize) && (base[j].Offset > info.Offset)) && (base[j].Offset > 0))
                            {
                                mapSize = base[j].Offset;
                            }
                        }
                        info.size = mapSize - info.Offset;
                        if (info.Size < 0)
                        {
                            info.size = 0;
                        }
                    }
                }
            }
        }

        public class StringID : CloneableObject
        {
            private int length;
            private string name;
            private int offset;

            public StringID(string name, int length, int offset)
            {
                this.name = name;
                this.length = length;
                this.offset = offset;
            }

            public int Length
            {
                get
                {
                    return this.length;
                }
            }

            public string Name
            {
                get
                {
                    return this.name;
                }
            }

            public int Offset
            {
                get
                {
                    return this.offset;
                }
            }
        }

        public class StringsInfo : List<Map.StringID>
        {
            public StringsInfo()
            {
            }

            public StringsInfo(ref EndianIO.EndianReader BR, Map map)
            {
                int num;
                string[] strArray = new string[map.Map_Header.StringCount];
                int[] numArray = new int[map.Map_Header.StringCount];
                int[] numArray2 = new int[map.Map_Header.StringCount];
                BR.BaseStream.Position = map.Map_Header.StringIndexOffset;
                for (num = 0; num < map.Map_Header.StringCount; num++)
                {
                    numArray2[num] = BR.ReadInt32();
                }
                for (num = 0; num < map.Map_Header.StringCount; num++)
                {
                    int num2;
                    if (num != (map.Map_Header.StringCount - 1))
                    {
                        num2 = numArray2[num + 1] - numArray2[num];
                    }
                    else
                    {
                        num2 = map.Map_Header.StringNames2Size - numArray2[num];
                    }
                    numArray[num] = num2 - 1;
                    BR.BaseStream.Position = map.Map_Header.StringNames2Offset + numArray2[num];
                    strArray[num] = new string(BR.ReadChars(num2 - 1));
                }
                for (num = 0; num < map.Map_Header.StringCount; num++)
                {
                    base.Add(new Map.StringID(strArray[num], numArray[num], numArray2[num]));
                }
            }

            public int FindStringByName(string name)
            {
                for (int i = 0; i < base.Count; i++)
                {
                    if (base[i].Name == name)
                    {
                        return i;
                    }
                }
                return -1;
            }

            public Map.StringID GetString(int index)
            {
                return base[index];
            }
        }
    }
}

