﻿namespace core
{
    using core.IO;
    using System;
    using System.IO;

    public class Meta : Map.MetaInfo
    {
        private Map.HaloVersionEnum haloversion;
        public MetaInterface Interface;
        internal int magic;
        public MemoryStream MS;
        internal string name;
        internal int tagnumber;

        public Meta()
        {
        }

        public Meta(int tagnumber, ref Map map)
        {
            this.haloversion = map.HaloVersion;
            base.offset = map.Meta[tagnumber].offset;
            base.tagType = map.Meta[tagnumber].tagType;
            base.size = map.Meta[tagnumber].size;
            base.ident = map.Meta[tagnumber].Ident;
            base.bitmap_index = map.Meta[tagnumber].External_Bitmap_Index;
            if (base.bitmap_index != -1)
            {
                map.BitmapLibary.io.Open();
                this.ReadMetaFromMap(ref map.BitmapLibary.io.In, tagnumber, ref map);
                map.BitmapLibary.io.Close();
            }
            else
            {
                map.io.Open();
                this.ReadMetaFromMap(ref map.io.In, tagnumber, ref map);
                map.io.Close();
            }
            this.magic = GetMetaMagic(tagnumber, ref map);
        }

        public static int GetMetaMagic(int tagnumber, ref Map map)
        {
            int num;
            if (map.Meta[tagnumber].TagType == "sbsp")
            {
                num = map.SBSP.FindBSPNumberByBSPIdent(map.Meta[tagnumber].Ident);
                return map.SBSP.sbsp[num].magic;
            }
            if (map.Meta[tagnumber].TagType == "ltmp")
            {
                num = map.SBSP.FindBSPNumberByLightMapIdent(map.Meta[tagnumber].Ident);
                return map.SBSP.sbsp[num].magic;
            }
            if ((map.HaloVersion == Map.HaloVersionEnum.HaloCE) || (map.HaloVersion == Map.HaloVersionEnum.Halo1))
            {
                return map.PrimaryMagic;
            }
            return map.SecondaryMagic;
        }

        private void ReadMetaFromMap(ref EndianIO.EndianReader BR, int tagnum, ref Map map)
        {
            this.name = map.FileName[tagnum].Name;
            this.tagnumber = tagnum;
            this.MS = new MemoryStream(base.Size);
            BR.SeekTo(base.Offset);
            this.MS.Write(BR.ReadBytes(base.Size), 0, base.Size);
            this.Interface = new CompiledTagInterface(tagnum, map);
            this.magic = GetMetaMagic(tagnum, ref map);
        }

        public void SaveMetaToFile(string outputFileName)
        {
            outputFileName = outputFileName.Replace("<", "_");
            outputFileName = outputFileName.Replace(">", "_");
            FileStream output = new FileStream(outputFileName, FileMode.Create);
            BinaryWriter writer = new BinaryWriter(output);
            writer.BaseStream.Position = 0L;
            writer.BaseStream.Write(this.MS.ToArray(), 0, base.size);
            writer.Close();
            output.Close();
        }

        public Map.HaloVersionEnum HaloVersion
        {
            get
            {
                return this.haloversion;
            }
        }

        public int Magic
        {
            get
            {
                return this.magic;
            }
        }

        public string Name
        {
            get
            {
                return this.name;
            }
        }

        public int Tag_Number
        {
            get
            {
                return this.tagnumber;
            }
        }
    }
}

