﻿namespace core
{
    using core.IO;
    using core.TagInterface;
    using System;
    using System.IO;

    public sealed class TagCompiler
    {
        private static EndianIO io;
        private static Meta meta;

        public static Meta CompileMetaFromTagInterface(MetaInterface metainterface, int mapoffset, int magic)
        {
            meta = new Meta();
            meta.offset = mapoffset;
            meta.MS = new MemoryStream();
            meta.magic = magic;
            meta.Interface = new CompiledTagInterface(metainterface.Type);
            meta.Interface.Header = metainterface.Header;
            meta.tagType = metainterface.Tag_Type;
            meta.name = metainterface.Tag_Name;
            io = new EndianIO(meta.MS, EndianType.LittleEndian);
            io.Open();
            io.Out.SeekTo(0);
            RecursivelyWriteElements(meta.Interface.Header);
            Padding.Write(ref io.Out, (long) ((int) io.Out.BaseStream.Position), 4);
            RecursivelyFixTagBlocks(meta.Interface.Header);
            meta.size = (int) io.Out.BaseStream.Length;
            io.Close();
            return meta;
        }

        public static Meta CompileSBSPMetaFromTagInterface(MetaInterface metainterface, int mapoffset, int magic, MetaInterface lightmap)
        {
            meta = new Meta();
            meta.offset = mapoffset;
            meta.MS = new MemoryStream();
            meta.magic = magic;
            meta.Interface = new CompiledTagInterface(metainterface.Type);
            meta.Interface.Header = (TagBlock) metainterface.Header.Clone();
            meta.tagType = metainterface.Tag_Type;
            meta.name = metainterface.Tag_Name;
            io = new EndianIO(meta.MS, EndianType.LittleEndian);
            io.Open();
            io.Out.SeekTo(0);
            RecursivelyWriteElements(meta.Interface.Header);
            RecursivelyFixTagBlocks(meta.Interface.Header);
            int length = (int) io.Out.BaseStream.Length;
            io.Out.SeekTo(length);
            RecursivelyWriteElements(lightmap.Header);
            RecursivelyFixTagBlocks(lightmap.Header);
            io.Out.SeekTo((int) io.Out.BaseStream.Length);
            Padding.Write(ref io.Out, (long) ((int) io.Out.BaseStream.Length), 0x1000);
            meta.size = (int) io.Out.BaseStream.Length;
            int num2 = (mapoffset + 0x10) + magic;
            int num3 = (mapoffset + length) + magic;
            io.Out.SeekTo(0);
            io.Out.Write(meta.size);
            io.Out.Write(num2);
            io.Out.Write(num3);
            io.Close();
            return meta;
        }

        private static void RecursivelyFixTagBlocks(TagBlock tagblock)
        {
            foreach (TagBlockLayout layout in tagblock.chunks)
            {
                foreach (BaseObject obj2 in layout)
                {
                    TagBlock block;
                    if (obj2 is Raw_Tag_Block)
                    {
                        io.Out.SeekTo(obj2.MetaOffset + 20);
                        block = ((Raw_Tag_Block) obj2).Resource_Block;
                        if (block.ChunkCount != 0)
                        {
                            block.Translation.Write(meta.magic, meta.offset, ref io.Out);
                        }
                    }
                    else if (obj2 is TagBlock)
                    {
                        io.Out.SeekTo(obj2.MetaOffset + 4);
                        block = (TagBlock) obj2;
                        if (block.ChunkCount != 0)
                        {
                            block.Translation.Write(meta.magic, meta.offset, ref io.Out);
                        }
                        RecursivelyFixTagBlocks(block);
                    }
                }
            }
        }

        private static void RecursivelyLookForTagBlocks(TagBlock tagblock)
        {
            foreach (TagBlockLayout layout in tagblock.chunks)
            {
                foreach (BaseObject obj2 in layout)
                {
                    if (obj2 is Raw_Tag_Block)
                    {
                        RecursivelyWriteElements(((Raw_Tag_Block) obj2).Resource_Block);
                    }
                    else if (obj2 is TagBlock)
                    {
                        TagBlock block = (TagBlock) obj2;
                        RecursivelyWriteElements(block);
                    }
                }
            }
        }

        private static void RecursivelyWriteElements(TagBlock tagblock)
        {
            if (tagblock.ChunkCount != 0)
            {
                long num;
                if ((meta.TagType == "phmo") && (tagblock.Name != "phmo"))
                {
                    num = (io.Out.BaseStream.Position + meta.offset) + meta.magic;
                    Padding.Write(ref io.Out, num, 0x10);
                }
                if ((tagblock.Padding_Interval != 0) && (meta.TagType != "phmo"))
                {
                    num = (io.Out.BaseStream.Position + meta.offset) + meta.magic;
                    Padding.Write(ref io.Out, num, tagblock.Padding_Interval);
                }
                tagblock.Translation.Meta_Translation_Offset = (int) io.Out.BaseStream.Position;
                tagblock.WriteChunks(ref io.Out, MetaInterface.InterfaceMode.CompiledTagMode);
                RecursivelyLookForTagBlocks(tagblock);
            }
        }
    }
}

