﻿namespace core.TagInterface
{
    using core.IO;
    using System;
    using System.ComponentModel;

    public class Angle : BaseObject
    {
        private float Value;

        public Angle(string name) : base(name)
        {
            base.size = 4;
            base.type = BaseObjectType.Angle;
        }

        public override BaseObject Clone()
        {
            Angle angle = new Angle(base.name);
            angle.Value = this.Value;
            angle.SetMetaOffset(base.MetaOffset);
            angle.SetOffset(base.Offset);
            angle.SetMapOffset(base.MapOffset);
            angle.Name = base.Name;
            angle.SetType(base.Type);
            return angle;
        }

        public float ConvertFromRadian(float angle)
        {
            return (float) ((angle * 180.0) / 3.1415926535897931);
        }

        public float ConvertToRadian(float angle)
        {
            if (!(angle == 0f))
            {
                double num = 0.017453292519943295 * angle;
                return (float) num;
            }
            return 0f;
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            this.Value = BR.ReadSingle();
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.Value);
        }

        [Category("Values")]
        public float AngleInDegrees
        {
            get
            {
                return this.ConvertFromRadian(this.Value);
            }
            set
            {
                this.Value = this.ConvertToRadian(value);
            }
        }

        [Category("Values")]
        public float AngleInRadians
        {
            get
            {
                return this.Value;
            }
            set
            {
                this.Value = value;
            }
        }
    }
}

