﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;

    public abstract class BaseObject
    {
        protected int mapOffset;
        protected int metaOffset;
        protected string name;
        protected int offset;
        protected TagBlock parent;
        protected int size;
        protected BaseObjectType type;

        public BaseObject()
        {
        }

        public BaseObject(string name)
        {
            if (name != null)
            {
                this.name = name;
            }
        }

        public virtual BaseObject Clone()
        {
            return null;
        }

        public void SeekToObjectMapOffset(ref EndianIO.EndianReader BR)
        {
            BR.SeekTo(this.mapOffset);
        }

        public void SeekToObjectMapOffset(ref EndianIO.EndianWriter BW)
        {
            BW.SeekTo(this.mapOffset);
        }

        internal void SetMapOffset(int value)
        {
            this.mapOffset = value;
        }

        internal void SetMetaOffset(int value)
        {
            this.metaOffset = value;
        }

        internal void SetOffset(int value)
        {
            this.offset = value;
        }

        internal void SetParent(TagBlock value)
        {
            this.parent = value;
        }

        internal void SetType(BaseObjectType value)
        {
            this.type = value;
        }

        [Category("Base Properties")]
        public int MapOffset
        {
            get
            {
                return this.mapOffset;
            }
        }

        [Category("Base Properties")]
        public int MetaOffset
        {
            get
            {
                return this.metaOffset;
            }
        }

        [Category("Base Properties")]
        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        [Category("Base Properties")]
        public int Offset
        {
            get
            {
                return this.offset;
            }
        }

        [Category("Base Properties"), TypeConverter(typeof(MetaInterface.BaseObjectConverter))]
        public TagBlock Parent
        {
            get
            {
                return this.parent;
            }
        }

        [Category("Base Properties")]
        public int Size
        {
            get
            {
                return this.size;
            }
        }

        [Category("Base Properties")]
        public BaseObjectType Type
        {
            get
            {
                return this.type;
            }
        }
    }
}

