﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;
    using System.IO;

    public class Data : BaseObject
    {
        protected Map.HaloVersionEnum HaloVersion;
        protected MemoryStream ms;
        protected int rawdatasize;
        protected RawDataTypes rawdatatype;

        public Data(string name, Map.HaloVersionEnum haloversion, RawDataTypes rawdatatype) : base(name)
        {
            this.rawdatatype = rawdatatype;
            this.HaloVersion = haloversion;
            switch (haloversion)
            {
                case Map.HaloVersionEnum.Halo2:
                    switch (rawdatatype)
                    {
                        case RawDataTypes.H2_jmad:
                        case RawDataTypes.H2_ugh_sound:
                            base.size = 8;
                            break;
                    }
                    break;

                case Map.HaloVersionEnum.HaloCE:
                    base.size = 20;
                    break;
            }
            base.type = BaseObjectType.Data;
        }

        public override BaseObject Clone()
        {
            Data data = new Data(base.name, this.HaloVersion, this.rawdatatype);
            data.ms = new MemoryStream(this.rawdatasize);
            if (this.ms != null)
            {
                data.ms.Write(this.ms.ToArray(), 0, this.rawdatasize);
            }
            data.rawdatasize = this.rawdatasize;
            data.size = base.size;
            data.SetMetaOffset(base.MetaOffset);
            data.SetOffset(base.Offset);
            data.SetMapOffset(base.MapOffset);
            data.Name = base.Name;
            data.SetType(base.Type);
            return data;
        }

        public virtual void Read(ref EndianIO.EndianReader BR)
        {
            switch (this.HaloVersion)
            {
                case Map.HaloVersionEnum.HaloCE:
                    this.rawdatasize = BR.ReadInt32();
                    BR.ReadInt32();
                    BR.ReadInt32();
                    BR.ReadInt32();
                    BR.ReadInt32();
                    break;
            }
        }

        public virtual void Read_Raw(ref EndianIO.EndianReader BR)
        {
            this.ms = new MemoryStream(this.Total_Raw_Data_Size);
            byte[] rawData = BR.ReadBytes(this.Total_Raw_Data_Size);
            this.ms.Write(rawData, 0, this.Total_Raw_Data_Size);
        }

        public virtual void Write(ref EndianIO.EndianWriter BW)
        {
            switch (this.HaloVersion)
            {
                case Map.HaloVersionEnum.HaloCE:
                    BW.Write(this.rawdatasize);
                    BW.Write(0);
                    BW.Write(0);
                    BW.Write(0);
                    BW.Write(0);
                    break;
            }
        }

        public virtual void Write_Raw(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.ms.ToArray());
        }

        public virtual void Write_Raw_Padding(ref EndianIO.EndianWriter BW)
        {
            BW.Write(new byte[Functions.Padding(this.Total_Raw_Data_Size, 0x200)]);
        }

        public Map.HaloVersionEnum Halo_Version
        {
            get
            {
                return this.HaloVersion;
            }
        }

        public MemoryStream MS
        {
            get
            {
                return this.ms;
            }
            set
            {
                this.ms = value;
            }
        }

        public RawDataTypes Raw_Data_Type
        {
            get
            {
                return this.rawdatatype;
            }
        }

        [Category("Values")]
        public int Total_Raw_Data_Size
        {
            get
            {
                return this.rawdatasize;
            }
            set
            {
                this.rawdatasize = value;
            }
        }

        public enum RawDataTypes
        {
            H1_vertices_uncompressed,
            H1_vertices_compressed,
            H1_bitm,
            H2_jmad,
            H2_mode1,
            H2_mode2,
            H2_bitm,
            H2_weat,
            H2_DECR,
            H2_PRTM,
            H2_ugh_sound,
            H2_ugh_coconuts,
            H2_sbsp_visualmesh,
            H2_sbsp_water,
            H2_sbsp_scenery,
            H2_sbsp_lightmap,
            H2_sbsp_unknown1
        }
    }
}

