﻿namespace core.TagInterface
{
    using core.IO;
    using System;

    public class Enumerator : BaseObject
    {
        public TagEnumeratorIndexType IndexType;
        public int Value;

        public Enumerator(string name, TagEnumeratorIndexType type) : base(name)
        {
            this.IndexType = type;
            switch (this.IndexType)
            {
                case TagEnumeratorIndexType.Word:
                    base.size = 2;
                    break;

                case TagEnumeratorIndexType.Long:
                    base.size = 4;
                    break;

                case TagEnumeratorIndexType.Byte:
                    base.size = 1;
                    break;
            }
            base.type = BaseObjectType.TagEnumerator;
        }

        public override BaseObject Clone()
        {
            core.TagInterface.Enumerator TagEnumerator = new core.TagInterface.Enumerator(base.name, this.IndexType);
            TagEnumerator.Value = this.Value;
            TagEnumerator.SetMetaOffset(base.MetaOffset);
            TagEnumerator.SetOffset(base.Offset);
            TagEnumerator.SetMapOffset(base.MapOffset);
            TagEnumerator.Name = base.Name;
            TagEnumerator.SetType(base.Type);
            return TagEnumerator;
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            switch (this.IndexType)
            {
                case TagEnumeratorIndexType.Word:
                    this.Value = BR.ReadInt16();
                    break;

                case TagEnumeratorIndexType.Long:
                    this.Value = BR.ReadInt32();
                    break;

                case TagEnumeratorIndexType.Byte:
                    this.Value = BR.ReadByte();
                    break;
            }
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            switch (this.IndexType)
            {
                case TagEnumeratorIndexType.Word:
                    BW.Write((short) this.Value);
                    break;

                case TagEnumeratorIndexType.Long:
                    BW.Write(this.Value);
                    break;

                case TagEnumeratorIndexType.Byte:
                    BW.Write((byte) this.Value);
                    break;
            }
        }

        public enum TagEnumeratorIndexType
        {
            Word,
            Long,
            Byte
        }
    }
}

