﻿namespace core.TagInterface
{
    using core.IO;
    using System;

    public class Padding : BaseObject
    {
        private byte[] data;

        public Padding()
        {
            base.name = "Padding";
            base.size = 4;
            base.type = BaseObjectType.Padding;
        }

        public Padding(int size)
        {
            base.name = "Padding";
            base.size = size;
            base.type = BaseObjectType.Padding;
        }

        public Padding(string name, int size) : base(name)
        {
            base.size = size;
            base.type = BaseObjectType.Padding;
        }

        public static int Calculate_Padding_Size(long offset, int alignment_interval)
        {
            long num = alignment_interval - (offset % ((long) alignment_interval));
            if (num == alignment_interval)
            {
                num = 0L;
            }
            return (int) num;
        }

        public override BaseObject Clone()
        {
            Padding padding = new Padding(base.name, base.size);
            if (this.data != null)
            {
                padding.data = (byte[]) this.data.Clone();
            }
            padding.SetMetaOffset(base.MetaOffset);
            padding.SetOffset(base.Offset);
            padding.SetMapOffset(base.MapOffset);
            padding.Name = base.Name;
            padding.SetType(base.Type);
            return padding;
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            this.data = BR.ReadBytes(base.size);
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.data);
        }

        public static void Write(ref EndianIO.EndianWriter BW, int size)
        {
            byte num = 0;
            for (int i = 0; i < size; i++)
            {
                BW.Write(num);
            }
        }

        public static void Write(ref EndianIO.EndianWriter BW, long offset, int alignment_interval)
        {
            byte[] buffer = new byte[Calculate_Padding_Size(offset, alignment_interval)];
            BW.Write(buffer);
        }

        public byte[] Data
        {
            get
            {
                return this.data;
            }
            set
            {
                this.data = value;
            }
        }
    }
}

