﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;

    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class Pointer : BaseObject
    {
        protected int magic;
        protected int mapTranslation;
        protected int memoryoffset;
        protected int metaTranslation;
        protected int offset_of_meta;
        protected int translation_tag;

        public Pointer(string name) : base(name)
        {
            base.size = 4;
            base.type = BaseObjectType.Pointer;
        }

        public override BaseObject Clone()
        {
            Pointer pointer = new Pointer(base.name);
            pointer.mapTranslation = this.mapTranslation;
            pointer.metaTranslation = this.metaTranslation;
            pointer.Magic = this.Magic;
            pointer.offset_of_meta = this.offset_of_meta;
            pointer.SetMetaOffset(base.MetaOffset);
            pointer.SetOffset(base.Offset);
            pointer.SetMapOffset(base.MapOffset);
            pointer.Name = base.Name;
            pointer.SetType(base.Type);
            return pointer;
        }

        public void Load(Map map, int magic)
        {
            if (this.memoryoffset != 0)
            {
                this.magic = magic;
                this.mapTranslation = this.memoryoffset - magic;
                this.translation_tag = map.FindMetaByOffset(this.mapTranslation);
                this.metaTranslation = this.mapTranslation - map.Meta[this.translation_tag].Offset;
                this.offset_of_meta = map.Meta[this.translation_tag].Offset;
            }
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            this.memoryoffset = BR.ReadInt32();
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            int num = (this.offset_of_meta + this.metaTranslation) + this.Magic;
            BW.Write(num);
        }

        public void Write(int magic, int offset_of_meta, ref EndianIO.EndianWriter BW)
        {
            int num = (offset_of_meta + this.metaTranslation) + magic;
            BW.Write(num);
        }

        public int Magic
        {
            get
            {
                return this.magic;
            }
            set
            {
                this.magic = value;
            }
        }

        public int Map_Translation_Offset
        {
            get
            {
                return this.mapTranslation;
            }
            set
            {
                this.mapTranslation = value;
            }
        }

        public int Memory_Address
        {
            get
            {
                return this.memoryoffset;
            }
            set
            {
                this.memoryoffset = value;
            }
        }

        public int Meta_Translation_Offset
        {
            get
            {
                return this.metaTranslation;
            }
            set
            {
                this.metaTranslation = value;
            }
        }

        public int Meta_Translation_Tag_Number
        {
            get
            {
                return this.translation_tag;
            }
            set
            {
                this.translation_tag = value;
            }
        }
    }
}

