﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;

    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class RawDataOffset : BaseObject
    {
        private int dataoffset;
        private MapTypes maplocation;

        public RawDataOffset(string name) : base(name)
        {
            this.maplocation = MapTypes.Internal;
            base.type = BaseObjectType.RawDataOffset;
        }

        public RawDataOffset(string name, int offset, MapTypes maplocation) : base(name)
        {
            this.maplocation = MapTypes.Internal;
            this.dataoffset = offset;
            this.maplocation = maplocation;
            base.type = BaseObjectType.RawDataOffset;
        }

        private int CalculateHalo2RawDataOffset()
        {
            uint num2;
            if (this.dataoffset == -1)
            {
                return -1;
            }
            int dataoffset = this.dataoffset;
            switch (this.maplocation)
            {
                case MapTypes.Internal:
                    return this.dataoffset;

                case MapTypes.MainMenu:
                    num2 = (uint) (dataoffset | 0x40000000);
                    return (int) num2;

                case MapTypes.SPShared:
                    num2 = (uint) (dataoffset | -1073741824);
                    return (int) num2;

                case MapTypes.MPShared:
                    num2 = (uint) (dataoffset | -2147483648);
                    return (int) num2;
            }
            return dataoffset;
        }

        public override BaseObject Clone()
        {
            RawDataOffset offset = new RawDataOffset(base.name);
            offset.dataoffset = this.dataoffset;
            offset.maplocation = this.maplocation;
            offset.SetMetaOffset(base.MetaOffset);
            offset.SetOffset(base.Offset);
            offset.SetMapOffset(base.MapOffset);
            offset.Name = base.Name;
            offset.SetType(base.Type);
            return offset;
        }

        private void ParseHalo2RawDataOffset()
        {
            if (this.dataoffset != -1)
            {
                long num = this.dataoffset & ((long) 0xc0000000L);
                this.dataoffset &= 0x3fffffff;
                switch (num)
                {
                    case 0L:
                        this.maplocation = MapTypes.Internal;
                        return;

                    case 0x80000000L:
                        this.maplocation = MapTypes.MPShared;
                        return;

                    case 0xc0000000L:
                        this.maplocation = MapTypes.SPShared;
                        break;

                    case 0x40000000L:
                        this.maplocation = MapTypes.MainMenu;
                        break;
                }
            }
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            this.dataoffset = BR.ReadInt32();
            this.ParseHalo2RawDataOffset();
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.CalculateHalo2RawDataOffset());
        }

        public int Data_Offset
        {
            get
            {
                return this.dataoffset;
            }
            set
            {
                this.dataoffset = value;
            }
        }

        public MapTypes Map_Location
        {
            get
            {
                return this.maplocation;
            }
            set
            {
                this.maplocation = value;
            }
        }
    }
}

