﻿namespace core.TagInterface
{
    using core.IO;
    using System;
    using System.ComponentModel;
    using System.Drawing;

    public class RealRGBAColor : BaseObject
    {
        public float a;
        public float b;
        private System.Drawing.Color color;
        public float g;
        public float r;

        public RealRGBAColor(string name) : base(name)
        {
            base.size = 0x10;
            base.type = BaseObjectType.RealRGBAColor;
        }

        public override BaseObject Clone()
        {
            RealARGBColor color = new RealARGBColor(base.name);
            color.a = this.a;
            color.r = this.r;
            color.g = this.g;
            color.b = this.b;
            color.SetMetaOffset(base.MetaOffset);
            color.SetOffset(base.Offset);
            color.SetMapOffset(base.MapOffset);
            color.Name = base.Name;
            color.SetType(base.Type);
            return color;
        }

        private void ConvertColor()
        {
            this.a = ((float) this.color.A) / 255f;
            this.r = ((float) this.color.R) / 255f;
            this.g = ((float) this.color.G) / 255f;
            this.b = ((float) this.color.B) / 255f;
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            this.r = BR.ReadSingle();
            this.g = BR.ReadSingle();
            this.b = BR.ReadSingle();
            this.a = BR.ReadSingle();
            this.UpdateColor();
        }

        private void UpdateColor()
        {
            if (this.r > 1f)
            {
                this.r = 1f;
            }
            if (this.g > 1f)
            {
                this.g = 1f;
            }
            if (this.b > 1f)
            {
                this.b = 1f;
            }
            if (this.a > 1f)
            {
                this.a = 1f;
            }
            if (this.r < 0f)
            {
                this.r = 0f;
            }
            if (this.g < 0f)
            {
                this.g = 0f;
            }
            if (this.b < 0f)
            {
                this.b = 0f;
            }
            if (this.a < 0f)
            {
                this.b = 0f;
            }
            this.color = System.Drawing.Color.FromArgb((int) (this.a * 255f), (int) (this.r * 255f), (int) (this.g * 255f), (int) (this.b * 255f));
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.r);
            BW.Write(this.g);
            BW.Write(this.b);
            BW.Write(this.a);
        }

        [Category("Values")]
        public float A
        {
            get
            {
                return this.a;
            }
            set
            {
                this.UpdateColor();
                this.a = value;
            }
        }

        [Category("Values")]
        public float B
        {
            get
            {
                return this.b;
            }
            set
            {
                this.UpdateColor();
                this.b = value;
            }
        }

        [Category("Values")]
        public System.Drawing.Color Color
        {
            get
            {
                return this.color;
            }
            set
            {
                this.color = value;
                this.ConvertColor();
            }
        }

        [Category("Values")]
        public float G
        {
            get
            {
                return this.g;
            }
            set
            {
                this.UpdateColor();
                this.g = value;
            }
        }

        [Category("Values")]
        public float R
        {
            get
            {
                return this.r;
            }
            set
            {
                this.UpdateColor();
                this.r = value;
            }
        }
    }
}

