﻿namespace core.TagInterface
{
    using core.IO;
    using System;
    using System.ComponentModel;
    using System.Drawing;

    public class RealRGBColor : BaseObject
    {
        private float b;
        private System.Drawing.Color color;
        private float g;
        private float r;

        public RealRGBColor(string name) : base(name)
        {
            base.size = 12;
            base.type = BaseObjectType.RealRGBColor;
        }

        public override BaseObject Clone()
        {
            RealRGBColor color = new RealRGBColor(base.name);
            color.r = this.r;
            color.g = this.g;
            color.b = this.b;
            color.SetMetaOffset(base.MetaOffset);
            color.SetOffset(base.Offset);
            color.SetMapOffset(base.MapOffset);
            color.Name = base.Name;
            color.SetType(base.Type);
            return color;
        }

        private void ConvertColor()
        {
            this.r = ((float) this.color.R) / 255f;
            this.g = ((float) this.color.G) / 255f;
            this.b = ((float) this.color.B) / 255f;
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            this.r = BR.ReadSingle();
            this.g = BR.ReadSingle();
            this.b = BR.ReadSingle();
            this.UpdateColor();
        }

        private void UpdateColor()
        {
            if (this.r > 1f)
            {
                this.r = 1f;
            }
            if (this.g > 1f)
            {
                this.g = 1f;
            }
            if (this.b > 1f)
            {
                this.b = 1f;
            }
            if (this.r < 0f)
            {
                this.r = 0f;
            }
            if (this.g < 0f)
            {
                this.g = 0f;
            }
            if (this.b < 0f)
            {
                this.b = 0f;
            }
            this.color = System.Drawing.Color.FromArgb(0xff, (int) (this.r * 255f), (int) (this.g * 255f), (int) (this.b * 255f));
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.r);
            BW.Write(this.g);
            BW.Write(this.b);
        }

        [Category("Values")]
        public float B
        {
            get
            {
                return this.b;
            }
            set
            {
                this.UpdateColor();
                this.b = value;
            }
        }

        [Category("Values")]
        public System.Drawing.Color Color
        {
            get
            {
                return this.color;
            }
            set
            {
                this.color = value;
                this.ConvertColor();
            }
        }

        [Category("Values")]
        public float G
        {
            get
            {
                return this.g;
            }
            set
            {
                this.UpdateColor();
                this.g = value;
            }
        }

        [Category("Values")]
        public float R
        {
            get
            {
                return this.r;
            }
            set
            {
                this.UpdateColor();
                this.r = value;
            }
        }
    }
}

