﻿namespace core.TagInterface
{
    using core.IO;
    using System;
    using System.ComponentModel;
    using System.IO;

    public class String : BaseObject
    {
        public int stringlength;
        public string Value;

        public String(string name, int length) : base(name)
        {
            base.size = length;
            this.stringlength = length;
            base.type = BaseObjectType.String;
        }

        public override BaseObject Clone()
        {
            core.TagInterface.String str = new core.TagInterface.String(base.name, this.stringlength);
            str.Value = this.Value;
            str.SetMetaOffset(base.MetaOffset);
            str.SetOffset(base.Offset);
            str.SetMapOffset(base.MapOffset);
            str.Name = base.Name;
            str.SetType(base.Type);
            return str;
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            this.Value = this.ReadString(ref BR);
        }

        public string ReadString(ref EndianIO.EndianReader BR)
        {
            string str = "";
            int num = 0;
            while (str != null)
            {
                int num2 = BR.ReadByte();
                num++;
                str = str + char.ConvertFromUtf32(num2);
                if (num == this.stringlength)
                {
                    break;
                }
            }
            int num3 = this.stringlength - num;
            Stream baseStream = BR.BaseStream;
            baseStream.Position += num3;
            return str;
        }

        public static string ReadString(ref EndianIO.EndianReader BR, int length)
        {
            string str = "";
            int num = 0;
            while (str != null)
            {
                int num2 = BR.ReadByte();
                num++;
                if (num2 == 0)
                {
                    break;
                }
                str = str + char.ConvertFromUtf32(num2);
                if (num == length)
                {
                    break;
                }
            }
            int num3 = length - num;
            Stream baseStream = BR.BaseStream;
            baseStream.Position += num3;
            return str;
        }

        public void ReverseString()
        {
            char[] array = this.Value.ToCharArray();
            Array.Reverse(array);
            this.Value = new string(array);
        }

        public static string ReverseString(string value)
        {
            char[] array = value.ToCharArray();
            Array.Reverse(array);
            return new string(array);
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            foreach (char ch in this.Value)
            {
                BW.Write(Convert.ToByte(ch));
            }
            int num = this.stringlength - this.Value.Length;
            byte num2 = 0;
            for (int i = 0; i < num; i++)
            {
                BW.Write(num2);
            }
        }

        public static void WriteString(ref EndianIO.EndianWriter BW, string value, int length)
        {
            foreach (char ch in value)
            {
                BW.Write(Convert.ToByte(ch));
            }
            int num = length - value.Length;
            byte num2 = 0;
            for (int i = 0; i < num; i++)
            {
                BW.Write(num2);
            }
        }

        [Category("Values")]
        public string Text
        {
            get
            {
                return this.Value;
            }
            set
            {
                this.Value = value;
            }
        }
    }
}

