﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;

    public class Syntax_Block : BaseObject
    {
        private byte[] bytevalue;
        private short expression_index;
        private short expression_type;
        private short identity;
        private int sibling_pointer;
        private int string_pointer;
        private core.TagInterface.StringID stringid;
        private TagIdent tagreference;
        private bool usesstringid;
        private bool usestagreference;
        private short value_type;

        public Syntax_Block(string name) : base(name)
        {
            this.tagreference = new TagIdent("Tag Ident");
            this.usestagreference = false;
            this.stringid = new core.TagInterface.StringID("String ID");
            this.usesstringid = false;
            this.bytevalue = new byte[4];
            base.size = 20;
            base.type = BaseObjectType.Syntax_Block;
        }

        public override BaseObject Clone()
        {
            Syntax_Block block = new Syntax_Block(base.name);
            block.expression_index = this.expression_index;
            block.identity = this.identity;
            block.value_type = this.value_type;
            block.expression_type = this.expression_type;
            block.sibling_pointer = this.sibling_pointer;
            block.string_pointer = this.string_pointer;
            block.tagreference = (TagIdent) this.tagreference.Clone();
            block.usestagreference = this.usestagreference;
            block.stringid = (core.TagInterface.StringID) this.stringid.Clone();
            block.usesstringid = this.usesstringid;
            block.SetMetaOffset(base.MetaOffset);
            block.SetOffset(base.Offset);
            block.SetMapOffset(base.MapOffset);
            block.Name = base.Name;
            block.SetType(base.Type);
            block.bytevalue = (byte[]) this.bytevalue.Clone();
            return block;
        }

        public void Read(ref EndianIO.EndianReader BR, MetaInterface.InterfaceMode mode)
        {
            this.expression_index = BR.ReadInt16();
            this.identity = BR.ReadInt16();
            this.value_type = BR.ReadInt16();
            this.expression_type = BR.ReadInt16();
            this.sibling_pointer = BR.ReadInt32();
            this.string_pointer = BR.ReadInt32();
            switch (this.Value_Type)
            {
                case ValueTypeEnum.sound:
                case ValueTypeEnum.effe:
                case ValueTypeEnum.jpt:
                case ValueTypeEnum.lsnd:
                case ValueTypeEnum.jmad:
                case ValueTypeEnum.tag:
                case ValueTypeEnum.mode:
                    if (this.Expression_Type != ExpressionTypeEnum.value)
                    {
                        break;
                    }
                    this.usestagreference = true;
                    this.tagreference.Read(ref BR, mode);
                    return;

                case ValueTypeEnum.stringid:
                    if (this.Expression_Type == ExpressionTypeEnum.value)
                    {
                        this.usesstringid = true;
                        this.stringid.Read(ref BR, mode);
                        return;
                    }
                    break;
            }
            this.Value = BR.ReadBytes(4);
        }

        public void Write(ref EndianIO.EndianWriter BW, MetaInterface.InterfaceMode mode)
        {
            BW.Write(this.expression_index);
            BW.Write(this.identity);
            BW.Write(this.value_type);
            BW.Write(this.expression_type);
            BW.Write(this.sibling_pointer);
            BW.Write(this.string_pointer);
            switch (this.Value_Type)
            {
                case ValueTypeEnum.sound:
                case ValueTypeEnum.effe:
                case ValueTypeEnum.jpt:
                case ValueTypeEnum.lsnd:
                case ValueTypeEnum.jmad:
                case ValueTypeEnum.tag:
                case ValueTypeEnum.mode:
                    if (this.Expression_Type != ExpressionTypeEnum.value)
                    {
                        break;
                    }
                    this.tagreference.Write(ref BW, mode);
                    return;

                case ValueTypeEnum.stringid:
                    if (this.Expression_Type == ExpressionTypeEnum.value)
                    {
                        this.stringid.Write(ref BW, mode);
                        return;
                    }
                    break;
            }
            BW.Write(this.Value);
        }

        [Category("Values")]
        public short Expression_Index
        {
            get
            {
                return this.expression_index;
            }
            set
            {
                this.expression_index = value;
            }
        }

        [Category("Values")]
        public ExpressionTypeEnum Expression_Type
        {
            get
            {
                return (ExpressionTypeEnum) this.expression_type;
            }
            set
            {
                this.expression_type = (short) value;
            }
        }

        [Category("Values")]
        public short Identity
        {
            get
            {
                return this.identity;
            }
            set
            {
                this.identity = value;
            }
        }

        public bool Is_String_ID_Referenced
        {
            get
            {
                return this.usesstringid;
            }
            set
            {
                this.usesstringid = value;
            }
        }

        public bool Is_Tag_Referenced
        {
            get
            {
                return this.usestagreference;
            }
            set
            {
                this.usestagreference = value;
            }
        }

        [Category("Values")]
        public int Sibling_Pointer
        {
            get
            {
                return this.sibling_pointer;
            }
            set
            {
                this.sibling_pointer = value;
            }
        }

        [Category("Values")]
        public core.TagInterface.StringID String_ID
        {
            get
            {
                return this.stringid;
            }
            set
            {
                this.stringid = value;
            }
        }

        [Category("Values")]
        public int String_Pointer
        {
            get
            {
                return this.string_pointer;
            }
            set
            {
                this.string_pointer = value;
            }
        }

        [Category("Values")]
        public TagIdent Tag_Reference
        {
            get
            {
                return this.tagreference;
            }
            set
            {
                this.tagreference = value;
            }
        }

        public byte[] Value
        {
            get
            {
                return this.bytevalue;
            }
            set
            {
                this.bytevalue = value;
            }
        }

        [Category("Values")]
        public ValueTypeEnum Value_Type
        {
            get
            {
                return (ValueTypeEnum) this.value_type;
            }
            set
            {
                this.value_type = (short) value;
            }
        }

        public enum ExpressionTypeEnum : short
        {
            global = 13,
            static_script = 10,
            syntax = 8,
            value = 9
        }

        public enum ValueTypeEnum
        {
            effe = 0x21,
            jmad = 0x24,
            jpt = 0x22,
            lsnd = 0x23,
            mode = 0x29,
            sound = 0x20,
            stringid = 11,
            tag = 0x26
        }
    }
}

