﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Reflection;

    [DefaultProperty("Name")]
    public class TagBlock : BaseObject
    {
        private int chunkCount;
        private int chunknumber;
        internal ChunkCollection chunks;
        private Map.HaloVersionEnum HaloVersion;
        private TagBlockLayout layout;
        private int paddinginterval;
        private core.TagInterface.Pointer translation;

        public TagBlock(string name, TagBlockLayout tagblocklayout, Map.HaloVersionEnum haloversion) : base(name)
        {
            this.paddinginterval = 0;
            this.layout = new TagBlockLayout();
            this.chunks = new ChunkCollection();
            if (tagblocklayout != null)
            {
                this.layout = tagblocklayout;
            }
            this.translation = new core.TagInterface.Pointer(name);
            this.translation.SetParent(this);
            this.Layout.Parent = this;
            base.size = 8;
            if (haloversion != Map.HaloVersionEnum.Halo2)
            {
                base.size += 4;
            }
            this.HaloVersion = haloversion;
            base.type = BaseObjectType.TagBlock;
        }

        public TagBlock(string name, TagBlockLayout tagblocklayout, Map.HaloVersionEnum haloversion, int padding_alignment_size) : base(name)
        {
            this.paddinginterval = 0;
            this.layout = new TagBlockLayout();
            this.chunks = new ChunkCollection();
            if (tagblocklayout != null)
            {
                this.layout = tagblocklayout;
            }
            this.translation = new core.TagInterface.Pointer(name);
            this.translation.SetParent(this);
            this.Layout.Parent = this;
            base.size = 8;
            if (haloversion != Map.HaloVersionEnum.Halo2)
            {
                base.size += 4;
            }
            this.HaloVersion = haloversion;
            this.paddinginterval = padding_alignment_size;
            base.type = BaseObjectType.TagBlock;
        }

        public TagBlock(string name, TagBlockLayout tagblocklayout, int ChunkCount, Map.HaloVersionEnum haloversion) : base(name)
        {
            this.paddinginterval = 0;
            this.layout = new TagBlockLayout();
            this.chunks = new ChunkCollection();
            if (tagblocklayout != null)
            {
                this.layout = tagblocklayout;
            }
            this.translation = new core.TagInterface.Pointer(name);
            this.translation.SetParent(this);
            this.Layout.Parent = this;
            base.size = 8;
            if (haloversion != Map.HaloVersionEnum.Halo2)
            {
                base.size += 4;
            }
            this.HaloVersion = haloversion;
            this.chunkCount = ChunkCount;
            this.CreateChunks();
            base.type = BaseObjectType.TagBlock;
        }

        public TagBlock(string name, TagBlockLayout tagblocklayout, int ChunkCount, Map.HaloVersionEnum haloversion, int padding_interval) : base(name)
        {
            this.paddinginterval = 0;
            this.layout = new TagBlockLayout();
            this.chunks = new ChunkCollection();
            if (tagblocklayout != null)
            {
                this.layout = tagblocklayout;
            }
            this.translation = new core.TagInterface.Pointer(name);
            this.translation.SetParent(this);
            this.paddinginterval = padding_interval;
            this.Layout.Parent = this;
            base.size = 8;
            if (haloversion != Map.HaloVersionEnum.Halo2)
            {
                base.size += 4;
            }
            this.HaloVersion = haloversion;
            this.chunkCount = ChunkCount;
            this.CreateChunks();
            base.type = BaseObjectType.TagBlock;
        }

        public void AddChunk(TagBlockLayout tagblocklayout)
        {
            this.chunks.Add(tagblocklayout);
            this.chunks[this.chunks.Count - 1].Chunk_Number = this.chunks.Count - 1;
            this.chunks[this.chunks.Count - 1].Parent = this;
            this.chunkCount++;
        }

        public void AddNewChunk()
        {
            this.chunks.Add(this.Layout.New());
            this.chunks[this.chunks.Count - 1].Chunk_Number = this.chunks.Count - 1;
            this.chunks[this.chunks.Count - 1].Parent = this;
            this.chunkCount++;
        }

        public override BaseObject Clone()
        {
            TagBlock block = new TagBlock(base.name, this.layout.Clone(), this.HaloVersion, this.paddinginterval);
            block.chunknumber = this.chunknumber;
            block.chunkCount = this.chunkCount;
            block.translation = (core.TagInterface.Pointer) this.translation.Clone();
            foreach (TagBlockLayout layout in this.chunks)
            {
                block.chunks.Add(layout.Clone());
            }
            block.SetMetaOffset(base.MetaOffset);
            block.SetOffset(base.Offset);
            block.SetMapOffset(base.MapOffset);
            block.Name = base.Name;
            block.SetType(base.Type);
            return block;
        }

        public void CloneChunk(int index)
        {
            this.chunks.Add(this.chunks[index].Clone());
            this.chunks[this.chunks.Count - 1].Chunk_Number = this.chunks.Count - 1;
            this.chunks[this.chunks.Count - 1].Parent = this;
            this.chunkCount++;
        }

        private void CreateChunks()
        {
            this.chunks.Clear();
            for (int i = 0; i < this.ChunkCount; i++)
            {
                this.chunks.Add(this.Layout.New());
                this.chunks[i].Chunk_Number = i;
                this.chunks[i].Parent = this;
            }
        }

        public virtual void CreateChunks(int chunkcount)
        {
            this.chunkCount = chunkcount;
            this.CreateChunks();
        }

        public TagBlockLayout GetChunk(int index)
        {
            return this.chunks[index];
        }

        public void InsertChunk(TagBlockLayout chunk, int index)
        {
            this.chunks.Insert(index, chunk.Clone());
            this.chunks[index].Chunk_Number = index;
            this.chunks[index].Parent = this;
            this.chunkCount++;
        }

        public void PadReflexive(TagBlockLayout chunk, int count)
        {
            int num = (((this.chunkCount - 1) / count) + 1) * count;
            for (int i = this.chunkCount; i < num; i++)
            {
                this.chunks.Add(chunk.Clone());
                this.chunks[i].Chunk_Number = i;
                this.chunks[i].Parent = this;
            }
            this.chunkCount = num;
        }

        public virtual void Read(ref EndianIO.EndianReader BR, MetaInterface.InterfaceMode mode)
        {
            this.chunkCount = BR.ReadInt32();
            this.translation.Read(ref BR);
            if (this.HaloVersion != Map.HaloVersionEnum.Halo2)
            {
                BR.ReadInt32();
            }
            this.CreateChunks();
        }

        private void Read_Reference_Tag_Paths(ref EndianIO.EndianReader BR, Map.HaloVersionEnum haloversion)
        {
            foreach (TagBlockLayout layout in this.chunks)
            {
                foreach (BaseObject obj2 in layout)
                {
                    string str;
                    int num;
                    if (obj2.Type == BaseObjectType.TagReference)
                    {
                        TagReference reference = (TagReference) obj2;
                        if (reference.Ident.Ident != -1)
                        {
                            str = core.TagInterface.String.ReadString(ref BR, reference.Ident.Ident + 1);
                            num = str.LastIndexOf('.');
                            reference.Ident.TagName = str.Substring(0, num);
                            reference.Ident.TagType = Tag_Layouts.GetTagCategory(haloversion).GetTagDefinitionByExtension(str.Substring(num + 1)).TagType;
                        }
                    }
                    else
                    {
                        TagIdent ident;
                        string str2;
                        if (obj2.Type == BaseObjectType.TagIdent)
                        {
                            ident = (TagIdent) obj2;
                            if (ident.Ident != -1)
                            {
                                str = core.TagInterface.String.ReadString(ref BR, ident.Ident + 1);
                                num = str.LastIndexOf('.');
                                ident.TagName = str.Substring(0, num);
                                str2 = str.Substring(num + 1);
                                ident.TagType = Tag_Layouts.GetTagCategory(haloversion).GetTagDefinitionByExtension(str2).TagType;
                            }
                        }
                        else if (obj2.Type == BaseObjectType.Raw_Tag_Block)
                        {
                            Raw_Tag_Block block = (Raw_Tag_Block) obj2;
                            ident = block.Tag_Ident;
                            if (ident.Ident != -1)
                            {
                                str = core.TagInterface.String.ReadString(ref BR, ident.Ident + 1);
                                num = str.LastIndexOf('.');
                                ident.TagName = str.Substring(0, num);
                                str2 = str.Substring(num + 1);
                                ident.TagType = Tag_Layouts.GetTagCategory(haloversion).GetTagDefinitionByExtension(str2).TagType;
                            }
                        }
                        else if (obj2.Type == BaseObjectType.Syntax_Block)
                        {
                            Syntax_Block block2 = (Syntax_Block) obj2;
                            if (block2.Is_Tag_Referenced && (block2.Tag_Reference.Ident != -1))
                            {
                                ident = block2.Tag_Reference;
                                str = core.TagInterface.String.ReadString(ref BR, ident.Ident + 1);
                                num = str.LastIndexOf('.');
                                ident.TagName = str.Substring(0, num);
                                str2 = str.Substring(num + 1);
                                ident.TagType = Tag_Layouts.GetTagCategory(haloversion).GetTagDefinitionByExtension(str2).TagType;
                            }
                        }
                    }
                }
            }
        }

        public virtual void ReadChunks(ref EndianIO.EndianReader BR, MetaInterface.InterfaceMode mode, Map.HaloVersionEnum haloversion)
        {
            for (int i = 0; i < this.ChunkCount; i++)
            {
                this.chunks[i].ReadFields(ref BR, mode);
            }
            if (mode == MetaInterface.InterfaceMode.DecompiledTagMode)
            {
                this.Read_Reference_Tag_Paths(ref BR, haloversion);
            }
        }

        public void RemoveAllChunks()
        {
            this.chunks.Clear();
            this.chunkCount = 0;
        }

        public void RemoveChunk(int index)
        {
            this.chunks.RemoveAt(index);
            for (int i = index; i < this.chunks.Count; i++)
            {
                this.chunks[i].Chunk_Number = i;
            }
            this.chunkCount--;
        }

        public void RemoveChunks(int index, int count)
        {
            this.chunks.RemoveRange(index, count);
            for (int i = index; i < this.chunks.Count; i++)
            {
                this.chunks[i].Chunk_Number = i;
            }
            this.chunkCount -= count;
        }

        public virtual void SetChunkCount(int count)
        {
            this.chunkCount = count;
        }

        public virtual void Write(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.chunkCount);
            int num = 0;
            if (this.ChunkCount == 0)
            {
                BW.Write(num);
            }
            else
            {
                this.translation.Write(ref BW);
            }
            if (this.HaloVersion != Map.HaloVersionEnum.Halo2)
            {
                BW.Write(num);
            }
        }

        private void Write_Reference_Tag_Paths(ref EndianIO.EndianWriter BW)
        {
            foreach (TagBlockLayout layout in this.chunks)
            {
                foreach (BaseObject obj2 in layout)
                {
                    string str;
                    if (obj2.Type == BaseObjectType.TagReference)
                    {
                        TagReference reference = (TagReference) obj2;
                        if (reference.Ident.TagName != "null")
                        {
                            str = Tag_Layouts.GetMetaPath(reference.Ident.TagName, reference.Ident.TagType, Map.HaloVersionEnum.Halo2);
                            core.TagInterface.String.WriteString(ref BW, str, str.Length + 1);
                        }
                    }
                    else
                    {
                        TagIdent ident;
                        if (obj2.Type == BaseObjectType.TagIdent)
                        {
                            ident = (TagIdent) obj2;
                            if (ident.TagName != "null")
                            {
                                str = Tag_Layouts.GetMetaPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                                core.TagInterface.String.WriteString(ref BW, str, str.Length + 1);
                            }
                        }
                        else if (obj2.Type == BaseObjectType.Raw_Tag_Block)
                        {
                            Raw_Tag_Block block = (Raw_Tag_Block) obj2;
                            ident = block.Tag_Ident;
                            if (ident.TagName != "null")
                            {
                                str = Tag_Layouts.GetMetaPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                                core.TagInterface.String.WriteString(ref BW, str, str.Length + 1);
                            }
                        }
                        else if (obj2.Type == BaseObjectType.Syntax_Block)
                        {
                            Syntax_Block block2 = (Syntax_Block) obj2;
                            if (block2.Is_Tag_Referenced && (block2.Tag_Reference.TagName != "null"))
                            {
                                ident = block2.Tag_Reference;
                                str = Tag_Layouts.GetMetaPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                                core.TagInterface.String.WriteString(ref BW, str, str.Length + 1);
                            }
                        }
                    }
                }
            }
        }

        public virtual void WriteChunks(ref EndianIO.EndianWriter BW, MetaInterface.InterfaceMode mode)
        {
            for (int i = 0; i < this.ChunkCount; i++)
            {
                this.chunks[i].WriteFields(ref BW, mode);
            }
            if (mode == MetaInterface.InterfaceMode.DecompiledTagMode)
            {
                this.Write_Reference_Tag_Paths(ref BW);
            }
        }

        [Category("Chunk Info")]
        public int ChunkCount
        {
            get
            {
                return this.chunkCount;
            }
        }

        [TypeConverter(typeof(MetaInterface.BaseObjectConverter)), Category("Chunk Info")]
        public TagBlockLayout ChunkInfo
        {
            get
            {
                return this.chunks[this.chunknumber];
            }
        }

        public ChunkCollection Chunks
        {
            get
            {
                return this.chunks;
            }
        }

        [Category("Chunk Info")]
        public int ChunkSize
        {
            get
            {
                return this.Layout.Chunk_Size;
            }
        }

        [Category("Chunk Info")]
        public int Current_Chunk_Number
        {
            get
            {
                return this.chunknumber;
            }
            set
            {
                this.chunknumber = value;
            }
        }

        [TypeConverter(typeof(MetaInterface.BaseObjectConverter))]
        public TagBlockLayout Layout
        {
            get
            {
                return this.layout;
            }
        }

        public int Padding_Interval
        {
            get
            {
                return this.paddinginterval;
            }
            set
            {
                this.paddinginterval = value;
            }
        }

        [TypeConverter(typeof(MetaInterface.BaseObjectConverter)), Category("Chunk Info")]
        public core.TagInterface.Pointer Translation
        {
            get
            {
                return this.translation;
            }
        }

        public class ChunkCollection : List<TagBlockLayout>
        {
            public TagBlockLayout this[int indexer]
            {
                get
                {
                    if ((indexer < 0) && (indexer > base.Count))
                    {
                        return null;
                    }
                    return base[indexer];
                }
                set
                {
                    if ((indexer >= 0) || (indexer <= base.Count))
                    {
                        base[indexer] = value;
                    }
                }
            }
        }
    }
}

