﻿namespace core.TagInterface
{
    using core.IO;
    using System;
    using System.ComponentModel;

    public class TagBlockIndex : BaseObject
    {
        public TagBlockIndexType IndexType;
        public int Value;

        public TagBlockIndex(string name, TagBlockIndexType type) : base(name)
        {
            this.IndexType = type;
            switch (type)
            {
                case TagBlockIndexType.Word:
                    base.size = 2;
                    break;

                case TagBlockIndexType.Long:
                    base.size = 4;
                    break;

                case TagBlockIndexType.Byte:
                    base.size = 1;
                    break;
            }
            base.type = BaseObjectType.TagBlockIndex;
        }

        public override BaseObject Clone()
        {
            TagBlockIndex index = new TagBlockIndex(base.name, this.IndexType);
            index.Value = this.Value;
            index.SetMetaOffset(base.MetaOffset);
            index.SetOffset(base.Offset);
            index.SetMapOffset(base.MapOffset);
            index.Name = base.Name;
            index.SetType(base.Type);
            return index;
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            switch (this.IndexType)
            {
                case TagBlockIndexType.Word:
                    this.Value = BR.ReadInt16();
                    break;

                case TagBlockIndexType.Long:
                    this.Value = BR.ReadInt32();
                    break;

                case TagBlockIndexType.Byte:
                    this.Value = BR.ReadByte();
                    break;
            }
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            switch (this.IndexType)
            {
                case TagBlockIndexType.Word:
                    BW.Write((short) this.Value);
                    break;

                case TagBlockIndexType.Long:
                    BW.Write(this.Value);
                    break;

                case TagBlockIndexType.Byte:
                    BW.Write((byte) this.Value);
                    break;
            }
        }

        [Category("Values")]
        public int Index
        {
            get
            {
                return this.Value;
            }
            set
            {
                this.Value = value;
            }
        }

        public enum TagBlockIndexType
        {
            Word,
            Long,
            Byte
        }
    }
}

