﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.Collections.Generic;
    using System.Reflection;

    public class TagBlockLayout : List<BaseObject>
    {
        protected int ChunkNumber;
        protected int ChunkSize;
        protected bool is_hard_coded;
        private TagBlock parent;

        public TagBlockLayout()
        {
            this.is_hard_coded = true;
        }

        public TagBlockLayout(bool hard_coded)
        {
            this.is_hard_coded = true;
            this.is_hard_coded = hard_coded;
        }

        public void Add(BaseObject obj)
        {
            obj.SetOffset(this.ChunkSize);
            obj.SetParent(this.Parent);
            base.Add(obj);
            this.ChunkSize += obj.Size;
        }

        public TagBlockLayout Clone()
        {
            BaseObject obj3;
            TagBlockLayout layout = new TagBlockLayout();
            if (this.is_hard_coded)
            {
                layout = (TagBlockLayout) Activator.CreateInstance(base.GetType());
                layout.Chunk_Number = this.Chunk_Number;
                Type type = layout.GetType();
                FieldInfo[] fields = type.GetFields(BindingFlags.Public | BindingFlags.Instance);
                FieldInfo[] infoArray2 = type.GetFields(BindingFlags.NonPublic | BindingFlags.Instance);
                int index = 0;
                foreach (FieldInfo info in base.GetType().GetFields(BindingFlags.Public | BindingFlags.Instance))
                {
                    BaseObject obj2 = (BaseObject) info.GetValue(this);
                    fields[index].SetValue(layout, obj2.Clone());
                    obj3 = (BaseObject) fields[index].GetValue(layout);
                    layout[index] = obj3;
                    index++;
                }
                return layout;
            }
            layout.Is_Hard_Coded = false;
            foreach (BaseObject obj2 in this)
            {
                obj3 = obj2.Clone();
                layout.Add(obj3);
            }
            return layout;
        }

        public TagBlockLayout New()
        {
            TagBlockLayout layout = new TagBlockLayout();
            if (this.is_hard_coded)
            {
                return (TagBlockLayout) Activator.CreateInstance(base.GetType());
            }
            layout.Is_Hard_Coded = false;
            foreach (BaseObject obj2 in this)
            {
                layout.Add(obj2.Clone());
            }
            return layout;
        }

        public virtual void ReadFields(ref EndianIO.EndianReader BR, MetaInterface.InterfaceMode mode)
        {
            foreach (BaseObject obj2 in this)
            {
                obj2.SetParent(this.Parent);
                switch (obj2.Type)
                {
                    case BaseObjectType.RealPoint2D:
                        ((RealPoint2D) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealPlane2D:
                        ((RealPlane2D) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealPlane3D:
                        ((RealPlane3D) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealQuaternion:
                        ((RealQuaternion) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealPoint3D:
                        ((RealPoint3D) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealVector3D:
                        ((RealVector3D) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Angle:
                        ((Angle) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealEulerAngles3D:
                        ((RealEulerAngles3D) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Flags:
                        ((core.TagInterface.Flags) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealBounds:
                        ((RealBounds) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealRGBColor:
                        ((RealRGBColor) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealARGBColor:
                        ((RealARGBColor) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealRGBAColor:
                        ((RealRGBAColor) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RGBAColor:
                        ((RGBAColor) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Data:
                        ((Data) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Animation_Raw:
                        ((Data) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Sound_Raw:
                        ((Data) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Bitmap_Raw:
                        ((Bitmap_Raw) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Unicode_Raw:
                        ((Unicode_Raw) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RawDataOffset:
                        ((RawDataOffset) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Raw_Tag_Block:
                        ((Raw_Tag_Block) obj2).Read(ref BR, mode);
                        break;

                    case BaseObjectType.TagEnumerator:
                        ((core.TagInterface.Enumerator)obj2).Read(ref BR);
                        break;

                    case BaseObjectType.EnumerableIndex:
                        ((EnumerableIndex) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.TagBlockIndex:
                        ((TagBlockIndex) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.LongInteger:
                        ((LongInteger) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.ShortInteger:
                        ((ShortInteger) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.ByteInteger:
                        ((ByteInteger) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealFloat:
                        ((RealFloat) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.RealFraction:
                        ((RealFraction) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Padding:
                        ((Padding) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.String:
                        ((core.TagInterface.String) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.StringID:
                        ((core.TagInterface.StringID) obj2).Read(ref BR, mode);
                        break;

                    case BaseObjectType.TagBlock:
                        ((TagBlock) obj2).Read(ref BR, mode);
                        break;

                    case BaseObjectType.TagReference:
                        ((TagReference) obj2).Read(ref BR, mode);
                        break;

                    case BaseObjectType.TagIdent:
                        ((TagIdent) obj2).Read(ref BR, mode);
                        break;

                    case BaseObjectType.HaloTagType:
                        ((HaloTagType) obj2).Read(ref BR);
                        break;

                    case BaseObjectType.Syntax_Block:
                        ((Syntax_Block) obj2).Read(ref BR, mode);
                        break;

                    case BaseObjectType.Model_Bounds:
                        ((Model_Bounds) obj2).Read(ref BR);
                        break;
                }
            }
        }

        public void RemoveAt(int index)
        {
            this.ChunkSize -= base[index].Size;
            base.RemoveAt(index);
        }

        public virtual void WriteFields(ref EndianIO.EndianWriter BW, MetaInterface.InterfaceMode mode)
        {
            foreach (BaseObject obj2 in this)
            {
                if (mode == MetaInterface.InterfaceMode.CompiledTagMode)
                {
                    obj2.SetMetaOffset((int) BW.BaseStream.Position);
                }
                switch (obj2.Type)
                {
                    case BaseObjectType.RealPoint2D:
                        ((RealPoint2D) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealPlane2D:
                        ((RealPlane2D) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealPlane3D:
                        ((RealPlane3D) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealQuaternion:
                        ((RealQuaternion) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealPoint3D:
                        ((RealPoint3D) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealVector3D:
                        ((RealVector3D) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Angle:
                        ((Angle) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealEulerAngles3D:
                        ((RealEulerAngles3D) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Flags:
                        ((core.TagInterface.Flags) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealBounds:
                        ((RealBounds) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealRGBColor:
                        ((RealRGBColor) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealARGBColor:
                        ((RealARGBColor) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealRGBAColor:
                        ((RealRGBAColor) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RGBAColor:
                        ((RGBAColor) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Data:
                        ((Data) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Animation_Raw:
                        ((Data) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Sound_Raw:
                        ((Data) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Bitmap_Raw:
                        ((Bitmap_Raw) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Unicode_Raw:
                        ((Unicode_Raw) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RawDataOffset:
                        ((RawDataOffset) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Raw_Tag_Block:
                        ((Raw_Tag_Block) obj2).Write(ref BW, mode);
                        break;

                    case BaseObjectType.TagEnumerator:
                        ((core.TagInterface.Enumerator)obj2).Write(ref BW);
                        break;

                    case BaseObjectType.EnumerableIndex:
                        ((EnumerableIndex) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.TagBlockIndex:
                        ((TagBlockIndex) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.LongInteger:
                        ((LongInteger) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.ShortInteger:
                        ((ShortInteger) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.ByteInteger:
                        ((ByteInteger) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealFloat:
                        ((RealFloat) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.RealFraction:
                        ((RealFraction) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Padding:
                        ((Padding) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.String:
                        ((core.TagInterface.String) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.StringID:
                        ((core.TagInterface.StringID) obj2).Write(ref BW, mode);
                        break;

                    case BaseObjectType.TagBlock:
                        ((TagBlock) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.TagReference:
                        ((TagReference) obj2).Write(ref BW, mode);
                        break;

                    case BaseObjectType.TagIdent:
                        ((TagIdent) obj2).Write(ref BW, mode);
                        break;

                    case BaseObjectType.HaloTagType:
                        ((HaloTagType) obj2).Write(ref BW);
                        break;

                    case BaseObjectType.Syntax_Block:
                        ((Syntax_Block) obj2).Write(ref BW, mode);
                        break;

                    case BaseObjectType.Model_Bounds:
                        ((Model_Bounds) obj2).Write(ref BW);
                        break;
                }
            }
        }

        public int Chunk_Number
        {
            get
            {
                return this.ChunkNumber;
            }
            set
            {
                this.ChunkNumber = value;
            }
        }

        public int Chunk_Size
        {
            get
            {
                return this.ChunkSize;
            }
        }

        public bool Is_Hard_Coded
        {
            get
            {
                return this.is_hard_coded;
            }
            set
            {
                this.is_hard_coded = value;
            }
        }

        public BaseObject this[int indexer]
        {
            get
            {
                return base[indexer];
            }
            set
            {
                base[indexer] = value;
            }
        }

        public TagBlock Parent
        {
            get
            {
                return this.parent;
            }
            set
            {
                this.parent = value;
            }
        }
    }
}

