﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;
    using System.Windows.Forms;

    [TypeConverter(typeof(MetaInterface.BaseObjectConverter))]
    public class TagIdent : BaseObject
    {
        protected int ident;
        protected string tagName;
        protected int tagNumber;
        protected string tagType;

        public TagIdent(string name) : base(name)
        {
            this.tagNumber = -1;
            this.tagType = "null";
            this.tagName = "null";
            base.size = 4;
            base.type = BaseObjectType.TagIdent;
        }

        public override BaseObject Clone()
        {
            TagIdent ident = new TagIdent(base.name);
            ident.ident = this.ident;
            ident.tagNumber = this.tagNumber;
            ident.tagType = this.tagType;
            ident.tagName = this.tagName;
            ident.SetMetaOffset(base.MetaOffset);
            ident.SetOffset(base.Offset);
            ident.SetMapOffset(base.MapOffset);
            ident.Name = base.Name;
            ident.SetType(base.Type);
            return ident;
        }

        public string GetTagPath()
        {
            return Tag_Layouts.GetMetaPath(this.TagName, this.TagType, Map.HaloVersionEnum.Halo2);
        }

        public void Load(Map map)
        {
            if ((this.ident != -1) && (this.ident != 0))
            {
                this.tagNumber = map.FindMetaByID(this.ident);
                if (this.tagNumber != -1)
                {
                    this.ident = map.Meta[this.TagNumber].Ident;
                    this.tagName = map.FileName[this.tagNumber].Name;
                    this.tagType = map.Meta[this.TagNumber].TagType;
                }
                else
                {
                    string text = "Broken Tag Reference(" + base.name + ")";
                    int num = map.FindMetaByOffset(base.mapOffset);
                    if (num != -1)
                    {
                        string str2 = text;
                        text = str2 + "in " + map.Meta[num].TagType + " - " + map.FileName[num].Name + " at " + base.MetaOffset.ToString("X");
                    }
                    MessageBox.Show(text);
                    this.ident = -1;
                    this.tagName = "null";
                    this.tagType = "null";
                }
            }
        }

        public void Load(Map map, string tagtype, string name)
        {
            this.tagNumber = map.FindByNameAndTagType(tagtype, name);
            if (this.tagNumber != -1)
            {
                this.ident = map.Meta[this.TagNumber].Ident;
                this.tagName = name;
                this.tagType = tagtype;
            }
            else
            {
                this.ident = -1;
                this.tagName = "null";
                this.tagType = "null";
            }
        }

        public void Read(ref EndianIO.EndianReader BR, MetaInterface.InterfaceMode mode)
        {
            this.ident = BR.ReadInt32();
        }

        public void Write(ref EndianIO.EndianWriter BW, MetaInterface.InterfaceMode mode)
        {
            if (mode != MetaInterface.InterfaceMode.CompiledTagMode)
            {
                if (this.tagName != "null")
                {
                    string str = Tag_Layouts.GetMetaPath(this.TagName, this.TagType, Map.HaloVersionEnum.Halo2);
                    BW.Write(str.Length);
                }
                else
                {
                    BW.Write(-1);
                }
            }
            else
            {
                BW.Write(this.ident);
            }
        }

        public int Ident
        {
            get
            {
                return this.ident;
            }
            set
            {
                this.ident = value;
            }
        }

        [TypeConverter(typeof(MetaInterface.TagReferenceConverter))]
        public string TagName
        {
            get
            {
                return this.tagName;
            }
            set
            {
                this.tagName = value;
            }
        }

        public int TagNumber
        {
            get
            {
                return this.tagNumber;
            }
        }

        [TypeConverter(typeof(MetaInterface.TagTypeConverter))]
        public string TagType
        {
            get
            {
                return this.tagType;
            }
            set
            {
                this.tagType = value;
            }
        }
    }
}

