﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;
    using System.IO;

    public class Unicode_Raw : BaseObject
    {
        public int Index_Offset;
        protected MemoryStream ms;
        public int String_Count;
        public int Table_Offset;
        public int Table_Size;
        private TagBlock unicode_strings;
        public int unknown;
        public int unknown1;
        public int unknown2;

        public Unicode_Raw(string name) : base(name)
        {
            base.size = 0x1c;
            base.type = BaseObjectType.Unicode_Raw;
        }

        public override BaseObject Clone()
        {
            Unicode_Raw raw = new Unicode_Raw(base.name);
            raw.String_Count = this.String_Count;
            raw.Table_Size = this.Table_Size;
            raw.Index_Offset = this.Index_Offset;
            raw.Table_Offset = this.Table_Offset;
            raw.unknown = this.unknown;
            raw.unknown1 = this.unknown1;
            raw.unknown2 = this.unknown2;
            if (raw.unicode_strings != null)
            {
                raw.unicode_strings = (TagBlock) this.unicode_strings.Clone();
            }
            if (this.ms != null)
            {
                raw.ms.Write(this.ms.ToArray(), 0, this.Table_Size);
            }
            raw.SetMetaOffset(base.MetaOffset);
            raw.SetOffset(base.Offset);
            raw.SetMapOffset(base.MapOffset);
            raw.Name = base.Name;
            raw.SetType(base.Type);
            return raw;
        }

        public virtual void Read(ref EndianIO.EndianReader BR)
        {
            this.String_Count = BR.ReadInt32();
            this.Table_Size = BR.ReadInt32();
            this.Index_Offset = BR.ReadInt32();
            this.Table_Offset = BR.ReadInt32();
            this.unknown = BR.ReadInt32();
            this.unknown1 = BR.ReadInt32();
            this.unknown2 = BR.ReadInt32();
            this.unicode_strings = new TagBlock("Unicode", Unicode_String.Layout, this.String_Count, Map.HaloVersionEnum.Halo2);
        }

        public virtual void Read_Raw(ref EndianIO.EndianReader BR)
        {
            this.ms = new MemoryStream(this.Table_Size);
            this.ms.Write(BR.ReadBytes(this.Table_Size), 0, this.Table_Size);
        }

        public virtual void Write(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.String_Count);
            BW.Write(this.Table_Size);
            BW.Write(this.Index_Offset);
            BW.Write(this.Table_Offset);
            BW.Write(this.unknown);
            BW.Write(this.unknown1);
            BW.Write(this.unknown2);
        }

        public virtual void Write_Raw(ref EndianIO.EndianWriter BW)
        {
            BW.Write(this.ms.ToArray());
        }

        public MemoryStream MS
        {
            get
            {
                return this.ms;
            }
            set
            {
                this.ms = value;
            }
        }

        [TypeConverter(typeof(MetaInterface.BaseObjectConverter))]
        public TagBlock Unicode_Strings
        {
            get
            {
                return this.unicode_strings;
            }
            set
            {
                this.unicode_strings = value;
            }
        }

        public class Unicode_String : TagBlockLayout
        {
            public static readonly Unicode_Raw.Unicode_String Layout = new Unicode_Raw.Unicode_String();
            public core.TagInterface.StringID name = new core.TagInterface.StringID("Name");
            public LongInteger string_offset = new LongInteger("String Offset");

            public Unicode_String()
            {
                base.Add(this.name);
                base.Add(this.string_offset);
            }
        }
    }
}

