﻿namespace core
{
    using core.TagInterface;
    using System;
    using System.ComponentModel;
    using System.Drawing;
    using System.Windows.Forms;

    public sealed class Tag_Layout_Manager : Form
    {
        private Button button1;
        private ComboBox comboBox1;
        private IContainer components = null;
        private Button exportcs;
        public static Tag_Layout_Manager Instance;
        private Layout_Control layout_Control1;
        private Layout_Control layout_Control2;
        private FlowLayoutPanel panel;

        public Tag_Layout_Manager()
        {
            this.InitializeComponent();
            this.AddTagCategories();
            this.comboBox1.SelectedIndex = 0;
        }

        private void AddTagCategories()
        {
            foreach (Tag_Category category in Tag_Layouts.Layouts)
            {
                this.comboBox1.Items.Add(category.HaloVersion.ToString());
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            if (dialog.ShowDialog() != DialogResult.Cancel)
            {
                dialog.SelectedPath = dialog.SelectedPath + @"\";
                foreach (Tag_Category category in Tag_Layouts.Layouts)
                {
                    if (category.HaloVersion.ToString() == this.comboBox1.Text)
                    {
                        foreach (TagDefinition definition in category)
                        {
                            EntIO.WriteXMLEnt(dialog.SelectedPath + definition.TagType.Replace("<", "_").Replace(">", "_") + ".ent", definition.Layout);
                        }
                    }
                }
                MessageBox.Show("Done");
            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            foreach (Tag_Category category in Tag_Layouts.Layouts)
            {
                if (category.HaloVersion.ToString() == this.comboBox1.Text)
                {
                    this.panel.Controls.Clear();
                    foreach (TagDefinition definition in category)
                    {
                        this.panel.Controls.Add(new Layout_Control(definition));
                        Application.DoEvents();
                    }
                }
            }
        }

        public static void Display()
        {
            Instance = new Tag_Layout_Manager();
            Instance.ShowDialog();
        }

        private void DisplayLayouts(Map.HaloVersionEnum halo_version)
        {
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void exportcs_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            if (dialog.ShowDialog() != DialogResult.Cancel)
            {
                dialog.SelectedPath = dialog.SelectedPath + @"\";
                foreach (Tag_Category category in Tag_Layouts.Layouts)
                {
                    if (category.HaloVersion.ToString() == this.comboBox1.Text)
                    {
                        foreach (TagDefinition definition in category)
                        {
                            EntIO.WriteCSClassEnt(dialog.SelectedPath + definition.TagType.Replace("<", "_").Replace(">", "_") + ".cs", definition.Layout, definition.TagType);
                        }
                    }
                }
                MessageBox.Show("Done");
            }
        }

        private void InitializeComponent()
        {
            this.panel = new FlowLayoutPanel();
            this.layout_Control1 = new Layout_Control();
            this.layout_Control2 = new Layout_Control();
            this.comboBox1 = new ComboBox();
            this.exportcs = new Button();
            this.button1 = new Button();
            this.panel.SuspendLayout();
            base.SuspendLayout();
            this.panel.AutoScroll = true;
            this.panel.BorderStyle = BorderStyle.FixedSingle;
            this.panel.Controls.Add(this.layout_Control1);
            this.panel.Controls.Add(this.layout_Control2);
            this.panel.Location = new Point(12, 0x27);
            this.panel.Name = "panel";
            this.panel.Size = new Size(0x1c1, 0x120);
            this.panel.TabIndex = 0;
            this.layout_Control1.BackColor = Color.DarkSlateGray;
            this.layout_Control1.BorderStyle = BorderStyle.FixedSingle;
            this.layout_Control1.ForeColor = Color.Silver;
            this.layout_Control1.Location = new Point(3, 3);
            this.layout_Control1.Name = "layout_Control1";
            this.layout_Control1.Size = new Size(0x194, 0x51);
            this.layout_Control1.TabIndex = 0;
            this.layout_Control2.BackColor = Color.DarkSlateGray;
            this.layout_Control2.BorderStyle = BorderStyle.FixedSingle;
            this.layout_Control2.ForeColor = Color.Silver;
            this.layout_Control2.Location = new Point(3, 90);
            this.layout_Control2.Name = "layout_Control2";
            this.layout_Control2.Size = new Size(0x194, 0x51);
            this.layout_Control2.TabIndex = 1;
            this.comboBox1.BackColor = Color.White;
            this.comboBox1.ForeColor = Color.DarkGray;
            this.comboBox1.FormattingEnabled = true;
            this.comboBox1.Location = new Point(12, 12);
            this.comboBox1.Name = "comboBox1";
            this.comboBox1.Size = new Size(0xa9, 0x15);
            this.comboBox1.TabIndex = 1;
            this.comboBox1.SelectedIndexChanged += new EventHandler(this.comboBox1_SelectedIndexChanged);
            this.exportcs.Location = new Point(0xe9, 10);
            this.exportcs.Name = "exportcs";
            this.exportcs.Size = new Size(0x6f, 0x17);
            this.exportcs.TabIndex = 2;
            this.exportcs.Text = "Export To C# Class";
            this.exportcs.UseVisualStyleBackColor = true;
            this.exportcs.Click += new EventHandler(this.exportcs_Click);
            this.button1.Location = new Point(350, 10);
            this.button1.Name = "button1";
            this.button1.Size = new Size(0x6f, 0x17);
            this.button1.TabIndex = 3;
            this.button1.Text = "Export To .ent";
            this.button1.UseVisualStyleBackColor = true;
            this.button1.Click += new EventHandler(this.button1_Click);
            base.AutoScaleDimensions = new SizeF(6f, 13f);
            base.AutoScaleMode = AutoScaleMode.Font;
            this.BackColor = Color.LightSlateGray;
            base.ClientSize = new Size(0x1d9, 460);
            base.Controls.Add(this.button1);
            base.Controls.Add(this.exportcs);
            base.Controls.Add(this.comboBox1);
            base.Controls.Add(this.panel);
            base.FormBorderStyle = FormBorderStyle.FixedToolWindow;
            base.Name = "Tag_Layout_Manager";
            this.Text = "Tag Layout Manager";
            base.Load += new EventHandler(this.Tag_Layout_Manager_Load);
            this.panel.ResumeLayout(false);
            base.ResumeLayout(false);
        }

        private void Tag_Layout_Manager_Load(object sender, EventArgs e)
        {
        }
    }
}

