﻿namespace core.Builder
{
    using core;
    using core.H2;
    using core.TagInterface;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.IO;
    using System.Windows.Forms;

    public class Builder : Form
    {
        private List<string> addedfilepaths;
        private Button browseforscenario;
        private IContainer components = null;
        private RichTextBox debugbox;
        private List<int> idents;
        private Label label1;
        private Map map;
        private List<MetaInterface> metas;
        private CheckBox overwrite;
        private List<bool> overwritten;
        private Button RebuildButton;
        private TextBox scenariopath;
        private List<string> stringids;
        private uint tempident;

        public Builder(Map map)
        {
            this.InitializeComponent();
            this.map = map;
        }

        private void browseforscenario_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.InitialDirectory = ProgramSettings.DecompiledTagsPath;
            if (dialog.ShowDialog() != DialogResult.Cancel)
            {
                this.scenariopath.Text = dialog.FileName;
            }
        }

        public void Build(List<MetaInterface> metalist, bool overwrite)
        {
            int num;
            string str;
            MetaInterface current;
            snd_ chunk;
            List<int> list2;
            List<int> list3;
            int num9;
            List<MetaInterface>.Enumerator TagEnumerator2;
            this.addedfilepaths = new List<string>();
            this.metas = new List<MetaInterface>();
            this.idents = new List<int>();
            this.stringids = new List<string>();
            this.overwritten = new List<bool>();
            foreach (Map.StringID gid in this.map.strings)
            {
                this.stringids.Add(gid.Name);
            }
            this.tempident = 0xe1740000;
            this.debugbox.Text = "Starting rebuild...\n";
            Application.DoEvents();
            ugh_ h_ = this.map.ugh_Interface;
            for (num = 0; num < (this.map.Meta.Count - 1); num++)
            {
                str = Tag_Layouts.GetFullTagPath(this.map.FileName[num].Name, this.map.Meta[num].TagType, Map.HaloVersionEnum.Halo2);
                this.debugbox.Text = "Processing " + str + "...\n";
                Application.DoEvents();
                Meta meta = new Meta(num, ref this.map);
                this.metas.Add(meta.Interface);
                this.addedfilepaths.Add(Tag_Layouts.GetFullTagPath(meta.Name, meta.TagType, Map.HaloVersionEnum.Halo2));
                this.idents.Add((int) this.tempident);
                this.tempident += 0x10001;
                this.overwritten.Add(false);
            }
            using (TagEnumerator2 = metalist.GetEnumerator())
            {
                while (TagEnumerator2.MoveNext())
                {
                    current = TagEnumerator2.Current;
                    str = Tag_Layouts.GetFullTagPath(current.Tag_Name, current.Tag_Type, Map.HaloVersionEnum.Halo2);
                    int index = this.addedfilepaths.IndexOf(str);
                    if (index == -1)
                    {
                        this.metas.Add(current);
                        this.addedfilepaths.Add(str);
                        this.idents.Add((int) this.tempident);
                        this.tempident += 0x10001;
                        this.overwritten.Add(true);
                    }
                    else if (!(this.overwritten[index] || !overwrite))
                    {
                        this.overwritten[index] = true;
                        this.metas[index] = current;
                    }
                }
            }
            matg matg = (matg) this.metas[0].Header.Chunks[0];
            scnr scnr = (scnr) this.metas[3].Header.Chunks[0];
            Application.DoEvents();
            this.metas.Add(this.map.ugh_Meta.Interface);
            this.addedfilepaths.Add(Tag_Layouts.GetFullTagPath(this.map.ugh_Meta.Name, this.map.ugh_Meta.TagType, Map.HaloVersionEnum.Halo2));
            this.idents.Add((int) this.tempident);
            this.tempident += 0x10001;
            Application.DoEvents();
            scnr.Predicted_Resources_0.RemoveAllChunks();
            using (TagEnumerator2 = this.metas.GetEnumerator())
            {
                while (TagEnumerator2.MoveNext())
                {
                    current = TagEnumerator2.Current;
                    switch (current.Tag_Type)
                    {
                        case "bipd":
                        case "bloc":
                        case "ctrl":
                        case "jpt!":
                        case "mach":
                        case "scen":
                        case "ssce":
                        case "vehi":
                        {
                            scnr.Predicted_Resources_0_Layout tagblocklayout = new scnr.Predicted_Resources_0_Layout();
                            tagblocklayout.Resource.TagName = current.Tag_Name;
                            tagblocklayout.Resource.TagType = current.Tag_Type;
                            scnr.Predicted_Resources_0.AddChunk(tagblocklayout);
                            break;
                        }
                        case "eqip":
                        case "garb":
                        case "proj":
                        {
                            scnr.Predicted_Resources_0_Layout layout2 = new scnr.Predicted_Resources_0_Layout();
                            layout2.Resource.TagName = current.Tag_Name;
                            layout2.Resource.TagType = current.Tag_Type;
                            scnr.Predicted_Resources_0.AddChunk(layout2);
                            scnr.Predicted_Resources_0.AddChunk(layout2.Clone());
                            break;
                        }
                        case "weap":
                        {
                            scnr.Predicted_Resources_0_Layout layout3 = new scnr.Predicted_Resources_0_Layout();
                            layout3.Resource.TagName = current.Tag_Name;
                            layout3.Resource.TagType = current.Tag_Type;
                            scnr.Predicted_Resources_0.AddChunk(layout3);
                            scnr.Predicted_Resources_0.AddChunk(layout3.Clone());
                            scnr.Predicted_Resources_0.AddChunk(layout3.Clone());
                            break;
                        }
                    }
                }
            }
            List<Sound_Resource_Layout> list = new List<Sound_Resource_Layout>();
            for (num = 0; num < this.metas.Count; num++)
            {
                current = this.metas[num];
                if (current.Tag_Type == "snd!")
                {
                    chunk = (snd_) current.Header.GetChunk(0);
                    list.Add(chunk.Sound_Resource);
                }
            }
            h_.Sound_Permutations35.RemoveAllChunks();
            h_.Sound_Choices44.RemoveAllChunks();
            h_.Sound_Raw_Data68.RemoveAllChunks();
            h_.Sound_Names25.RemoveAllChunks();
            int num3 = 0;
            num = 0;
            while (num < this.metas.Count)
            {
                current = this.metas[num];
                if (current.Tag_Type == "snd!")
                {
                    chunk = (snd_) current.Header.GetChunk(0);
                    chunk.Index_to_5th.Value = (short) h_.Sound_Permutations35.ChunkCount;
                    Sound_Resource_Layout layout4 = list[num3];
                    num3++;
                    for (int i = 0; i < layout4.Sound_Permutations.ChunkCount; i++)
                    {
                        ugh_.Sound_Permutations35_Layout layout5 = (ugh_.Sound_Permutations35_Layout) layout4.Sound_Permutations.Chunks[i];
                        int chunkCount = h_.Sound_Choices44.ChunkCount;
                        for (int j = 0; j < layout5.chunkCount.Value; j++)
                        {
                            ugh_.Sound_Choices44_Layout layout6 = (ugh_.Sound_Choices44_Layout) layout4.Sound_Choices.Chunks[layout5.Index.Value + j];
                            int num7 = h_.Sound_Raw_Data68.ChunkCount;
                            for (int k = 0; k < layout6.chunkCount.Value; k++)
                            {
                                h_.Sound_Raw_Data68.AddChunk(layout4.Sound_Raw_Data.Chunks[layout6.Index.Value + k]);
                            }
                            bool flag = false;
                            ugh_.Sound_Names25_Layout layout7 = (ugh_.Sound_Names25_Layout) layout4.Sound_Names.Chunks[layout6.Name_Index.Value];
                            foreach (ugh_.Sound_Names25_Layout layout8 in h_.Sound_Names25.Chunks)
                            {
                                if (layout7.Name.Value == layout8.Name.Value)
                                {
                                    layout6.Name_Index.Value = (short) layout8.Chunk_Number;
                                    flag = true;
                                    break;
                                }
                            }
                            if (!flag)
                            {
                                h_.Sound_Names25.AddChunk(layout7);
                                layout6.Name_Index.Value = (short) (h_.Sound_Names25.ChunkCount - 1);
                            }
                            layout6.Index.Value = (short) num7;
                            h_.Sound_Choices44.AddChunk(layout6);
                        }
                        layout5.Index.Value = (short) chunkCount;
                        h_.Sound_Permutations35.AddChunk(layout5);
                    }
                }
                num++;
            }
            this.FixTagReferences();
            this.map.io.Open();
            this.map.io.In.SeekTo(this.map.Map_Header.GeometryCacheOffset);
            byte[] buffer = this.map.io.In.ReadBytes(this.map.Map_Header.GeometryCacheSize);
            this.map.io.In.SeekTo(this.map.Map_Header.IndexOffset);
            byte[] buffer2 = this.map.io.In.ReadBytes(this.map.Map_Header.Index_HeaderSize);
            this.map.io.Out.BaseStream.SetLength(0x800L);
            this.map.io.Out.SeekTo(0x800);
            this.debugbox.Text = this.debugbox.Text + "Writing internal sound raw\n";
            foreach (ugh_.Sound_Raw_Data68_Layout layout9 in h_.Sound_Raw_Data68.Chunks)
            {
                if (layout9.Sound_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                {
                    Application.DoEvents();
                    layout9.Sound_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                    layout9.Sound_Raw.Write_Raw(ref this.map.io.Out);
                    this.WritePadding();
                }
            }
            foreach (MetaInterface interface2 in this.metas)
            {
                if (interface2.Tag_Type == "mode")
                {
                    mode mode = (mode) interface2.Header.GetChunk(0);
                    list2 = new List<int>();
                    list3 = new List<int>();
                    this.debugbox.Text = this.debugbox.Text + "Writing internal model type1 raw for " + interface2.Tag_Name + "\n";
                    Application.DoEvents();
                    foreach (mode.Part_Layout layout10 in mode.Part.Chunks)
                    {
                        if (layout10.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            num9 = list2.IndexOf(layout10.Visual_Raw.Raw_Offset.Data_Offset);
                            if (num9 == -1)
                            {
                                list2.Add(layout10.Visual_Raw.Raw_Offset.Data_Offset);
                                layout10.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                list3.Add(layout10.Visual_Raw.Raw_Offset.Data_Offset);
                                layout10.Visual_Raw.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                            else if (num9 != -1)
                            {
                                layout10.Visual_Raw.Raw_Offset.Data_Offset = list3[num9];
                            }
                        }
                    }
                    this.debugbox.Text = this.debugbox.Text + "Writing internal model type2 raw for " + interface2.Tag_Name + "\n";
                    Application.DoEvents();
                    foreach (mode.Unknown_Raw_Data_Layout layout11 in mode.Unknown_5.Chunks)
                    {
                        if (layout11.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            num9 = list2.IndexOf(layout11.Visual_Raw.Raw_Offset.Data_Offset);
                            if (num9 == -1)
                            {
                                list2.Add(layout11.Visual_Raw.Raw_Offset.Data_Offset);
                                layout11.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                list3.Add(layout11.Visual_Raw.Raw_Offset.Data_Offset);
                                layout11.Visual_Raw.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                            else if (num9 != -1)
                            {
                                layout11.Visual_Raw.Raw_Offset.Data_Offset = list3[num9];
                            }
                        }
                    }
                }
            }
            List<int> list4 = new List<int>();
            List<int> list5 = new List<int>();
            List<int> list6 = new List<int>();
            List<int> list7 = new List<int>();
            List<int> list8 = new List<int>();
            int num10 = 0;
            foreach (scnr.Structure_BSPS_Layout layout12 in scnr.Structure_BSPS.Chunks)
            {
                int num11 = 0;
                TagIdent ident = layout12.Light_Map_0;
                if (ident.TagName != "null")
                {
                    str = Tag_Layouts.GetFullTagPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                    num = this.addedfilepaths.IndexOf(str);
                    list8.Add(num);
                    num11 = num;
                }
                else
                {
                    list8.Add(-1);
                }
                int num12 = 0;
                ident = layout12.Structure_BSP_0;
                if (ident.TagName != "null")
                {
                    str = Tag_Layouts.GetFullTagPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                    num = this.addedfilepaths.IndexOf(str);
                    list7.Add(num);
                    num12 = num;
                }
                SBSP sbsp = (SBSP) this.metas[num12].Header.GetChunk(0);
                this.debugbox.Text = this.debugbox.Text + "Writing internal bsp cluster raw for " + ident.Name + "\n";
                Application.DoEvents();
                foreach (SBSP.Detail_Objects cluster in sbsp.detail_objects.Chunks)
                {
                    if (cluster.rawdata.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        cluster.rawdata.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        cluster.rawdata.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
                num10++;
                this.debugbox.Text = this.debugbox.Text + "Writing internal bsp scenery raw for " + ident.Name + "\n";
                Application.DoEvents();
                foreach (SBSP.Scenery_Cluster cluster2 in sbsp.bsp_permutations.Chunks)
                {
                    if (cluster2.rawdata.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        cluster2.rawdata.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        cluster2.rawdata.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
                if (num11 != -1)
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal lightmap raw for " + ident.Name + "\n";
                    Application.DoEvents();
                    ltmp ltmp = (ltmp) this.metas[num11].Header.Chunks[0];
                    foreach (ltmp.Unknown17_Layout layout13 in ltmp.Unknown17.Chunks)
                    {
                        foreach (ltmp.Unknown17_Layout.Lightmap_Raw_Layout layout14 in layout13.lightmap_raw.Chunks)
                        {
                            if (layout14.raw_data.Raw_Offset.Map_Location == MapTypes.Internal)
                            {
                                layout14.raw_data.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                layout14.raw_data.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                        }
                    }
                }
                list2 = new List<int>();
                list3 = new List<int>();
                this.debugbox.Text = this.debugbox.Text + "Writing internal unknown bsp raw for " + ident.Name + "\n";
                Application.DoEvents();
                foreach (SBSP.Decorators block in sbsp.decorators.Chunks)
                {
                    foreach (SBSP.Decorators.Raw_Data_Tag_Block block2 in block.raw_data_block.Chunks)
                    {
                        if (block2.rawdata.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            num9 = list2.IndexOf(block2.rawdata.Raw_Offset.Data_Offset);
                            if (num9 == -1)
                            {
                                list2.Add(block2.rawdata.Raw_Offset.Data_Offset);
                                block2.rawdata.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                list3.Add(block2.rawdata.Raw_Offset.Data_Offset);
                                block2.rawdata.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                            else
                            {
                                block2.rawdata.Raw_Offset.Data_Offset = list3[num9];
                            }
                        }
                    }
                }
                this.debugbox.Text = this.debugbox.Text + "Writing internal bsp raw for " + ident.Name + "\n";
                Application.DoEvents();
                foreach (SBSP.SBSP_Water water in sbsp.water_definitions.Chunks)
                {
                    if (water.rawdata.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        water.rawdata.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        water.rawdata.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
            }
            foreach (MetaInterface interface2 in this.metas)
            {
                if (interface2.Tag_Type == "weat")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal weather raw for " + interface2.Tag_Name + "\n";
                    Application.DoEvents();
                    weat weat = (weat) interface2.Header.GetChunk(0);
                    foreach (weat.Particle_Layout layout15 in weat.Particle.Chunks)
                    {
                        if (layout15.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            layout15.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                            layout15.Visual_Raw.Write_Raw(ref this.map.io.Out);
                            this.WritePadding();
                        }
                    }
                }
            }
            foreach (MetaInterface interface2 in this.metas)
            {
                if (interface2.Tag_Type == "DECR")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal DECR raw for " + interface2.Tag_Name + "\n";
                    Application.DoEvents();
                    DECR decr = (DECR) interface2.Header.GetChunk(0);
                    if (decr.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        decr.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        decr.Visual_Raw.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
            }
            foreach (MetaInterface interface2 in this.metas)
            {
                if (interface2.Tag_Type == "PRTM")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal PRTM raw for " + interface2.Tag_Name + "\n";
                    Application.DoEvents();
                    PRTM prtm = (PRTM) interface2.Header.GetChunk(0);
                    if (prtm.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        prtm.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        prtm.Visual_Raw.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
            }
            this.debugbox.Text = this.debugbox.Text + "Writing internal ugh raw type2 \n";
            Application.DoEvents();
            foreach (ugh_.Diff_Model_Raw_Data87_Layout layout16 in h_.Diff_Model_Raw_Data87.Chunks)
            {
                if (layout16.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                {
                    layout16.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                    layout16.Visual_Raw.Write_Raw(ref this.map.io.Out);
                    this.WritePadding();
                }
            }
            foreach (MetaInterface interface2 in this.metas)
            {
                if (interface2.Tag_Type == "jmad")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal animation raw for " + interface2.Tag_Name + "\n";
                    Application.DoEvents();
                    jmad jmad = (jmad) interface2.Header.GetChunk(0);
                    list2 = new List<int>();
                    list3 = new List<int>();
                    foreach (jmad.Unknown_11_Layout layout17 in jmad.Unknown_11.Chunks)
                    {
                        if (layout17.Raw_Data.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            num9 = list2.IndexOf(layout17.Raw_Data.Raw_Offset.Data_Offset);
                            if (num9 == -1)
                            {
                                list2.Add(layout17.Raw_Data.Raw_Offset.Data_Offset);
                                layout17.Raw_Data.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                list3.Add(layout17.Raw_Data.Raw_Offset.Data_Offset);
                                layout17.Raw_Data.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                            else if (num9 != -1)
                            {
                                layout17.Raw_Data.Raw_Offset.Data_Offset = list3[num9];
                            }
                        }
                    }
                }
            }
            for (num = 0; num < list7.Count; num++)
            {
                Meta meta2;
                int item = (int) this.map.io.Out.BaseStream.Position;
                list5.Add(item);
                int num14 = (this.map.Index_Header.Constant - 0x20) + this.map.Map_Header.Index_HeaderSize;
                int num15 = num14 - item;
                int size = 0;
                if (list8[num] == -1)
                {
                    meta2 = TagCompiler.CompileSBSPMetaFromTagInterface(this.metas[list7[num]], item, num15, null);
                }
                else
                {
                    meta2 = TagCompiler.CompileSBSPMetaFromTagInterface(this.metas[list7[num]], item, num15, this.metas[list8[num]]);
                }
                this.map.io.Out.Write(meta2.MS.ToArray());
                size = meta2.Size;
                list4.Add(size);
                scnr.Structure_BSPS_Layout layout18 = (scnr.Structure_BSPS_Layout) scnr.Structure_BSPS.Chunks[num];
                layout18.Structure_BSP_Offset.Value = item;
                layout18.Structure_BSP_Size.Value = size;
                layout18.Primary_Magic_Meta_Offset.Value = num14;
            }
            int count = this.stringids.Count;
            int position = (int) this.map.io.Out.BaseStream.Position;
            for (num = 0; num < count; num++)
            {
                core.TagInterface.String.WriteString(ref this.map.io.Out, this.stringids[num], 0x80);
            }
            this.WritePadding();
            int num19 = (int) this.map.io.Out.BaseStream.Position;
            int num20 = 0;
            for (num = 0; num < count; num++)
            {
                this.map.io.Out.Write(num20);
                num20 += this.stringids[num].Length + 1;
            }
            this.WritePadding();
            int num21 = (int) this.map.io.Out.BaseStream.Position;
            for (num = 0; num < count; num++)
            {
                core.TagInterface.String.WriteString(ref this.map.io.Out, this.stringids[num], this.stringids[num].Length + 1);
            }
            this.WritePadding();
            int num22 = (int) this.map.io.Out.BaseStream.Position;
            for (num = 0; num < this.metas.Count; num++)
            {
                core.TagInterface.String.WriteString(ref this.map.io.Out, this.metas[num].Tag_Name, this.metas[num].Tag_Name.Length + 1);
            }
            this.WritePadding();
            int num23 = (int) this.map.io.Out.BaseStream.Position;
            int num24 = 0;
            for (num = 0; num < this.metas.Count; num++)
            {
                this.map.io.Out.Write(num24);
                num24 += this.metas[num].Tag_Name.Length + 1;
            }
            this.WritePadding();
            num = 0;
            while (num < 8)
            {
                matg.Unicode[num].Index_Offset = (int) this.map.io.Out.BaseStream.Position;
                matg.Unicode[num].Unicode_Strings.WriteChunks(ref this.map.io.Out, MetaInterface.InterfaceMode.CompiledTagMode);
                this.WritePadding();
                matg.Unicode[num].Table_Offset = (int) this.map.io.Out.BaseStream.Position;
                matg.Unicode[num].Write_Raw(ref this.map.io.Out);
                this.WritePadding();
                num++;
            }
            matg.crazy_offset1.Value = (int) this.map.io.Out.BaseStream.Position;
            matg.crazy_offset2.Value = (int) this.map.io.Out.BaseStream.Position;
            this.map.io.Out.Write(buffer);
            this.WritePadding();
            foreach (MetaInterface interface2 in this.metas)
            {
                if (interface2.Tag_Type == "bitm")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal bitmap raw for " + interface2.Tag_Name + "\n";
                    Application.DoEvents();
                    bitm bitm = (bitm) interface2.Header.GetChunk(0);
                    foreach (bitm.Bitmap_Data_Layout layout19 in bitm.Bitmap_Data.Chunks)
                    {
                        num = 5;
                        while (num > -1)
                        {
                            if ((layout19.rawdata.lod_offset[num].Data_Offset != -1) && (layout19.rawdata.lod_offset[num].Map_Location == MapTypes.Internal))
                            {
                                layout19.rawdata.lod_offset[num].Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                this.map.io.Out.Write(layout19.rawdata.LOD_MS[num].ToArray());
                                this.WritePadding();
                            }
                            num--;
                        }
                    }
                }
            }
            int num25 = (int) this.map.io.Out.BaseStream.Position;
            int num26 = (this.map.Index_Header.Constant - num25) - 0x20;
            int magic = num26 + list4[0];
            this.map.io.Out.Write(buffer2);
            List<int> list9 = new List<int>();
            List<int> list10 = new List<int>();
            for (num = 0; num < this.metas.Count; num++)
            {
                if ((this.metas[num].Tag_Type == "ltmp") || (this.metas[num].Tag_Type == "sbsp"))
                {
                    list9.Add(0);
                    list10.Add(0);
                }
                else
                {
                    this.debugbox.Text = "Compiling " + this.metas[num].Tag_Name + "...\n";
                    Application.DoEvents();
                    try
                    {
                        int mapoffset = (int) this.map.io.Out.BaseStream.Position;
                        Meta meta3 = TagCompiler.CompileMetaFromTagInterface(this.metas[num], mapoffset, magic);
                        this.map.io.Out.Write(meta3.MS.ToArray());
                        list9.Add(mapoffset);
                        list10.Add(meta3.size);
                    }
                    catch
                    {
                        MessageBox.Show("Error Compiling");
                    }
                }
            }
            this.WritePadding();
            int num29 = this.map.Index_Header.Tags_Offset - this.map.Map_Header.IndexOffset;
            this.map.io.Out.SeekTo(num25 + num29);
            for (num = 0; num < this.metas.Count; num++)
            {
                char[] array = this.metas[num].Tag_Type.ToCharArray();
                Array.Reverse(array);
                this.map.io.Out.Write(array);
                this.map.io.Out.Write(this.idents[num]);
                int num30 = list9[num] + magic;
                if ((this.metas[num].Tag_Type == "ltmp") || (this.metas[num].Tag_Type == "sbsp"))
                {
                    this.map.io.Out.Write(0);
                    this.map.io.Out.Write(0);
                }
                else
                {
                    this.map.io.Out.Write(num30);
                    this.map.io.Out.Write(list10[num]);
                }
            }
            int length = (int) this.map.io.Out.BaseStream.Length;
            this.map.io.Out.SeekTo(8);
            this.map.io.Out.Write(length);
            this.map.io.Out.SeekTo(0x10);
            this.map.io.Out.Write(num25);
            int num32 = (length - num25) - this.map.Map_Header.Index_HeaderSize;
            int num33 = (num32 + list4[0]) + this.map.Map_Header.Index_HeaderSize;
            this.map.io.Out.SeekTo(0x18);
            this.map.io.Out.Write(num32);
            this.map.io.Out.Write(num33);
            this.map.io.Out.SeekTo(320);
            this.map.io.Out.Write(0);
            this.map.io.Out.SeekTo(0x158);
            this.map.io.Out.Write(matg.crazy_offset1.Value);
            this.map.io.Out.SeekTo(0x160);
            this.map.io.Out.Write(position);
            this.map.io.Out.Write(count);
            this.map.io.Out.Write(num20);
            this.map.io.Out.Write(num19);
            this.map.io.Out.Write(num21);
            int num34 = this.metas.Count;
            this.map.io.Out.SeekTo(0x2c0);
            this.map.io.Out.Write(num34);
            this.map.io.Out.Write(num22);
            this.map.io.Out.Write(num24);
            this.map.io.Out.Write(num23);
            this.map.io.Out.SeekTo(num25 + 0x18);
            this.map.io.Out.Write(num34);
            this.map.io.Close();
            this.metas = null;
            GC.Collect(GC.MaxGeneration);
            MessageBox.Show("Done");
        }

        public void Build(string filepath, bool overwrite)
        {
            int num2;
            string str;
            MetaInterface current;
            snd_ chunk;
            Sound_Resource_Layout layout4;
            List<int> list2;
            List<int> list3;
            int index;
            this.addedfilepaths = new List<string>();
            this.metas = new List<MetaInterface>();
            this.idents = new List<int>();
            this.stringids = new List<string>();
            this.overwritten = new List<bool>();
            foreach (Map.StringID gid in this.map.strings)
            {
                this.stringids.Add(gid.Name);
            }
            this.tempident = 0xe1740000;
            this.debugbox.Text = "Starting rebuild...\n";
            Application.DoEvents();
            DecompiledTagInterface item = new DecompiledTagInterface(filepath, Map.HaloVersionEnum.Halo2, false);
            int num = -1;
            for (num2 = 0; num2 < (this.map.Meta.Count - 1); num2++)
            {
                if ((this.map.Meta[num2].TagType == item.Tag_Type) && (this.map.FileName[num2].Name == item.Tag_Name))
                {
                    num = num2;
                    break;
                }
            }
            num2 = 0;
            while (num2 < (this.map.Meta.Count - 1))
            {
                str = Tag_Layouts.GetFullTagPath(this.map.FileName[num2].Name, this.map.Meta[num2].TagType, Map.HaloVersionEnum.Halo2);
                this.debugbox.Text = "Processing " + str + "...\n";
                Application.DoEvents();
                Meta meta = new Meta(num2, ref this.map);
                this.metas.Add(meta.Interface);
                this.addedfilepaths.Add(Tag_Layouts.GetFullTagPath(meta.Name, meta.TagType, Map.HaloVersionEnum.Halo2));
                this.idents.Add((int) this.tempident);
                this.tempident += 0x10001;
                this.overwritten.Add(false);
                num2++;
            }
            matg matg = (matg) this.metas[0].Header.Chunks[0];
            ugh_ h_ = this.map.ugh_Interface;
            scnr scnr = (scnr) this.metas[3].Header.Chunks[0];
            if (num == -1)
            {
                this.metas.Add(item);
                this.addedfilepaths.Add(Tag_Layouts.GetFullTagPath(item.Tag_Name, item.Tag_Type, Map.HaloVersionEnum.Halo2));
                this.idents.Add((int) this.tempident);
                this.tempident += 0x10001;
                this.overwritten.Add(true);
            }
            else if (overwrite)
            {
                this.overwritten[num] = true;
                this.metas[num] = item;
            }
            this.RecursivelyLoadDecompiledMetas(item.Header, overwrite);
            Application.DoEvents();
            this.metas.Add(this.map.ugh_Meta.Interface);
            this.addedfilepaths.Add(Tag_Layouts.GetFullTagPath(this.map.ugh_Meta.Name, this.map.ugh_Meta.TagType, Map.HaloVersionEnum.Halo2));
            this.idents.Add((int) this.tempident);
            this.tempident += 0x10001;
            Application.DoEvents();
            scnr.Predicted_Resources_0.RemoveAllChunks();
            using (List<MetaInterface>.Enumerator TagEnumerator2 = this.metas.GetEnumerator())
            {
                while (TagEnumerator2.MoveNext())
                {
                    current = TagEnumerator2.Current;
                    switch (current.Tag_Type)
                    {
                        case "bipd":
                        case "bloc":
                        case "ctrl":
                        case "jpt!":
                        case "mach":
                        case "scen":
                        case "ssce":
                        case "vehi":
                        {
                            scnr.Predicted_Resources_0_Layout tagblocklayout = new scnr.Predicted_Resources_0_Layout();
                            tagblocklayout.Resource.TagName = current.Tag_Name;
                            tagblocklayout.Resource.TagType = current.Tag_Type;
                            scnr.Predicted_Resources_0.AddChunk(tagblocklayout);
                            break;
                        }
                        case "eqip":
                        case "garb":
                        case "proj":
                        {
                            scnr.Predicted_Resources_0_Layout layout2 = new scnr.Predicted_Resources_0_Layout();
                            layout2.Resource.TagName = current.Tag_Name;
                            layout2.Resource.TagType = current.Tag_Type;
                            scnr.Predicted_Resources_0.AddChunk(layout2);
                            scnr.Predicted_Resources_0.AddChunk(layout2.Clone());
                            break;
                        }
                        case "weap":
                        {
                            scnr.Predicted_Resources_0_Layout layout3 = new scnr.Predicted_Resources_0_Layout();
                            layout3.Resource.TagName = current.Tag_Name;
                            layout3.Resource.TagType = current.Tag_Type;
                            scnr.Predicted_Resources_0.AddChunk(layout3);
                            scnr.Predicted_Resources_0.AddChunk(layout3.Clone());
                            scnr.Predicted_Resources_0.AddChunk(layout3.Clone());
                            break;
                        }
                    }
                }
            }
            List<Sound_Resource_Layout> list = new List<Sound_Resource_Layout>();
            for (num2 = 0; num2 < this.metas.Count; num2++)
            {
                current = this.metas[num2];
                if (current.Tag_Type == "snd!")
                {
                    chunk = (snd_) current.Header.GetChunk(0);
                    layout4 = chunk.Sound_Resource;
                    list.Add(layout4);
                }
            }
            h_.Sound_Permutations35.RemoveAllChunks();
            h_.Sound_Choices44.RemoveAllChunks();
            h_.Sound_Raw_Data68.RemoveAllChunks();
            h_.Sound_Names25.RemoveAllChunks();
            int num3 = 0;
            num2 = 0;
            while (num2 < this.metas.Count)
            {
                current = this.metas[num2];
                if (current.Tag_Type == "snd!")
                {
                    chunk = (snd_) current.Header.GetChunk(0);
                    chunk.Index_to_5th.Value = (short) h_.Sound_Permutations35.ChunkCount;
                    layout4 = list[num3];
                    num3++;
                    for (int i = 0; i < layout4.Sound_Permutations.ChunkCount; i++)
                    {
                        ugh_.Sound_Permutations35_Layout layout5 = (ugh_.Sound_Permutations35_Layout) layout4.Sound_Permutations.Chunks[i];
                        int chunkCount = h_.Sound_Choices44.ChunkCount;
                        for (int j = 0; j < layout5.chunkCount.Value; j++)
                        {
                            ugh_.Sound_Choices44_Layout layout6 = (ugh_.Sound_Choices44_Layout) layout4.Sound_Choices.Chunks[layout5.Index.Value + j];
                            int num7 = h_.Sound_Raw_Data68.ChunkCount;
                            for (int k = 0; k < layout6.chunkCount.Value; k++)
                            {
                                h_.Sound_Raw_Data68.AddChunk(layout4.Sound_Raw_Data.Chunks[layout6.Index.Value + k]);
                            }
                            bool flag = false;
                            ugh_.Sound_Names25_Layout layout7 = (ugh_.Sound_Names25_Layout) layout4.Sound_Names.Chunks[layout6.Name_Index.Value];
                            foreach (ugh_.Sound_Names25_Layout layout8 in h_.Sound_Names25.Chunks)
                            {
                                if (layout7.Name.Value == layout8.Name.Value)
                                {
                                    layout6.Name_Index.Value = (short) layout8.Chunk_Number;
                                    flag = true;
                                    break;
                                }
                            }
                            if (!flag)
                            {
                                h_.Sound_Names25.AddChunk(layout7);
                                layout6.Name_Index.Value = (short) (h_.Sound_Names25.ChunkCount - 1);
                            }
                            layout6.Index.Value = (short) num7;
                            h_.Sound_Choices44.AddChunk(layout6);
                        }
                        layout5.Index.Value = (short) chunkCount;
                        h_.Sound_Permutations35.AddChunk(layout5);
                    }
                }
                num2++;
            }
            this.FixTagReferences();
            this.map.io.Open();
            this.map.io.In.SeekTo(this.map.Map_Header.GeometryCacheOffset);
            byte[] buffer = this.map.io.In.ReadBytes(this.map.Map_Header.GeometryCacheSize);
            this.map.io.In.SeekTo(this.map.Map_Header.IndexOffset);
            byte[] buffer2 = this.map.io.In.ReadBytes(this.map.Map_Header.Index_HeaderSize);
            this.map.io.Out.BaseStream.SetLength(0x800L);
            this.map.io.Out.SeekTo(0x800);
            this.debugbox.Text = this.debugbox.Text + "Writing internal sound raw\n";
            foreach (ugh_.Sound_Raw_Data68_Layout layout9 in h_.Sound_Raw_Data68.Chunks)
            {
                if (layout9.Sound_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                {
                    Application.DoEvents();
                    layout9.Sound_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                    layout9.Sound_Raw.Write_Raw(ref this.map.io.Out);
                    this.WritePadding();
                }
            }
            foreach (MetaInterface interface3 in this.metas)
            {
                if (interface3.Tag_Type == "mode")
                {
                    mode mode = (mode) interface3.Header.GetChunk(0);
                    list2 = new List<int>();
                    list3 = new List<int>();
                    this.debugbox.Text = this.debugbox.Text + "Writing internal model type1 raw for " + interface3.Tag_Name + "\n";
                    Application.DoEvents();
                    foreach (mode.Part_Layout layout10 in mode.Part.Chunks)
                    {
                        if (layout10.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            index = list2.IndexOf(layout10.Visual_Raw.Raw_Offset.Data_Offset);
                            if (index == -1)
                            {
                                list2.Add(layout10.Visual_Raw.Raw_Offset.Data_Offset);
                                layout10.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                list3.Add(layout10.Visual_Raw.Raw_Offset.Data_Offset);
                                layout10.Visual_Raw.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                            else if (index != -1)
                            {
                                layout10.Visual_Raw.Raw_Offset.Data_Offset = list3[index];
                            }
                        }
                    }
                    this.debugbox.Text = this.debugbox.Text + "Writing internal model type2 raw for " + interface3.Tag_Name + "\n";
                    Application.DoEvents();
                    foreach (mode.Unknown_Raw_Data_Layout layout11 in mode.Unknown_5.Chunks)
                    {
                        if (layout11.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            index = list2.IndexOf(layout11.Visual_Raw.Raw_Offset.Data_Offset);
                            if (index == -1)
                            {
                                list2.Add(layout11.Visual_Raw.Raw_Offset.Data_Offset);
                                layout11.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                list3.Add(layout11.Visual_Raw.Raw_Offset.Data_Offset);
                                layout11.Visual_Raw.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                            else if (index != -1)
                            {
                                layout11.Visual_Raw.Raw_Offset.Data_Offset = list3[index];
                            }
                        }
                    }
                }
            }
            List<int> list4 = new List<int>();
            List<int> list5 = new List<int>();
            List<int> list6 = new List<int>();
            List<int> list7 = new List<int>();
            List<int> list8 = new List<int>();
            int num10 = 0;
            foreach (scnr.Structure_BSPS_Layout layout12 in scnr.Structure_BSPS.Chunks)
            {
                int num11 = 0;
                TagIdent ident = layout12.Light_Map_0;
                if (ident.TagName != "null")
                {
                    str = Tag_Layouts.GetFullTagPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                    num2 = this.addedfilepaths.IndexOf(str);
                    list8.Add(num2);
                    num11 = num2;
                }
                else
                {
                    list8.Add(-1);
                }
                int num12 = 0;
                ident = layout12.Structure_BSP_0;
                if (ident.TagName != "null")
                {
                    str = Tag_Layouts.GetFullTagPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                    num2 = this.addedfilepaths.IndexOf(str);
                    list7.Add(num2);
                    num12 = num2;
                }
                SBSP sbsp = (SBSP) this.metas[num12].Header.GetChunk(0);
                this.debugbox.Text = this.debugbox.Text + "Writing internal bsp cluster raw for " + ident.Name + "\n";
                Application.DoEvents();
                foreach (SBSP.Detail_Objects cluster in sbsp.detail_objects.Chunks)
                {
                    if (cluster.rawdata.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        cluster.rawdata.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        cluster.rawdata.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
                num10++;
                this.debugbox.Text = this.debugbox.Text + "Writing internal bsp scenery raw for " + ident.Name + "\n";
                Application.DoEvents();
                foreach (SBSP.Scenery_Cluster cluster2 in sbsp.bsp_permutations.Chunks)
                {
                    if (cluster2.rawdata.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        cluster2.rawdata.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        cluster2.rawdata.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
                if (num11 != -1)
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal lightmap raw for " + ident.Name + "\n";
                    Application.DoEvents();
                    ltmp ltmp = (ltmp) this.metas[num11].Header.Chunks[0];
                    foreach (ltmp.Unknown17_Layout layout13 in ltmp.Unknown17.Chunks)
                    {
                        foreach (ltmp.Unknown17_Layout.Lightmap_Raw_Layout layout14 in layout13.lightmap_raw.Chunks)
                        {
                            if (layout14.raw_data.Raw_Offset.Map_Location == MapTypes.Internal)
                            {
                                layout14.raw_data.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                layout14.raw_data.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                        }
                    }
                }
                list2 = new List<int>();
                list3 = new List<int>();
                this.debugbox.Text = this.debugbox.Text + "Writing internal unknown bsp raw for " + ident.Name + "\n";
                Application.DoEvents();
                foreach (SBSP.Decorators block in sbsp.decorators.Chunks)
                {
                    foreach (SBSP.Decorators.Raw_Data_Tag_Block block2 in block.raw_data_block.Chunks)
                    {
                        if (block2.rawdata.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            index = list2.IndexOf(block2.rawdata.Raw_Offset.Data_Offset);
                            if (index == -1)
                            {
                                list2.Add(block2.rawdata.Raw_Offset.Data_Offset);
                                block2.rawdata.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                list3.Add(block2.rawdata.Raw_Offset.Data_Offset);
                                block2.rawdata.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                            else
                            {
                                block2.rawdata.Raw_Offset.Data_Offset = list3[index];
                            }
                        }
                    }
                }
                this.debugbox.Text = this.debugbox.Text + "Writing internal bsp raw for " + ident.Name + "\n";
                Application.DoEvents();
                foreach (SBSP.SBSP_Water water in sbsp.water_definitions.Chunks)
                {
                    if (water.rawdata.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        water.rawdata.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        water.rawdata.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
            }
            foreach (MetaInterface interface3 in this.metas)
            {
                if (interface3.Tag_Type == "weat")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal weather raw for " + interface3.Tag_Name + "\n";
                    Application.DoEvents();
                    weat weat = (weat) interface3.Header.GetChunk(0);
                    foreach (weat.Particle_Layout layout15 in weat.Particle.Chunks)
                    {
                        if (layout15.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            layout15.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                            layout15.Visual_Raw.Write_Raw(ref this.map.io.Out);
                            this.WritePadding();
                        }
                    }
                }
            }
            foreach (MetaInterface interface3 in this.metas)
            {
                if (interface3.Tag_Type == "DECR")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal DECR raw for " + interface3.Tag_Name + "\n";
                    Application.DoEvents();
                    DECR decr = (DECR) interface3.Header.GetChunk(0);
                    if (decr.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        decr.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        decr.Visual_Raw.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
            }
            foreach (MetaInterface interface3 in this.metas)
            {
                if (interface3.Tag_Type == "PRTM")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal PRTM raw for " + interface3.Tag_Name + "\n";
                    Application.DoEvents();
                    PRTM prtm = (PRTM) interface3.Header.GetChunk(0);
                    if (prtm.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                    {
                        prtm.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                        prtm.Visual_Raw.Write_Raw(ref this.map.io.Out);
                        this.WritePadding();
                    }
                }
            }
            this.debugbox.Text = this.debugbox.Text + "Writing internal ugh raw type2 \n";
            Application.DoEvents();
            foreach (ugh_.Diff_Model_Raw_Data87_Layout layout16 in h_.Diff_Model_Raw_Data87.Chunks)
            {
                if (layout16.Visual_Raw.Raw_Offset.Map_Location == MapTypes.Internal)
                {
                    layout16.Visual_Raw.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                    layout16.Visual_Raw.Write_Raw(ref this.map.io.Out);
                    this.WritePadding();
                }
            }
            foreach (MetaInterface interface3 in this.metas)
            {
                if (interface3.Tag_Type == "jmad")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal animation raw for " + interface3.Tag_Name + "\n";
                    Application.DoEvents();
                    jmad jmad = (jmad) interface3.Header.GetChunk(0);
                    list2 = new List<int>();
                    list3 = new List<int>();
                    foreach (jmad.Unknown_11_Layout layout17 in jmad.Unknown_11.Chunks)
                    {
                        if (layout17.Raw_Data.Raw_Offset.Map_Location == MapTypes.Internal)
                        {
                            index = list2.IndexOf(layout17.Raw_Data.Raw_Offset.Data_Offset);
                            if (index == -1)
                            {
                                list2.Add(layout17.Raw_Data.Raw_Offset.Data_Offset);
                                layout17.Raw_Data.Raw_Offset.Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                list3.Add(layout17.Raw_Data.Raw_Offset.Data_Offset);
                                layout17.Raw_Data.Write_Raw(ref this.map.io.Out);
                                this.WritePadding();
                            }
                            else if (index != -1)
                            {
                                layout17.Raw_Data.Raw_Offset.Data_Offset = list3[index];
                            }
                        }
                    }
                }
            }
            for (num2 = 0; num2 < list7.Count; num2++)
            {
                Meta meta2;
                int num13 = (int) this.map.io.Out.BaseStream.Position;
                list5.Add(num13);
                int num14 = (this.map.Index_Header.Constant - 0x20) + this.map.Map_Header.Index_HeaderSize;
                int num15 = num14 - num13;
                int size = 0;
                if (list8[num2] == -1)
                {
                    meta2 = TagCompiler.CompileSBSPMetaFromTagInterface(this.metas[list7[num2]], num13, num15, null);
                }
                else
                {
                    meta2 = TagCompiler.CompileSBSPMetaFromTagInterface(this.metas[list7[num2]], num13, num15, this.metas[list8[num2]]);
                }
                this.map.io.Out.Write(meta2.MS.ToArray());
                size = meta2.Size;
                list4.Add(size);
                scnr.Structure_BSPS_Layout layout18 = (scnr.Structure_BSPS_Layout) scnr.Structure_BSPS.Chunks[num2];
                layout18.Structure_BSP_Offset.Value = num13;
                layout18.Structure_BSP_Size.Value = size;
                layout18.Primary_Magic_Meta_Offset.Value = num14;
            }
            int count = this.stringids.Count;
            int position = (int) this.map.io.Out.BaseStream.Position;
            for (num2 = 0; num2 < count; num2++)
            {
                core.TagInterface.String.WriteString(ref this.map.io.Out, this.stringids[num2], 0x80);
            }
            this.WritePadding();
            int num19 = (int) this.map.io.Out.BaseStream.Position;
            int num20 = 0;
            for (num2 = 0; num2 < count; num2++)
            {
                this.map.io.Out.Write(num20);
                num20 += this.stringids[num2].Length + 1;
            }
            this.WritePadding();
            int num21 = (int) this.map.io.Out.BaseStream.Position;
            for (num2 = 0; num2 < count; num2++)
            {
                core.TagInterface.String.WriteString(ref this.map.io.Out, this.stringids[num2], this.stringids[num2].Length + 1);
            }
            this.WritePadding();
            int num22 = (int) this.map.io.Out.BaseStream.Position;
            for (num2 = 0; num2 < this.metas.Count; num2++)
            {
                core.TagInterface.String.WriteString(ref this.map.io.Out, this.metas[num2].Tag_Name, this.metas[num2].Tag_Name.Length + 1);
            }
            this.WritePadding();
            int num23 = (int) this.map.io.Out.BaseStream.Position;
            int num24 = 0;
            for (num2 = 0; num2 < this.metas.Count; num2++)
            {
                this.map.io.Out.Write(num24);
                num24 += this.metas[num2].Tag_Name.Length + 1;
            }
            this.WritePadding();
            num2 = 0;
            while (num2 < 8)
            {
                matg.Unicode[num2].Index_Offset = (int) this.map.io.Out.BaseStream.Position;
                matg.Unicode[num2].Unicode_Strings.WriteChunks(ref this.map.io.Out, MetaInterface.InterfaceMode.CompiledTagMode);
                this.WritePadding();
                matg.Unicode[num2].Table_Offset = (int) this.map.io.Out.BaseStream.Position;
                matg.Unicode[num2].Write_Raw(ref this.map.io.Out);
                this.WritePadding();
                num2++;
            }
            matg.crazy_offset1.Value = (int) this.map.io.Out.BaseStream.Position;
            matg.crazy_offset2.Value = (int) this.map.io.Out.BaseStream.Position;
            this.map.io.Out.Write(buffer);
            this.WritePadding();
            foreach (MetaInterface interface3 in this.metas)
            {
                if (interface3.Tag_Type == "bitm")
                {
                    this.debugbox.Text = this.debugbox.Text + "Writing internal bitmap raw for " + interface3.Tag_Name + "\n";
                    Application.DoEvents();
                    bitm bitm = (bitm) interface3.Header.GetChunk(0);
                    foreach (bitm.Bitmap_Data_Layout layout19 in bitm.Bitmap_Data.Chunks)
                    {
                        num2 = 5;
                        while (num2 > -1)
                        {
                            if ((layout19.rawdata.lod_offset[num2].Data_Offset != -1) && (layout19.rawdata.lod_offset[num2].Map_Location == MapTypes.Internal))
                            {
                                layout19.rawdata.lod_offset[num2].Data_Offset = (int) this.map.io.Out.BaseStream.Position;
                                this.map.io.Out.Write(layout19.rawdata.LOD_MS[num2].ToArray());
                                this.WritePadding();
                            }
                            num2--;
                        }
                    }
                }
            }
            int num25 = (int) this.map.io.Out.BaseStream.Position;
            int num26 = (this.map.Index_Header.Constant - num25) - 0x20;
            int magic = num26 + list4[0];
            this.map.io.Out.Write(buffer2);
            List<int> list9 = new List<int>();
            List<int> list10 = new List<int>();
            for (num2 = 0; num2 < this.metas.Count; num2++)
            {
                if ((this.metas[num2].Tag_Type == "ltmp") || (this.metas[num2].Tag_Type == "sbsp"))
                {
                    list9.Add(0);
                    list10.Add(0);
                }
                else
                {
                    this.debugbox.Text = "Compiling " + this.metas[num2].Tag_Name + "...\n";
                    Application.DoEvents();
                    try
                    {
                        int mapoffset = (int) this.map.io.Out.BaseStream.Position;
                        Meta meta3 = TagCompiler.CompileMetaFromTagInterface(this.metas[num2], mapoffset, magic);
                        this.map.io.Out.Write(meta3.MS.ToArray());
                        list9.Add(mapoffset);
                        list10.Add(meta3.size);
                    }
                    catch
                    {
                        MessageBox.Show("Error Compiling");
                    }
                }
            }
            this.WritePadding();
            int num29 = this.map.Index_Header.Tags_Offset - this.map.Map_Header.IndexOffset;
            this.map.io.Out.SeekTo(num25 + num29);
            for (num2 = 0; num2 < this.metas.Count; num2++)
            {
                char[] array = this.metas[num2].Tag_Type.ToCharArray();
                Array.Reverse(array);
                this.map.io.Out.Write(array);
                this.map.io.Out.Write(this.idents[num2]);
                int num30 = list9[num2] + magic;
                if ((this.metas[num2].Tag_Type == "ltmp") || (this.metas[num2].Tag_Type == "sbsp"))
                {
                    this.map.io.Out.Write(0);
                    this.map.io.Out.Write(0);
                }
                else
                {
                    this.map.io.Out.Write(num30);
                    this.map.io.Out.Write(list10[num2]);
                }
            }
            int length = (int) this.map.io.Out.BaseStream.Length;
            this.map.io.Out.SeekTo(8);
            this.map.io.Out.Write(length);
            this.map.io.Out.SeekTo(0x10);
            this.map.io.Out.Write(num25);
            int num32 = (length - num25) - this.map.Map_Header.Index_HeaderSize;
            int num33 = (num32 + list4[0]) + this.map.Map_Header.Index_HeaderSize;
            this.map.io.Out.SeekTo(0x18);
            this.map.io.Out.Write(num32);
            this.map.io.Out.Write(num33);
            this.map.io.Out.SeekTo(320);
            this.map.io.Out.Write(0);
            this.map.io.Out.SeekTo(0x158);
            this.map.io.Out.Write(matg.crazy_offset1.Value);
            this.map.io.Out.SeekTo(0x160);
            this.map.io.Out.Write(position);
            this.map.io.Out.Write(count);
            this.map.io.Out.Write(num20);
            this.map.io.Out.Write(num19);
            this.map.io.Out.Write(num21);
            int num34 = this.metas.Count;
            this.map.io.Out.SeekTo(0x2c0);
            this.map.io.Out.Write(num34);
            this.map.io.Out.Write(num22);
            this.map.io.Out.Write(num24);
            this.map.io.Out.Write(num23);
            this.map.io.Out.SeekTo(num25 + 0x18);
            this.map.io.Out.Write(num34);
            this.map.io.Close();
            this.metas = null;
            GC.Collect(GC.MaxGeneration);
            MessageBox.Show("Done");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void FixTagReferences()
        {
            Stack<TagBlock> stack = new Stack<TagBlock>();
            int num = 0;
            foreach (MetaInterface interface2 in this.metas)
            {
                stack.Push(interface2.Header);
                while (stack.Count > 0)
                {
                    TagBlock block = stack.Pop();
                    foreach (TagBlockLayout layout in block.Chunks)
                    {
                        foreach (BaseObject obj2 in layout)
                        {
                            string str;
                            int index;
                            if (obj2.Type == BaseObjectType.TagIdent)
                            {
                                TagIdent ident = (TagIdent) obj2;
                                if (ident.TagName != "null")
                                {
                                    str = Tag_Layouts.GetFullTagPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                                    index = this.addedfilepaths.IndexOf(str);
                                    if (index == -1)
                                    {
                                        ident.Ident = -1;
                                    }
                                    else
                                    {
                                        ident.Ident = this.idents[index];
                                    }
                                }
                                else
                                {
                                    ident.Ident = -1;
                                }
                            }
                            else if (obj2.Type == BaseObjectType.TagReference)
                            {
                                TagReference reference = (TagReference) obj2;
                                if (reference.Ident.TagName != "null")
                                {
                                    str = Tag_Layouts.GetFullTagPath(reference.Ident.TagName, reference.Ident.TagType, Map.HaloVersionEnum.Halo2);
                                    index = this.addedfilepaths.IndexOf(str);
                                    if (index == -1)
                                    {
                                        reference.Ident.Ident = -1;
                                    }
                                    else
                                    {
                                        reference.Ident.Ident = this.idents[index];
                                    }
                                }
                                else
                                {
                                    reference.Ident.Ident = -1;
                                }
                            }
                            else if (obj2.Type == BaseObjectType.Raw_Tag_Block)
                            {
                                Raw_Tag_Block block2 = (Raw_Tag_Block) obj2;
                                if (block2.Tag_Ident.TagName != "null")
                                {
                                    str = Tag_Layouts.GetFullTagPath(block2.Tag_Ident.TagName, block2.Tag_Ident.TagType, Map.HaloVersionEnum.Halo2);
                                    index = this.addedfilepaths.IndexOf(str);
                                    block2.Tag_Ident.Ident = this.idents[num];
                                }
                                else
                                {
                                    block2.Tag_Ident.Ident = -1;
                                }
                            }
                            else
                            {
                                int num3;
                                if (obj2.Type == BaseObjectType.Unicode_Raw)
                                {
                                    Unicode_Raw raw = (Unicode_Raw) obj2;
                                    foreach (Unicode_Raw.Unicode_String str2 in raw.Unicode_Strings.Chunks)
                                    {
                                        num3 = this.stringids.IndexOf(str2.name.Value);
                                        if (num3 == -1)
                                        {
                                            this.stringids.Add(str2.name.Value);
                                            num3 = this.stringids.Count - 1;
                                        }
                                        str2.name.Index = (short) num3;
                                    }
                                }
                                else if (obj2.Type == BaseObjectType.StringID)
                                {
                                    core.TagInterface.StringID gid = (core.TagInterface.StringID) obj2;
                                    num3 = this.stringids.IndexOf(gid.Text);
                                    if (num3 == -1)
                                    {
                                        this.stringids.Add(gid.Text);
                                        num3 = this.stringids.Count - 1;
                                    }
                                    gid.Index = (short) num3;
                                    gid.StringLength = gid.Text.Length;
                                    if (gid.Index == -1)
                                    {
                                    }
                                }
                                else if (obj2.Type == BaseObjectType.TagBlock)
                                {
                                    stack.Push((TagBlock) obj2);
                                }
                            }
                        }
                    }
                }
                num++;
            }
        }

        private void InitializeComponent()
        {
            this.scenariopath = new TextBox();
            this.label1 = new Label();
            this.browseforscenario = new Button();
            this.RebuildButton = new Button();
            this.debugbox = new RichTextBox();
            this.overwrite = new CheckBox();
            base.SuspendLayout();
            this.scenariopath.Location = new Point(0x43, 12);
            this.scenariopath.Name = "scenariopath";
            this.scenariopath.Size = new Size(0x1cf, 20);
            this.scenariopath.TabIndex = 0;
            this.label1.AutoSize = true;
            this.label1.Font = new Font("Microsoft Sans Serif", 12f, FontStyle.Regular, GraphicsUnit.Point, 0);
            this.label1.Location = new Point(12, 12);
            this.label1.Name = "label1";
            this.label1.Size = new Size(0x31, 20);
            this.label1.TabIndex = 1;
            this.label1.Text = "Meta:";
            this.browseforscenario.Location = new Point(0x218, 11);
            this.browseforscenario.Name = "browseforscenario";
            this.browseforscenario.Size = new Size(0x4b, 0x17);
            this.browseforscenario.TabIndex = 2;
            this.browseforscenario.Text = "...";
            this.browseforscenario.UseVisualStyleBackColor = true;
            this.browseforscenario.Click += new EventHandler(this.browseforscenario_Click);
            this.RebuildButton.Location = new Point(0x266, 11);
            this.RebuildButton.Name = "RebuildButton";
            this.RebuildButton.Size = new Size(0x4b, 0x17);
            this.RebuildButton.TabIndex = 3;
            this.RebuildButton.Text = "Rebuild";
            this.RebuildButton.UseVisualStyleBackColor = true;
            this.RebuildButton.Click += new EventHandler(this.RebuildButton_Click);
            this.debugbox.Dock = DockStyle.Bottom;
            this.debugbox.Location = new Point(0, 0x3f);
            this.debugbox.Name = "debugbox";
            this.debugbox.Size = new Size(0x2b1, 0x87);
            this.debugbox.TabIndex = 4;
            this.debugbox.Text = "";
            this.overwrite.AutoSize = true;
            this.overwrite.Location = new Point(0x10, 0x26);
            this.overwrite.Name = "overwrite";
            this.overwrite.Size = new Size(0x8d, 0x11);
            this.overwrite.TabIndex = 5;
            this.overwrite.Text = "Overwrite Original Metas";
            this.overwrite.UseVisualStyleBackColor = true;
            base.AutoScaleDimensions = new SizeF(6f, 13f);
            base.AutoScaleMode = AutoScaleMode.Font;
            base.ClientSize = new Size(0x2b1, 0xc6);
            base.Controls.Add(this.overwrite);
            base.Controls.Add(this.debugbox);
            base.Controls.Add(this.RebuildButton);
            base.Controls.Add(this.browseforscenario);
            base.Controls.Add(this.label1);
            base.Controls.Add(this.scenariopath);
            base.Name = "Builder";
            this.Text = "Builder";
            base.ResumeLayout(false);
            base.PerformLayout();
        }

        private void LoadMeta(Meta m)
        {
        }

        private void RebuildButton_Click(object sender, EventArgs e)
        {
            this.Build(this.scenariopath.Text, this.overwrite.Checked);
        }

        private void RecursivelyLoadDecompiledMetas(TagBlock tagblock, bool overwrite)
        {
            Stack<TagBlock> stack = new Stack<TagBlock>();
            stack.Push(tagblock);
            while (stack.Count > 0)
            {
                TagBlock block = stack.Pop();
                Application.DoEvents();
                foreach (TagBlockLayout layout in block.Chunks)
                {
                    foreach (BaseObject obj2 in layout)
                    {
                        string str;
                        int index;
                        DecompiledTagInterface interface2;
                        string tagType;
                        if (obj2.Type == BaseObjectType.TagReference)
                        {
                            TagReference reference = (TagReference) obj2;
                            if (reference.Ident.TagName != "null")
                            {
                                tagType = reference.Ident.TagType;
                                if ((tagType != null) && ((((tagType == "sncl") || (tagType == "spk!")) || (tagType == "matg")) || (tagType == "ugh!")))
                                {
                                    continue;
                                }
                                str = Tag_Layouts.GetFullTagPath(reference.Ident.TagName, reference.Ident.TagType, Map.HaloVersionEnum.Halo2);
                                this.debugbox.Text = "Processing " + str + "...\n";
                                Application.DoEvents();
                                index = this.addedfilepaths.IndexOf(str);
                                if (!((index == -1) || overwrite) || ((index != -1) && this.overwritten[index]))
                                {
                                    continue;
                                }
                                if (File.Exists(str))
                                {
                                    interface2 = new DecompiledTagInterface(str, Map.HaloVersionEnum.Halo2, false);
                                    if (index != -1)
                                    {
                                        this.overwritten[index] = true;
                                        this.metas[index] = interface2;
                                        stack.Push(interface2.Header);
                                    }
                                    else
                                    {
                                        this.metas.Add(interface2);
                                        this.addedfilepaths.Add(str);
                                        this.idents.Add((int) this.tempident);
                                        this.tempident += 0x10001;
                                        this.overwritten.Add(true);
                                        stack.Push(interface2.Header);
                                    }
                                }
                                else
                                {
                                    reference.Ident.TagName = "null";
                                }
                            }
                        }
                        else if (obj2.Type == BaseObjectType.TagIdent)
                        {
                            TagIdent ident = (TagIdent) obj2;
                            if (ident.TagName != "null")
                            {
                                tagType = ident.TagType;
                                if ((tagType != null) && ((((tagType == "sncl") || (tagType == "spk!")) || (tagType == "matg")) || (tagType == "ugh!")))
                                {
                                    continue;
                                }
                                str = Tag_Layouts.GetFullTagPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                                this.debugbox.Text = "Processing " + str + "...\n";
                                Application.DoEvents();
                                index = this.addedfilepaths.IndexOf(str);
                                if (!((index == -1) || overwrite) || ((index != -1) && this.overwritten[index]))
                                {
                                    continue;
                                }
                                if (File.Exists(str))
                                {
                                    interface2 = new DecompiledTagInterface(str, Map.HaloVersionEnum.Halo2, false);
                                    if (index != -1)
                                    {
                                        this.metas[index] = interface2;
                                        stack.Push(interface2.Header);
                                        this.overwritten[index] = true;
                                    }
                                    else
                                    {
                                        this.metas.Add(interface2);
                                        this.addedfilepaths.Add(str);
                                        this.idents.Add((int) this.tempident);
                                        this.tempident += 0x10001;
                                        this.overwritten.Add(true);
                                        stack.Push(interface2.Header);
                                    }
                                }
                                else
                                {
                                    ident.TagName = "null";
                                }
                            }
                        }
                        else if (obj2.Type == BaseObjectType.TagBlock)
                        {
                            stack.Push((TagBlock) obj2);
                        }
                    }
                }
            }
        }

        private void WritePadding()
        {
            int size = 0x200 - (((int) this.map.io.Out.BaseStream.Position) % 0x200);
            if (size != 0x200)
            {
                core.TagInterface.Padding.Write(ref this.map.io.Out, size);
            }
        }
    }
}

