﻿namespace core
{
    using core.H2;
    using core.IO;
    using core.TagInterface;
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    public class DecompiledTagInterface : MetaInterface
    {
        protected Map.HaloVersionEnum haloversion;
        protected EndianIO io;

        public DecompiledTagInterface() : base(MetaInterface.InterfaceMode.DecompiledTagMode)
        {
        }

        public DecompiledTagInterface(string filename, Map.HaloVersionEnum haloversion, bool force_soft_coded) : base(MetaInterface.InterfaceMode.DecompiledTagMode)
        {
            this.haloversion = haloversion;
            this.io = new EndianIO(filename, EndianType.BigEndian);
            this.io.Open();
            this.io.In.SeekTo(0x24);
            try
            {
                base.tagtype = core.TagInterface.String.ReadString(ref this.io.In, 4);
            }
            catch
            {
            }
            string str = filename.Substring(Tag_Layouts.GetTagPath(haloversion).Length);
            str = str.Substring(0, str.LastIndexOf('.'));
            base.tagname = str;
            base.tagdef = Tag_Layouts.GetTagCategory(haloversion).GetTagDefinitionByTagType(base.tagtype);
            if (base.tagdef != null)
            {
                if (force_soft_coded)
                {
                    base.Header = new TagBlock(base.tagdef.TagType, base.tagdef.XML_Layout, 1, haloversion);
                }
                else
                {
                    base.Header = new TagBlock(base.tagdef.TagType, base.tagdef.Layout, 1, haloversion);
                }
                this.io.In.SeekTo(64);

                if (filename == "G:\\XBox\\Halo 2\\Halo 2 Programs Source Code\\H2Core Decompile\\H2\\sound\\vehicles\\phantom\\phantom_turret_fire_c_ap.sound")
                {
                    bool test = false;
                }

                try
                {
                    RecursivelyReadElements(base.header, ref this.io.In, haloversion);
                }
                catch
                {
                }
                this.io.Close();
                if ((base.tagtype == "snd!") && (haloversion == Map.HaloVersionEnum.Halo2))
                {
                    snd_ chunk = (snd_) base.header.GetChunk(0);
                    string str2 = filename;
                    int length = str2.LastIndexOf(".");
                    EndianIO nio = new EndianIO(str2.Substring(0, length) + ".srl", EndianType.BigEndian);
                    nio.Open();
                    nio.In.SeekTo(0);
                    TagBlock tagblock = new TagBlock("", Sound_Resource_Layout.Layout, 1, Map.HaloVersionEnum.Halo2);
                    RecursivelyReadElements(tagblock, ref nio.In, Map.HaloVersionEnum.Halo2);
                    chunk.Sound_Resource = (Sound_Resource_Layout) tagblock.GetChunk(0);
                    nio.Close();
                }
            }
        }

        public static void RecursivelyReadElements(TagBlock tagblock, ref EndianIO.EndianReader br, Map.HaloVersionEnum haloversion)
        {
            Stack<TagBlock> stack = new Stack<TagBlock>();
            stack.Push(tagblock);
            while (stack.Count > 0)
            {
                TagBlock block = stack.Pop();
                Application.DoEvents();
                if (block.ChunkCount != 0)
                {
                    block.ReadChunks(ref br, MetaInterface.InterfaceMode.DecompiledTagMode, haloversion);
                    foreach (TagBlockLayout layout in block.chunks)
                    {
                        foreach (BaseObject obj2 in layout)
                        {
                            if (obj2.Type == BaseObjectType.Raw_Tag_Block)
                            {
                                Raw_Tag_Block block2 = (Raw_Tag_Block) obj2;
                                stack.Push(block2.Resource_Block);
                                if (block2.Raw_Offset.Data_Offset == -1)
                                {
                                    continue;
                                }
                                block2.Read_Raw(ref br);
                            }
                            else if (obj2.Type == BaseObjectType.Data)
                            {
                                ((Data) obj2).Read_Raw(ref br);
                            }
                            else if (obj2.Type == BaseObjectType.Sound_Raw)
                            {
                                Data data2 = (Data) obj2;
                                data2.Read_Raw(ref br);
                            }
                            else if (obj2.Type == BaseObjectType.Animation_Raw)
                            {
                                ((Data) obj2).Read_Raw(ref br);
                            }
                            else if (obj2.Type == BaseObjectType.Bitmap_Raw)
                            {
                                Bitmap_Raw raw = (Bitmap_Raw) obj2;
                                for (int i = 0; i < 6; i++)
                                {
                                    if (raw.LOD_Raw_Data_Offset[i].Data_Offset != -1)
                                    {
                                        raw.Read_LOD_Raw(ref br, i);
                                    }
                                }
                            }
                            else if (obj2.Type == BaseObjectType.TagBlock)
                            {
                                TagBlock item = (TagBlock) obj2;
                                stack.Push(item);
                            }
                        }
                    }
                }
            }
        }
    }
}

