﻿namespace core.H2Sound
{
    using core;
    using core.H2;
    using core.IO;
    using core.TagInterface;
    using System;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    public class Sound
    {
        public string metaLocation = "";
        public int permutationsCount;
        public snd_ snd;
        public byte soundFormat;
        public Sound_Resource_Layout srl;

        public Sound(DecompiledTagInterface tagInterface, string MetaLocation)
        {
            this.metaLocation = MetaLocation;
            this.snd = (snd_) tagInterface.Header.GetChunk(0);
            string metaLocation = this.metaLocation;
            int length = metaLocation.LastIndexOf(".");
            EndianIO nio = new EndianIO(metaLocation.Substring(0, length) + ".srl", EndianType.BigEndian);
            nio.Open();
            nio.In.SeekTo(0);
            TagBlock tagblock = new TagBlock("", Sound_Resource_Layout.Layout, 1, Map.HaloVersionEnum.Halo2);
            DecompiledTagInterface.RecursivelyReadElements(tagblock, ref nio.In, Map.HaloVersionEnum.Halo2);
            this.srl = (Sound_Resource_Layout) tagblock.Chunks[0];
            this.snd.Sound_Resource = this.srl;
            this.soundFormat = (byte) (this.snd.Format.Value + 1);
            this.permutationsCount = this.snd.Chunk_Count.Value;
        }

        public void ExtractAll(string filePath)
        {
            ugh_.Sound_Permutations35_Layout chunk = (ugh_.Sound_Permutations35_Layout) this.srl.Sound_Permutations.GetChunk(0);
            int num = chunk.Index.Value;
            for (int i = num; i < (chunk.chunkCount.Value + num); i++)
            {
                this.ExtractPermutation(i, filePath, false);
            }
        }

        public string ExtractPermutation(int Index, string filePath, bool tempFile)
        {
            int num2;
            ugh_.Sound_Raw_Data68_Layout layout3;
            ugh_.Sound_Choices44_Layout chunk = (ugh_.Sound_Choices44_Layout) this.srl.Sound_Choices.GetChunk(Index);
            ugh_.Sound_Names25_Layout layout2 = (ugh_.Sound_Names25_Layout) this.srl.Sound_Names.GetChunk(chunk.Name_Index.Value);
            string path = filePath + @"\";
            if (tempFile)
            {
                path = path + layout2.Name.Value + "-temp";
            }
            else
            {
                path = path + layout2.Name.Value;
            }
            if (this.soundFormat != 3)
            {
                path = path + ".wav";
            }
            else
            {
                path = path + ".wma";
            }
            FileStream output = new FileStream(path, FileMode.Create, FileAccess.ReadWrite);
            BinaryWriter writer = new BinaryWriter(output);
            if (this.soundFormat != 3)
            {
                int size = 0;
                for (num2 = chunk.Index.Value; num2 < (chunk.Index.Value + chunk.chunkCount.Value); num2++)
                {
                    layout3 = (ugh_.Sound_Raw_Data68_Layout) this.srl.Sound_Raw_Data.GetChunk(num2);
                    size += (int) layout3.Sound_Raw.MS.Length;
                }
                writer.Write(this.writeHeader(size, this.soundFormat));
            }
            for (num2 = chunk.Index.Value; num2 < (chunk.Index.Value + chunk.chunkCount.Value); num2++)
            {
                layout3 = (ugh_.Sound_Raw_Data68_Layout) this.srl.Sound_Raw_Data.GetChunk(num2);
                writer.Write(layout3.Sound_Raw.MS.ToArray());
            }
            writer.Close();
            output.Close();
            return path;
        }

        public void getNeededRawCount(int fileSize, out int count, out int remainderSize)
        {
            count = 0;
            int num = 0;
            for (int i = 0; i < fileSize; i += 0x7ff8)
            {
                count++;
                num = i;
            }
            remainderSize = fileSize - num;
        }

        private int getSoundFormatFromFile(string filePath)
        {
            int num = 0x46464952;
            int num2 = 0x75b22630;
            FileStream input = new FileStream(filePath, FileMode.Open, FileAccess.Read);
            BinaryReader reader = new BinaryReader(input);
            reader.BaseStream.Position = 0L;
            int num3 = reader.ReadInt32();
            if (num3 == num)
            {
                reader.BaseStream.Position = 0x16L;
                return (reader.ReadInt16() - 1);
            }
            if (num3 == num2)
            {
                return 2;
            }
            reader.Close();
            input.Close();
            return 3;
        }

        public bool InjectSoundIntoChoice(string filePath, int ChoiceIndex)
        {
            BinaryWriter writer;
            ugh_.Sound_Choices44_Layout chunk = (ugh_.Sound_Choices44_Layout) this.srl.Sound_Choices.GetChunk(ChoiceIndex);
            FileStream input = new FileStream(filePath, FileMode.Open, FileAccess.Read);
            BinaryReader reader = new BinaryReader(input);
            int num = this.snd.Format.Value;
            int num2 = this.getSoundFormatFromFile(filePath);
            if (num != num2)
            {
                MessageBox.Show("The sound you are injecting is not the correct format.");
                return false;
            }
            int count = 0;
            int remainderSize = 0;
            int length = (int) reader.BaseStream.Length;
            if (num != 2)
            {
                int num6;
                this.getNeededRawCount(length - 60, out count, out remainderSize);
                ugh_.Sound_Permutations35_Layout layout2 = (ugh_.Sound_Permutations35_Layout) this.srl.Sound_Permutations.GetChunk(0);
                for (num6 = 0; num6 < layout2.chunkCount.Value; num6++)
                {
                    ugh_.Sound_Choices44_Layout layout3 = (ugh_.Sound_Choices44_Layout) this.srl.Sound_Choices.GetChunk(num6);
                    if (layout3.Index.Value > chunk.Index.Value)
                    {
                        layout3.Index.Value = (short) (layout3.Index.Value + ((short) ((count - 1) - chunk.chunkCount.Value)));
                    }
                }
                for (num6 = 0; num6 < chunk.chunkCount.Value; num6++)
                {
                    this.srl.Sound_Raw_Data.RemoveChunk(chunk.Index.Value);
                }
                chunk.chunkCount.Value = (short) (count - 1);
                if (num != 2)
                {
                    reader.BaseStream.Position = 60L;
                }
                int num7 = 0;
                for (num6 = 1; num6 <= (count - 1); num6++)
                {
                    int num8 = 0;
                    if ((num6 == count) && (count == 1))
                    {
                        num8 += remainderSize;
                    }
                    else if (num6 == (count - 1))
                    {
                        num8 += 0x7ff8;
                        num8 += remainderSize;
                    }
                    else
                    {
                        num8 += 0x7ff8;
                    }
                    ugh_.Sound_Raw_Data68_Layout layout4 = new ugh_.Sound_Raw_Data68_Layout();
                    layout4.Sound_Raw.MS = new MemoryStream();
                    layout4.Padding.Data = new byte[4];
                    writer = new BinaryWriter(layout4.Sound_Raw.MS);
                    writer.BaseStream.Position = 0L;
                    writer.Write(reader.ReadBytes(num8));
                    layout4.Sound_Raw.Total_Raw_Data_Size = num8;
                    this.srl.Sound_Raw_Data.InsertChunk(layout4, chunk.Index.Value + num7);
                    num7++;
                }
            }
            else
            {
                ugh_.Sound_Raw_Data68_Layout layout5 = (ugh_.Sound_Raw_Data68_Layout) this.srl.Sound_Raw_Data.GetChunk(0);
                layout5.Sound_Raw.MS = new MemoryStream();
                writer = new BinaryWriter(layout5.Sound_Raw.MS);
                int num9 = (int) reader.BaseStream.Length;
                writer.Write(reader.ReadBytes(num9));
                layout5.Sound_Raw.Total_Raw_Data_Size = num9;
            }
            reader.Close();
            input.Close();
            return true;
        }

        private byte[] writeHeader(int Size, byte Format)
        {
            MemoryStream output = new MemoryStream();
            BinaryWriter writer = new BinaryWriter(output);
            writer.Write("RIFF".ToCharArray());
            writer.Write(Convert.ToInt32((long) (Size + 40)));
            writer.Write("WAVEfmt ".ToCharArray());
            writer.Write(Convert.ToInt32(20));
            writer.Write(Convert.ToInt16(0x69));
            writer.Write(Convert.ToInt16(Format));
            writer.Write(Convert.ToInt32(0xac44));
            writer.Write(Convert.ToInt32((int) (0x5622 * Format)));
            writer.Write(Convert.ToInt16((int) (0x24 * Format)));
            writer.Write(Convert.ToInt16(4));
            writer.Write(Convert.ToInt16(2));
            writer.Write(Convert.ToInt16(0x40));
            writer.Write("data".ToCharArray());
            writer.Write(Convert.ToInt32((long) (Size - 4)));
            writer.Close();
            output.Close();
            return output.ToArray();
        }
    }
}

