﻿namespace core
{
    using core.TagInterface;
    using System;
    using System.ComponentModel;
    using System.Globalization;

    public class MetaInterface
    {
        protected TagBlock header;
        protected InterfaceMode mode;
        protected TagDefinition tagdef;
        protected string tagname;
        protected string tagtype;
        protected Map.HaloVersionEnum type;

        public MetaInterface(InterfaceMode mode)
        {
            this.mode = mode;
        }

        public virtual void Read()
        {
        }

        public TagBlock Header
        {
            get
            {
                return this.header;
            }
            set
            {
                this.header = value;
            }
        }

        public InterfaceMode Mode
        {
            get
            {
                return this.mode;
            }
            set
            {
                this.mode = value;
            }
        }

        public TagDefinition Tag_Definition
        {
            get
            {
                return this.tagdef;
            }
            set
            {
                this.tagdef = value;
            }
        }

        public string Tag_Name
        {
            get
            {
                return this.tagname;
            }
            set
            {
                this.tagname = value;
            }
        }

        public string Tag_Type
        {
            get
            {
                return this.tagtype;
            }
            set
            {
                this.tagtype = value;
            }
        }

        public Map.HaloVersionEnum Type
        {
            get
            {
                return this.type;
            }
            set
            {
                this.type = value;
            }
        }

        public class BaseObjectConverter : ExpandableObjectConverter
        {
            public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destType)
            {
                if (destType == typeof(string))
                {
                    if (value is Pointer)
                    {
                        Pointer pointer = (Pointer) value;
                        return "Offsets";
                    }
                    if (value is TagBlockLayout)
                    {
                        TagBlockLayout layout = (TagBlockLayout) value;
                        return layout.Parent.Name;
                    }
                    if (value is TagBlock)
                    {
                        TagBlock block = (TagBlock) value;
                        return block.Name;
                    }
                    if (value is TagReference)
                    {
                        TagReference reference = (TagReference) value;
                        return reference.Name;
                    }
                }
                return "";
            }
        }

        public enum InterfaceMode
        {
            DecompiledTagMode,
            CompiledTagMode
        }

        public class TagReferenceConverter : StringConverter
        {
            public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
            {
                return true;
            }

            public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
            {
                return true;
            }
        }

        public class TagTypeConverter : StringConverter
        {
            public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
            {
                return true;
            }

            public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
            {
                return true;
            }
        }
    }
}

