﻿namespace core
{
    using core.IO;
    using core.TagInterface;
    using System;
    using System.ComponentModel;
    using System.Drawing;
    using System.IO;
    using System.Windows.Forms;

    public class ProgramSettings : Form
    {
        private Button button1;
        private Button button2;
        private Button button3;
        private Button button4;
        private Button button5;
        private CheckBox checkBox1;
        private IContainer components = null;
        private static string decompiledtagspath;
        private FolderBrowserDialog folderBrowserDialog1;
        private GroupBox groupBox1;
        private GroupBox groupBox2;
        private GroupBox groupBox3;
        private GroupBox groupBox4;
        private GroupBox groupBox5;
        private static string halo2_Main_Menu_Path;
        private static string halo2_MP_Shared_Path;
        private static string halo2_SP_Shared_Path;
        private static EndianIO io;
        private static EndianIO mainmenuio;
        private static EndianIO mpsharedio;
        private static OpenFileDialog OpenHaloMapDialog = new OpenFileDialog();
        private static bool overwriteExistingTags;
        private string path;
        private static EndianIO spsharedio;
        private Button TagLayoutManagerButton;
        private TextBox textBox1;
        private TextBox textBox2;
        private TextBox textBox3;
        private TextBox textBox4;

        public ProgramSettings()
        {
            this.InitializeComponent();
            this.path = Application.StartupPath + @"\Program_Settings.data";
            bool flag = false;
            if (File.Exists(this.path))
            {
                flag = true;
            }
            io = new EndianIO(this.path, EndianType.LittleEndian);
            if (flag)
            {
                this.LoadSettings();
                CreateIOs();
            }
            else
            {
                decompiledtagspath = @"C:\Tags";
                overwriteExistingTags = true;
                Halo2_Main_Menu_Path = "";
                Halo2_MP_Shared_Path = "";
                Halo2_SP_Shared_Path = "";
                SaveSettings();
                this.LoadSettings();
            }
            SP_Shared_IO.Open();
            Main_Menu_IO.Open();
            MP_Shared_IO.Open();
            OpenHaloMapDialog.Filter = "Map (*.map) | *.map";
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (this.folderBrowserDialog1.ShowDialog() != DialogResult.Cancel)
            {
                DecompiledTagsPath = this.folderBrowserDialog1.SelectedPath;
                this.textBox1.Text = this.folderBrowserDialog1.SelectedPath;
                SaveSettings();
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            OpenHaloMapDialog.FileName = "mainmenu.map";
            if (OpenHaloMapDialog.ShowDialog() != DialogResult.Cancel)
            {
                Halo2_Main_Menu_Path = OpenHaloMapDialog.FileName;
                this.textBox2.Text = OpenHaloMapDialog.FileName;
                SaveSettings();
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            OpenHaloMapDialog.FileName = "single_player_shared.map";
            if (OpenHaloMapDialog.ShowDialog() != DialogResult.Cancel)
            {
                Halo2_SP_Shared_Path = OpenHaloMapDialog.FileName;
                this.textBox3.Text = OpenHaloMapDialog.FileName;
                SaveSettings();
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            OpenHaloMapDialog.FileName = "shared.map";
            if (OpenHaloMapDialog.ShowDialog() != DialogResult.Cancel)
            {
                Halo2_MP_Shared_Path = OpenHaloMapDialog.FileName;
                this.textBox4.Text = OpenHaloMapDialog.FileName;
                SaveSettings();
            }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            Tag_Layouts.Layouts = new Tag_Layouts();
            MessageBox.Show("Done");
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            OverwriteExistingTags = this.checkBox1.Checked;
            SaveSettings();
        }

        public static void Create_Main_Menu_IO()
        {
            if (!(((Halo2_Main_Menu_Path != null) && (Halo2_Main_Menu_Path != "")) && File.Exists(Halo2_Main_Menu_Path)))
            {
                OpenHaloMapDialog.FileName = "mainmenu.map";
                if (OpenHaloMapDialog.ShowDialog() == DialogResult.Cancel)
                {
                    return;
                }
                Halo2_Main_Menu_Path = OpenHaloMapDialog.FileName;
            }
            mainmenuio = new EndianIO(Halo2_Main_Menu_Path, EndianType.LittleEndian);
        }

        public static void Create_MP_Shared_IO()
        {
            if (!(((Halo2_MP_Shared_Path != null) && (Halo2_Main_Menu_Path != "")) && File.Exists(Halo2_MP_Shared_Path)))
            {
                OpenHaloMapDialog.FileName = "shared.map";
                if (OpenHaloMapDialog.ShowDialog() == DialogResult.Cancel)
                {
                    return;
                }
                Halo2_MP_Shared_Path = OpenHaloMapDialog.FileName;
            }
            mpsharedio = new EndianIO(Halo2_MP_Shared_Path, EndianType.LittleEndian);
        }

        public static void Create_SP_Shared_IO()
        {
            if (!(!((Halo2_SP_Shared_Path == null) | (Halo2_Main_Menu_Path == "")) && File.Exists(Halo2_SP_Shared_Path)))
            {
                OpenHaloMapDialog.FileName = "single_player_shared.map";
                if (OpenHaloMapDialog.ShowDialog() == DialogResult.Cancel)
                {
                    return;
                }
                Halo2_SP_Shared_Path = OpenHaloMapDialog.FileName;
            }
            spsharedio = new EndianIO(Halo2_SP_Shared_Path, EndianType.LittleEndian);
        }

        public static void CreateIOs()
        {
            Create_Main_Menu_IO();
            Create_MP_Shared_IO();
            Create_SP_Shared_IO();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            this.textBox1 = new TextBox();
            this.folderBrowserDialog1 = new FolderBrowserDialog();
            this.button1 = new Button();
            this.groupBox1 = new GroupBox();
            this.groupBox2 = new GroupBox();
            this.checkBox1 = new CheckBox();
            this.groupBox3 = new GroupBox();
            this.textBox2 = new TextBox();
            this.button2 = new Button();
            this.groupBox4 = new GroupBox();
            this.textBox3 = new TextBox();
            this.button3 = new Button();
            this.groupBox5 = new GroupBox();
            this.textBox4 = new TextBox();
            this.button4 = new Button();
            this.TagLayoutManagerButton = new Button();
            this.button5 = new Button();
            this.groupBox1.SuspendLayout();
            this.groupBox2.SuspendLayout();
            this.groupBox3.SuspendLayout();
            this.groupBox4.SuspendLayout();
            this.groupBox5.SuspendLayout();
            base.SuspendLayout();
            this.textBox1.Location = new Point(6, 0x13);
            this.textBox1.Name = "textBox1";
            this.textBox1.ReadOnly = true;
            this.textBox1.Size = new Size(0xf7, 20);
            this.textBox1.TabIndex = 0;
            this.textBox1.Text = @"C:\Tags";
            this.button1.Location = new Point(0xb1, 0x2d);
            this.button1.Name = "button1";
            this.button1.Size = new Size(0x4b, 0x17);
            this.button1.TabIndex = 2;
            this.button1.Text = "Browse";
            this.button1.UseVisualStyleBackColor = true;
            this.button1.Click += new EventHandler(this.button1_Click);
            this.groupBox1.BackColor = Color.WhiteSmoke;
            this.groupBox1.Controls.Add(this.textBox1);
            this.groupBox1.Controls.Add(this.button1);
            this.groupBox1.Location = new Point(6, 2);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new Size(0x102, 0x4b);
            this.groupBox1.TabIndex = 3;
            this.groupBox1.TabStop = false;
            this.groupBox1.Text = "Decompiled Tags Path";
            this.groupBox2.Controls.Add(this.checkBox1);
            this.groupBox2.Location = new Point(6, 0x53);
            this.groupBox2.Name = "groupBox2";
            this.groupBox2.Size = new Size(0x102, 0x25);
            this.groupBox2.TabIndex = 4;
            this.groupBox2.TabStop = false;
            this.groupBox2.Text = "Overwrite Tags";
            this.checkBox1.AutoSize = true;
            this.checkBox1.Checked = true;
            this.checkBox1.CheckState = CheckState.Checked;
            this.checkBox1.Location = new Point(0x1c, 14);
            this.checkBox1.Name = "checkBox1";
            this.checkBox1.Size = new Size(0xc4, 0x11);
            this.checkBox1.TabIndex = 0;
            this.checkBox1.Text = "Overwrite Existing Decompiled Tags";
            this.checkBox1.UseVisualStyleBackColor = true;
            this.checkBox1.CheckedChanged += new EventHandler(this.checkBox1_CheckedChanged);
            this.groupBox3.BackColor = Color.WhiteSmoke;
            this.groupBox3.Controls.Add(this.textBox2);
            this.groupBox3.Controls.Add(this.button2);
            this.groupBox3.Location = new Point(6, 0x7e);
            this.groupBox3.Name = "groupBox3";
            this.groupBox3.Size = new Size(0x102, 0x4b);
            this.groupBox3.TabIndex = 5;
            this.groupBox3.TabStop = false;
            this.groupBox3.Text = "Halo 2 Main Menu Map Path";
            this.textBox2.Location = new Point(6, 0x13);
            this.textBox2.Name = "textBox2";
            this.textBox2.ReadOnly = true;
            this.textBox2.Size = new Size(0xf7, 20);
            this.textBox2.TabIndex = 0;
            this.textBox2.Text = "<Not Set>";
            this.button2.Location = new Point(0xb1, 0x2d);
            this.button2.Name = "button2";
            this.button2.Size = new Size(0x4b, 0x17);
            this.button2.TabIndex = 2;
            this.button2.Text = "Browse";
            this.button2.UseVisualStyleBackColor = true;
            this.button2.Click += new EventHandler(this.button2_Click);
            this.groupBox4.BackColor = Color.WhiteSmoke;
            this.groupBox4.Controls.Add(this.textBox3);
            this.groupBox4.Controls.Add(this.button3);
            this.groupBox4.Location = new Point(6, 0xcf);
            this.groupBox4.Name = "groupBox4";
            this.groupBox4.Size = new Size(0x102, 0x4b);
            this.groupBox4.TabIndex = 6;
            this.groupBox4.TabStop = false;
            this.groupBox4.Text = "Halo 2 SP Shared Map Path";
            this.textBox3.Location = new Point(6, 0x13);
            this.textBox3.Name = "textBox3";
            this.textBox3.ReadOnly = true;
            this.textBox3.Size = new Size(0xf7, 20);
            this.textBox3.TabIndex = 0;
            this.textBox3.Text = "<Not Set>";
            this.button3.Location = new Point(0xb1, 0x2d);
            this.button3.Name = "button3";
            this.button3.Size = new Size(0x4b, 0x17);
            this.button3.TabIndex = 2;
            this.button3.Text = "Browse";
            this.button3.UseVisualStyleBackColor = true;
            this.button3.Click += new EventHandler(this.button3_Click);
            this.groupBox5.BackColor = Color.WhiteSmoke;
            this.groupBox5.Controls.Add(this.textBox4);
            this.groupBox5.Controls.Add(this.button4);
            this.groupBox5.Location = new Point(6, 0x120);
            this.groupBox5.Name = "groupBox5";
            this.groupBox5.Size = new Size(0x102, 0x4b);
            this.groupBox5.TabIndex = 7;
            this.groupBox5.TabStop = false;
            this.groupBox5.Text = "Halo 2 MP Shared Map Path";
            this.textBox4.Location = new Point(6, 0x13);
            this.textBox4.Name = "textBox4";
            this.textBox4.ReadOnly = true;
            this.textBox4.Size = new Size(0xf7, 20);
            this.textBox4.TabIndex = 0;
            this.textBox4.Text = "<Not Set>";
            this.button4.Location = new Point(0xb1, 0x2d);
            this.button4.Name = "button4";
            this.button4.Size = new Size(0x4b, 0x17);
            this.button4.TabIndex = 2;
            this.button4.Text = "Browse";
            this.button4.UseVisualStyleBackColor = true;
            this.button4.Click += new EventHandler(this.button4_Click);
            this.TagLayoutManagerButton.Location = new Point(6, 0x170);
            this.TagLayoutManagerButton.Name = "TagLayoutManagerButton";
            this.TagLayoutManagerButton.Size = new Size(0x102, 0x17);
            this.TagLayoutManagerButton.TabIndex = 8;
            this.TagLayoutManagerButton.Text = "Tag Layout Manager";
            this.TagLayoutManagerButton.UseVisualStyleBackColor = true;
            this.TagLayoutManagerButton.Click += new EventHandler(this.TagLayoutManagerButton_Click);
            this.button5.Location = new Point(6, 0x18d);
            this.button5.Name = "button5";
            this.button5.Size = new Size(0x102, 0x17);
            this.button5.TabIndex = 9;
            this.button5.Text = "Refresh Tag Layouts";
            this.button5.UseVisualStyleBackColor = true;
            this.button5.Click += new EventHandler(this.button5_Click);
            base.AutoScaleDimensions = new SizeF(6f, 13f);
            base.AutoScaleMode = AutoScaleMode.Font;
            this.BackColor = Color.WhiteSmoke;
            base.ClientSize = new Size(0x10f, 0x1b4);
            base.Controls.Add(this.button5);
            base.Controls.Add(this.TagLayoutManagerButton);
            base.Controls.Add(this.groupBox5);
            base.Controls.Add(this.groupBox4);
            base.Controls.Add(this.groupBox3);
            base.Controls.Add(this.groupBox2);
            base.Controls.Add(this.groupBox1);
            base.FormBorderStyle = FormBorderStyle.FixedToolWindow;
            base.Name = "ProgramSettings";
            this.Text = "Settings";
            this.groupBox1.ResumeLayout(false);
            this.groupBox1.PerformLayout();
            this.groupBox2.ResumeLayout(false);
            this.groupBox2.PerformLayout();
            this.groupBox3.ResumeLayout(false);
            this.groupBox3.PerformLayout();
            this.groupBox4.ResumeLayout(false);
            this.groupBox4.PerformLayout();
            this.groupBox5.ResumeLayout(false);
            this.groupBox5.PerformLayout();
            base.ResumeLayout(false);
        }

        public void LoadSettings()
        {
            io.Open();
            io.In.SeekTo(0);
            DecompiledTagsPath = core.TagInterface.String.ReadString(ref io.In, 0x400);
            this.textBox1.Text = DecompiledTagsPath;
            Halo2_Main_Menu_Path = core.TagInterface.String.ReadString(ref io.In, 0x400);
            this.textBox2.Text = Halo2_Main_Menu_Path;
            io.Open();
            Halo2_SP_Shared_Path = core.TagInterface.String.ReadString(ref io.In, 0x400);
            this.textBox3.Text = Halo2_SP_Shared_Path;
            io.Open();
            Halo2_MP_Shared_Path = core.TagInterface.String.ReadString(ref io.In, 0x400);
            this.textBox4.Text = Halo2_MP_Shared_Path;
            io.Open();
            OverwriteExistingTags = io.In.ReadBoolean();
            this.checkBox1.Checked = OverwriteExistingTags;
            io.Close();
            SaveSettings();
        }

        public static void SaveSettings()
        {
            io.Open();
            io.Out.SeekTo(0);
            core.TagInterface.String.WriteString(ref io.Out, DecompiledTagsPath, 0x400);
            core.TagInterface.String.WriteString(ref io.Out, Halo2_Main_Menu_Path, 0x400);
            core.TagInterface.String.WriteString(ref io.Out, Halo2_SP_Shared_Path, 0x400);
            core.TagInterface.String.WriteString(ref io.Out, Halo2_MP_Shared_Path, 0x400);
            io.Out.Write(OverwriteExistingTags);
            io.Close();
        }

        private void TagLayoutManagerButton_Click(object sender, EventArgs e)
        {
            Tag_Layout_Manager.Display();
        }

        public static string DecompiledTagsPath
        {
            get
            {
                return decompiledtagspath;
            }
            set
            {
                decompiledtagspath = value;
            }
        }

        public static string Halo2_Main_Menu_Path
        {
            get
            {
                return halo2_Main_Menu_Path;
            }
            set
            {
                halo2_Main_Menu_Path = value;
                Create_Main_Menu_IO();
            }
        }

        public static string Halo2_MP_Shared_Path
        {
            get
            {
                return halo2_MP_Shared_Path;
            }
            set
            {
                halo2_MP_Shared_Path = value;
                Create_MP_Shared_IO();
            }
        }

        public static string Halo2_SP_Shared_Path
        {
            get
            {
                return halo2_SP_Shared_Path;
            }
            set
            {
                halo2_SP_Shared_Path = value;
                Create_SP_Shared_IO();
            }
        }

        public static EndianIO Main_Menu_IO
        {
            get
            {
                return mainmenuio;
            }
        }

        public static EndianIO MP_Shared_IO
        {
            get
            {
                return mpsharedio;
            }
        }

        public static bool OverwriteExistingTags
        {
            get
            {
                return overwriteExistingTags;
            }
            set
            {
                overwriteExistingTags = value;
            }
        }

        public static EndianIO SP_Shared_IO
        {
            get
            {
                return spsharedio;
            }
        }
    }
}

