﻿namespace core
{
    using core.H2;
    using core.IO;
    using core.TagInterface;
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Windows.Forms;

    public sealed class TagDecompiler
    {
        private static EndianIO io;
        private static bool overwrite;
        private static List<int> tagnumbers = new List<int>();
        private static Map tempmap;
        private static ugh_ ugh;

        public static void Decompile(Meta m, bool overwrite)
        {
            string path = TagPath(ref m);
            if (overwrite || !File.Exists(path))
            {
                DecompileRoutine(path, m, overwrite);
            }
        }

        public static void Decompile(Meta m, bool overwrite, Map map)
        {
            tempmap = map;
            string path = TagPath(ref m);
            if (overwrite || !File.Exists(path))
            {
                DecompileRoutine(path, m, overwrite);
            }
        }

        public static void DecompileAs(Meta m)
        {
            string decompiledTagsPath = ProgramSettings.DecompiledTagsPath;
            if (decompiledTagsPath[decompiledTagsPath.Length - 1] != '\\')
            {
                decompiledTagsPath = decompiledTagsPath + @"\";
            }
            int num = m.Name.LastIndexOf('\\');
            switch (m.HaloVersion)
            {
                case Map.HaloVersionEnum.Halo2:
                    decompiledTagsPath = decompiledTagsPath + @"H2\";
                    break;

                case Map.HaloVersionEnum.HaloCE:
                    decompiledTagsPath = decompiledTagsPath + @"H1\";
                    break;
            }
            decompiledTagsPath = decompiledTagsPath + m.Name.Substring(0, num + 1);
            if (!Directory.Exists(decompiledTagsPath))
            {
                Directory.CreateDirectory(decompiledTagsPath);
            }
            string str2 = (decompiledTagsPath + m.Name.Substring(num + 1)) + "." + m.Interface.Tag_Definition.Extension;
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.FileName = str2;
            dialog.Filter = m.Interface.Tag_Definition.Extension + "|*." + m.Interface.Tag_Definition.Extension;
            dialog.ShowDialog();
            DecompileRoutine(dialog.FileName, m, true);
        }

        public static void DecompileMap(Map map, bool overwrite)
        {
            tagnumbers = new List<int>();
            tempmap = map;
            TagDecompiler.overwrite = overwrite;
            ugh = map.ugh_Interface;
            for (int i = 0; i < map.Index_Header.Meta_Count; i++)
            {
                if (map.Meta[i].TagType != "ugh!")
                {
                    Meta meta = new Meta(i, ref map);
                    if (meta.TagType == "sbsp")
                    {
                        SaveBSPScenery(PSRTagPath(ref meta), ref meta);
                        Decompile(meta, overwrite, map);
                    }
                    else
                    {
                        Decompile(meta, overwrite);
                        meta = null;
                    }
                }
            }
        }

        public static void DecompileRecursive(Meta m, Map map, bool overwrite)
        {
            tagnumbers = new List<int>();
            tempmap = map;
            TagDecompiler.overwrite = overwrite;
            ugh = map.ugh_Interface;
            Decompile(m, overwrite);
            tagnumbers.Add(m.tagnumber);
            if (m.TagType == "sbsp")
            {
                SaveBSPScenery(PSRTagPath(ref m), ref m);
            }
            DecompileRecursivelyRoutine(m.Interface.Header);
        }

        private static void DecompileRecursivelyRoutine(TagBlock tagblock)
        {
            Stack<TagBlock> stack = new Stack<TagBlock>();
            stack.Push(tagblock);
            while (stack.Count > 0)
            {
                TagBlock block = stack.Pop();
                Application.DoEvents();
                foreach (TagBlockLayout layout in block.chunks)
                {
                    foreach (BaseObject obj2 in layout)
                    {
                        Meta meta;
                        if (obj2.Type == BaseObjectType.TagReference)
                        {
                            TagReference reference = (TagReference) obj2;
                            if (reference.Ident.TagName == "null")
                            {
                                continue;
                            }
                            if (tagnumbers.IndexOf(reference.Ident.TagNumber) == -1)
                            {
                                tagnumbers.Add(reference.Ident.TagNumber);
                                meta = new Meta(reference.Ident.TagNumber, ref tempmap);
                                if ((meta.TagType == "sbsp") && (tempmap != null))
                                {
                                    SaveBSPScenery(PSRTagPath(ref meta), ref meta);
                                }
                                Decompile(meta, overwrite);
                                stack.Push(meta.Interface.Header);
                                meta = null;
                            }
                        }
                        else
                        {
                            TagIdent ident;
                            if (obj2.Type == BaseObjectType.TagIdent)
                            {
                                ident = (TagIdent) obj2;
                                if (ident.TagName == "null")
                                {
                                    continue;
                                }
                                if (tagnumbers.IndexOf(ident.TagNumber) == -1)
                                {
                                    tagnumbers.Add(ident.TagNumber);
                                    meta = new Meta(ident.TagNumber, ref tempmap);
                                    if ((meta.TagType == "sbsp") && (tempmap != null))
                                    {
                                        SaveBSPScenery(PSRTagPath(ref meta), ref meta);
                                    }
                                    Decompile(meta, overwrite);
                                    stack.Push(meta.Interface.Header);
                                    meta = null;
                                }
                            }
                            else if (obj2.Type == BaseObjectType.Syntax_Block)
                            {
                                ident = ((Syntax_Block) obj2).Tag_Reference;
                                if (ident.TagName == "null")
                                {
                                    continue;
                                }
                                if (tagnumbers.IndexOf(ident.TagNumber) == -1)
                                {
                                    tagnumbers.Add(ident.TagNumber);
                                    meta = new Meta(ident.TagNumber, ref tempmap);
                                    if ((meta.TagType == "sbsp") && (tempmap != null))
                                    {
                                        SaveBSPScenery(PSRTagPath(ref meta), ref meta);
                                    }
                                    Decompile(meta, overwrite);
                                    stack.Push(meta.Interface.Header);
                                    meta = null;
                                }
                            }
                            else if (obj2.Type == BaseObjectType.TagBlock)
                            {
                                stack.Push((TagBlock) obj2);
                            }
                        }
                    }
                }
            }
        }

        private static void DecompileRoutine(string tagpath, Meta m, bool overwrite)
        {
            if (tagpath == "G:\\XBox\\Halo 2\\Halo 2 Programs Source Code\\H2Core Decompile\\H2\\sound\\vehicles\\phantom\\phantom_turret_fire_c_ap.sound")
            {
                bool test = false;
            }

            io = new EndianIO(tagpath, EndianType.BigEndian);
            io.Open();
            io.Out.SeekTo(0);
            io.Out.Write((int) m.HaloVersion);
            core.TagInterface.Padding.Write(ref io.Out, 32);
            core.TagInterface.String.WriteString(ref io.Out, m.TagType, 4);
            io.Out.Write(0);
            io.Out.Write(0x40);
            core.TagInterface.Padding.Write(ref io.Out, 8);
            io.Out.Write(m.Interface.Tag_Definition.ID);
            io.Out.Write((byte) 0);
            io.Out.Write((byte) 0xff);
            core.TagInterface.String.WriteString(ref io.Out, "blam", 4);
            RecursivelyWriteElements(m.Interface.Header, ref io.Out);
            io.Close();
            if (m.tagType == "snd!")
            {
                SaveSoundResourceLayout(SRLTagPath(ref m), ref m);
            }
        }

        private static string PSRTagPath(ref Meta m)
        {
            string decompiledTagsPath = ProgramSettings.DecompiledTagsPath;
            if (decompiledTagsPath[decompiledTagsPath.Length - 1] != '\\')
            {
                decompiledTagsPath = decompiledTagsPath + @"\";
            }
            int num = m.Name.LastIndexOf('\\');
            switch (m.HaloVersion)
            {
                case Map.HaloVersionEnum.Halo2:
                    decompiledTagsPath = decompiledTagsPath + @"H2\";
                    break;

                case Map.HaloVersionEnum.HaloCE:
                    decompiledTagsPath = decompiledTagsPath + @"H1\";
                    break;
            }
            decompiledTagsPath = decompiledTagsPath + m.Name.Substring(0, num + 1);
            if (!Directory.Exists(decompiledTagsPath))
            {
                Directory.CreateDirectory(decompiledTagsPath);
            }
            return (decompiledTagsPath + m.Name.Substring(num + 1) + ".psr");
        }

        public static void RecursivelyWriteElements(TagBlock tagblock, ref EndianIO.EndianWriter bw)
        {
            Stack<TagBlock> stack = new Stack<TagBlock>();
            stack.Push(tagblock);
            while (stack.Count > 0)
            {
                TagBlock block = stack.Pop();
                Application.DoEvents();
                if (block.ChunkCount != 0)
                {
                    block.WriteChunks(ref bw, MetaInterface.InterfaceMode.DecompiledTagMode);
                    foreach (TagBlockLayout layout in block.chunks)
                    {
                        foreach (BaseObject obj2 in layout)
                        {
                            TagBlock block3;
                            if (obj2.Type == BaseObjectType.Raw_Tag_Block)
                            {
                                Raw_Tag_Block block2 = (Raw_Tag_Block) obj2;
                                block3 = block2.Resource_Block;
                                stack.Push(block3);
                                if (block2.Raw_Offset.Data_Offset == -1)
                                {
                                    continue;
                                }
                                block2.Write_Raw(ref bw);
                            }
                            else
                            {
                                Data data;
                                if (obj2.Type == BaseObjectType.Data)
                                {
                                    data = (Data) obj2;
                                    data.Write_Raw(ref bw);
                                }
                                else if (obj2.Type == BaseObjectType.Sound_Raw)
                                {
                                    data = (Data) obj2;
                                    data.Write_Raw(ref bw);
                                }
                                else if (obj2.Type == BaseObjectType.Animation_Raw)
                                {
                                    ((Data) obj2).Write_Raw(ref bw);
                                }
                                else if (obj2.Type == BaseObjectType.Bitmap_Raw)
                                {
                                    Bitmap_Raw raw = (Bitmap_Raw) obj2;
                                    for (int i = 0; i < 6; i++)
                                    {
                                        if (raw.LOD_Raw_Data_Offset[i].Data_Offset != -1)
                                        {
                                            raw.Write_LOD_Raw(ref bw, i);
                                        }
                                    }
                                }
                                else if (obj2.Type == BaseObjectType.TagBlock)
                                {
                                    block3 = (TagBlock) obj2;
                                    stack.Push(block3);
                                }
                            }
                        }
                    }
                }
            }
        }

        private static bool SaveBSPScenery(string tagpath, ref Meta meta)
        {
            int num2;
            int current;
            Meta meta3;
            int num4;
            Meta meta2 = new Meta(3, ref tempmap);
            scnr chunk = (scnr) meta2.Interface.Header.GetChunk(0);
            int num = 0;
            foreach (scnr.Structure_BSPS_Layout layout in chunk.Structure_BSPS.Chunks)
            {
                if ((layout.Structure_BSP_0.TagName == meta.Name) && (layout.Structure_BSP_0.TagType == meta.TagType))
                {
                    break;
                }
                num++;
            }
            EndianIO nio = new EndianIO(tagpath, EndianType.BigEndian);
            nio.Open();
            nio.Out.SeekTo(0);
            Predicted_Scenario_Resources tagblocklayout = new Predicted_Scenario_Resources();
            List<int> list = new List<int>();
            foreach (scnr.Scenery_Placement_Layout layout2 in chunk.Scenery_Placement.Chunks)
            {
                if (layout2.Scenery_Chunk__Number_.Value == -1)
                {
                    continue;
                }
                if ((layout2.bsp_index_1.Value == num) || (layout2.bsp_index_2.Value == num))
                {
                    num2 = list.IndexOf(layout2.Scenery_Chunk__Number_.Value);
                    if (num2 == -1)
                    {
                        list.Add(layout2.Scenery_Chunk__Number_.Value);
                        layout2.Scenery_Chunk__Number_.Value = (short) (list.Count - 1);
                    }
                    else
                    {
                        layout2.Scenery_Chunk__Number_.Value = (short) num2;
                    }
                    tagblocklayout.scenery_placement.AddChunk(layout2);
                }
            }
            for (current = 0; current < list.Count; current++)
            {
                scnr.Scenery_Palette_Layout layout3 = (scnr.Scenery_Palette_Layout) chunk.Scenery_Palette.Chunks[list[current]];
                tagblocklayout.scenery_pallete.AddChunk(layout3);
                if (layout3.Scenery_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout3.Scenery_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
            }
            list = new List<int>();
            List<int> list2 = new List<int>();
            foreach (scnr.Machine_Placement_Layout layout4 in chunk.Machine_Placement.Chunks)
            {
                if (layout4.Palette_Chunk__Number_.Value == -1)
                {
                    continue;
                }
                if (layout4.bsp_index_1.Value == num)
                {
                    num2 = list.IndexOf(layout4.Palette_Chunk__Number_.Value);
                    if (num2 == -1)
                    {
                        list.Add(layout4.Palette_Chunk__Number_.Value);
                        layout4.Palette_Chunk__Number_.Value = (short) (list.Count - 1);
                    }
                    else
                    {
                        layout4.Palette_Chunk__Number_.Value = (short) num2;
                    }
                    if (layout4.Device_Group_Chunk__Number_.Value != -1)
                    {
                        num2 = list2.IndexOf(layout4.Device_Group_Chunk__Number_.Value);
                        if (num2 == -1)
                        {
                            list2.Add(layout4.Device_Group_Chunk__Number_.Value);
                            layout4.Device_Group_Chunk__Number_.Value = (short) (list2.Count - 1);
                        }
                        else
                        {
                            layout4.Device_Group_Chunk__Number_.Value = (short) num2;
                        }
                    }
                    tagblocklayout.machine_placement.AddChunk(layout4);
                }
            }
            for (current = 0; current < list.Count; current++)
            {
                scnr.Machine_Palette_Layout layout5 = (scnr.Machine_Palette_Layout) chunk.Machine_Palette.Chunks[list[current]];
                tagblocklayout.machine_pallete.AddChunk(layout5);
                if (layout5.Machine_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout5.Machine_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
            }
            list = new List<int>();
            foreach (scnr.Control_Placement_Layout layout6 in chunk.Control_Placement.Chunks)
            {
                if (layout6.Palette_Chunk__Number_.Value == -1)
                {
                    continue;
                }
                if (layout6.bsp_index_1.Value == num)
                {
                    num2 = list.IndexOf(layout6.Palette_Chunk__Number_.Value);
                    if (num2 == -1)
                    {
                        list.Add(layout6.Palette_Chunk__Number_.Value);
                        layout6.Palette_Chunk__Number_.Value = (short) (list.Count - 1);
                    }
                    else
                    {
                        layout6.Palette_Chunk__Number_.Value = (short) num2;
                    }
                    if (layout6.Device_Group_Chunk__Number_.Value != -1)
                    {
                        num2 = list2.IndexOf(layout6.Device_Group_Chunk__Number_.Value);
                        if (num2 == -1)
                        {
                            list2.Add(layout6.Device_Group_Chunk__Number_.Value);
                            layout6.Device_Group_Chunk__Number_.Value = (short) (list2.Count - 1);
                        }
                        else
                        {
                            layout6.Device_Group_Chunk__Number_.Value = (short) num2;
                        }
                    }
                    tagblocklayout.control_placement.AddChunk(layout6);
                }
            }
            for (current = 0; current < list.Count; current++)
            {
                scnr.Control_Palette_Layout layout7 = (scnr.Control_Palette_Layout) chunk.Control_Palette.Chunks[list[current]];
                tagblocklayout.control_pallete.AddChunk(layout7);
                if (layout7.Control_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout7.Control_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
            }
            for (current = 0; current < list2.Count; current++)
            {
                scnr.Device_Groups_Layout layout8 = (scnr.Device_Groups_Layout) chunk.Device_Groups.Chunks[list2[current]];
                tagblocklayout.device_groups.AddChunk(layout8);
            }
            list = new List<int>();
            foreach (scnr.Obstacle_Placement_Layout layout9 in chunk.Obstacle_Placement.Chunks)
            {
                if (layout9.Chunk_Number == 100)
                {
                }
                if ((layout9.Palette_Chunk__Number_.Value != -1) && (layout9.bsp_index_1.Value == num))
                {
                    num2 = list.IndexOf(layout9.Palette_Chunk__Number_.Value);
                    if (num2 == -1)
                    {
                        list.Add(layout9.Palette_Chunk__Number_.Value);
                        layout9.Palette_Chunk__Number_.Value = (short) (list.Count - 1);
                    }
                    else
                    {
                        layout9.Palette_Chunk__Number_.Value = (short) num2;
                    }
                    tagblocklayout.obstacle_placement.AddChunk(layout9);
                }
            }
            for (current = 0; current < list.Count; current++)
            {
                scnr.Obstacle_Palette_Layout layout10 = (scnr.Obstacle_Palette_Layout) chunk.Obstacle_Palette.Chunks[list[current]];
                tagblocklayout.obstacle_pallete.AddChunk(layout10);
                if (layout10.Obstacle_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout10.Obstacle_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
            }
            list = new List<int>();
            foreach (scnr.Vehicle_Placement_Layout layout11 in chunk.Vehicle_Placement.Chunks)
            {
                if (layout11.Vehicle_Chunk__Number_.Value == -1)
                {
                    continue;
                }
                if (layout11.bsp_index_1.Value == num)
                {
                    num2 = list.IndexOf(layout11.Vehicle_Chunk__Number_.Value);
                    if (num2 == -1)
                    {
                        list.Add(layout11.Vehicle_Chunk__Number_.Value);
                        layout11.Vehicle_Chunk__Number_.Value = (short) (list.Count - 1);
                    }
                    else
                    {
                        layout11.Vehicle_Chunk__Number_.Value = (short) num2;
                    }
                    tagblocklayout.vehicle_placement.AddChunk(layout11);
                }
            }
            for (current = 0; current < list.Count; current++)
            {
                scnr.Vehicle_Palette_Layout layout12 = (scnr.Vehicle_Palette_Layout) chunk.Vehicle_Palette.Chunks[list[current]];
                tagblocklayout.vehicle_pallete.AddChunk(layout12);
                if (layout12.Vehicle_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout12.Vehicle_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
            }
            list = new List<int>();
            foreach (scnr.Weapon_Placement_Layout layout13 in chunk.Weapon_Placement.Chunks)
            {
                if (layout13.Palette_Chunk__Number_.Value == -1)
                {
                    continue;
                }
                if (layout13.bsp_index_1.Value == num)
                {
                    num2 = list.IndexOf(layout13.Palette_Chunk__Number_.Value);
                    if (num2 == -1)
                    {
                        list.Add(layout13.Palette_Chunk__Number_.Value);
                        layout13.Palette_Chunk__Number_.Value = (short) (list.Count - 1);
                    }
                    else
                    {
                        layout13.Palette_Chunk__Number_.Value = (short) num2;
                    }
                    tagblocklayout.weapon_placement.AddChunk(layout13);
                }
            }
            for (current = 0; current < list.Count; current++)
            {
                scnr.Weapon_Palette_Layout layout14 = (scnr.Weapon_Palette_Layout) chunk.Weapon_Palette.Chunks[list[current]];
                tagblocklayout.weapon_pallete.AddChunk(layout14);
                if (layout14.Weapon_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout14.Weapon_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
            }
            SBSP sbsp = (SBSP) meta.Interface.Header.Chunks[0];
            list = new List<int>();
            foreach (scnr.Sound_Scenery_Placement_Layout layout15 in chunk.Sound_Scenery_Placement.Chunks)
            {
                if (layout15.palette_index.Value == -1)
                {
                    continue;
                }
                if (layout15.bsp_index.Value == num)
                {
                    num2 = list.IndexOf(layout15.palette_index.Value);
                    if (num2 == -1)
                    {
                        list.Add(layout15.palette_index.Value);
                        layout15.palette_index.Value = (short) (list.Count - 1);
                    }
                    else
                    {
                        layout15.palette_index.Value = (short) num2;
                    }
                    tagblocklayout.sound_scenery_placement.AddChunk(layout15);
                }
            }
            current = 0;
            while (current < list.Count)
            {
                scnr.Sound_Scenery_Palette_Layout layout16 = (scnr.Sound_Scenery_Palette_Layout) chunk.Sound_Scenery_Palette.Chunks[list[current]];
                tagblocklayout.sound_scenery_palette.AddChunk(layout16);
                if (layout16.Sound_Scenery_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout16.Sound_Scenery_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
                current++;
            }
            list = new List<int>();
            foreach (SBSP.Background_Sound_Palette sound in sbsp.background_sound_palette.Chunks)
            {
                foreach (scnr.Background_Sound_Palette_Layout layout17 in chunk.Background_Sound_Palette.Chunks)
                {
                    if (!(layout17.Name.Value != sound.name.Value))
                    {
                        tagblocklayout.looping_sounds.AddChunk(layout17);
                    }
                }
            }
            foreach (scnr.Light_Volumes_Palette_Layout layout18 in chunk.Light_Volumes_Palette.Chunks)
            {
                if (layout18.Light_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout18.Light_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
                tagblocklayout.light_pallete.AddChunk(layout18);
            }
            foreach (scnr.Light_Volumes_Placement_Layout layout19 in chunk.Light_Volumes_Placement.Chunks)
            {
                tagblocklayout.light_placement.AddChunk(layout19);
            }
            List<int> list3 = new List<int>();
            List<int> list4 = new List<int>();
            foreach (SBSP.Detail_Objects cluster in sbsp.detail_objects.Chunks)
            {
                if (cluster.sky_index.Value == 0xff)
                {
                    list4.Add(-1);
                    continue;
                }
                num4 = list3.IndexOf(cluster.sky_index.Value);
                if (num4 == -1)
                {
                    list3.Add(cluster.sky_index.Value);
                    scnr.Sky_Palette_Layout layout20 = (scnr.Sky_Palette_Layout) chunk.Sky_Palette.Chunks[cluster.sky_index.Value];
                    tagblocklayout.sky_pallete.AddChunk(layout20);
                    list4.Add(tagblocklayout.sky_pallete.ChunkCount - 1);
                    meta3 = new Meta(layout20.Sky_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                }
                else
                {
                    list4.Add(num4);
                }
            }
            using (List<int>.Enumerator TagEnumerator3 = list4.GetEnumerator())
            {
                while (TagEnumerator3.MoveNext())
                {
                    current = TagEnumerator3.Current;
                    Predicted_Scenario_Resources.Sky_Cluster_Index index = new Predicted_Scenario_Resources.Sky_Cluster_Index();
                    index.Index.Value = (byte) current;
                    tagblocklayout.sky_indexes.AddChunk(index);
                }
            }
            foreach (scnr.Decal_Palette_Layout layout21 in chunk.Decal_Palette.Chunks)
            {
                meta3 = new Meta(layout21.Decal_0.TagNumber, ref tempmap);
                Decompile(meta3, overwrite);
                DecompileRecursivelyRoutine(meta3.Interface.Header);
                tagblocklayout.decal_pallete.AddChunk(layout21);
            }
            foreach (scnr.Decal_Placement_Layout layout22 in chunk.Decal_Placement.Chunks)
            {
                tagblocklayout.decal_placement.AddChunk(layout22);
            }
            list = new List<int>();
            foreach (scnr.Scenario_Kill_Triggers_Layout zone in chunk.Scenario_Kill_Triggers.Chunks)
            {
                num4 = zone.trigger_volume.Value;
                if (num4 != -1)
                {
                    int num5 = list.IndexOf(num4);
                    if (num5 == -1)
                    {
                        list.Add(num4);
                        zone.trigger_volume.Value = (short) (list.Count - 1);
                    }
                    else
                    {
                        zone.trigger_volume.Value = (short) num5;
                    }
                    tagblocklayout.kill_zones.AddChunk(zone);
                }
            }
            for (current = 0; current < list.Count; current++)
            {
                scnr.Trigger_Volumes_Layout layout23 = (scnr.Trigger_Volumes_Layout) chunk.Trigger_Volumes.Chunks[list[current]];
                tagblocklayout.death_zones.AddChunk(layout23);
            }
            foreach (scnr.Decorator_Palette_Layout layout24 in chunk.Decorator_Palette.Chunks)
            {
                if (layout24.Decorator_0.TagNumber != -1)
                {
                    meta3 = new Meta(layout24.Decorator_0.TagNumber, ref tempmap);
                    Decompile(meta3, overwrite);
                    DecompileRecursivelyRoutine(meta3.Interface.Header);
                    tagblocklayout.decorators.AddChunk(layout24);
                }
            }
            TagBlock tagblock = new TagBlock("Temp", Predicted_Scenario_Resources.Layout, Map.HaloVersionEnum.Halo2);
            tagblock.AddChunk(tagblocklayout);
            RecursivelyWriteElements(tagblock, ref nio.Out);
            nio.Close();
            return true;
        }

        private static bool SaveSoundResourceLayout(string tagpath, ref Meta meta)
        {
            snd_ chunk = (snd_) meta.Interface.Header.GetChunk(0);
            EndianIO nio = new EndianIO(tagpath, EndianType.BigEndian);
            nio.Open();
            nio.Out.SeekTo(0);
            TagBlock tagblock = new TagBlock("Temp", Sound_Resource_Layout.Layout, Map.HaloVersionEnum.Halo2);
            tagblock.AddChunk(chunk.Sound_Resource);
            RecursivelyWriteElements(tagblock, ref nio.Out);
            nio.Close();
            return true;
        }

        private static string SRLTagPath(ref Meta m)
        {
            string decompiledTagsPath = ProgramSettings.DecompiledTagsPath;
            if (decompiledTagsPath[decompiledTagsPath.Length - 1] != '\\')
            {
                decompiledTagsPath = decompiledTagsPath + @"\";
            }
            int num = m.Name.LastIndexOf('\\');
            switch (m.HaloVersion)
            {
                case Map.HaloVersionEnum.Halo2:
                    decompiledTagsPath = decompiledTagsPath + @"H2\";
                    break;

                case Map.HaloVersionEnum.HaloCE:
                    decompiledTagsPath = decompiledTagsPath + @"H1\";
                    break;
            }
            decompiledTagsPath = decompiledTagsPath + m.Name.Substring(0, num + 1);
            if (!Directory.Exists(decompiledTagsPath))
            {
                Directory.CreateDirectory(decompiledTagsPath);
            }
            return (decompiledTagsPath + m.Name.Substring(num + 1) + ".srl");
        }

        private static string TagPath(ref Meta m)
        {
            string decompiledTagsPath = ProgramSettings.DecompiledTagsPath;
            if (decompiledTagsPath[decompiledTagsPath.Length - 1] != '\\')
            {
                decompiledTagsPath = decompiledTagsPath + @"\";
            }
            int num = m.Name.LastIndexOf('\\');
            switch (m.HaloVersion)
            {
                case Map.HaloVersionEnum.Halo2:
                    decompiledTagsPath = decompiledTagsPath + @"H2\";
                    break;

                case Map.HaloVersionEnum.HaloCE:
                    decompiledTagsPath = decompiledTagsPath + @"H1\";
                    break;
            }
            decompiledTagsPath = decompiledTagsPath + m.Name.Substring(0, num + 1);
            if (!Directory.Exists(decompiledTagsPath))
            {
                Directory.CreateDirectory(decompiledTagsPath);
            }
            return ((decompiledTagsPath + m.Name.Substring(num + 1)) + "." + m.Interface.Tag_Definition.Extension);
        }
    }
}

