﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;
    using System.IO;

    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class Raw_Tag_Block : Data
    {
        protected int datasize;
        protected TagBlock header;
        protected int header_Size;
        protected RawDataOffset raw_offset;
        protected TagBlock resourceblock;
        protected TagIdent tagident;
        protected short unk1;
        protected short unk2;
        protected int unk3;

        public Raw_Tag_Block(string name, Map.HaloVersionEnum haloversion, Data.RawDataTypes rawdatatype, TagBlockLayout layout) : base(name, haloversion, rawdatatype)
        {
            this.tagident = new TagIdent("Tag Ident");
            if (haloversion == Map.HaloVersionEnum.Halo2)
            {
                switch (rawdatatype)
                {
                    case Data.RawDataTypes.H2_mode1:
                    case Data.RawDataTypes.H2_mode2:
                    case Data.RawDataTypes.H2_weat:
                    case Data.RawDataTypes.H2_DECR:
                    case Data.RawDataTypes.H2_PRTM:
                    case Data.RawDataTypes.H2_ugh_coconuts:
                    case Data.RawDataTypes.H2_sbsp_visualmesh:
                    case Data.RawDataTypes.H2_sbsp_water:
                    case Data.RawDataTypes.H2_sbsp_scenery:
                    case Data.RawDataTypes.H2_sbsp_lightmap:
                    case Data.RawDataTypes.H2_sbsp_unknown1:
                        base.size = 0x24;
                        this.header = new TagBlock("Raw Header", layout, 1, haloversion);
                        this.resourceblock = new TagBlock("Resource_Block", Rsrc_Block.Layout, haloversion, 4);
                        this.raw_offset = new RawDataOffset("");
                        break;
                }
            }
            base.type = BaseObjectType.Raw_Tag_Block;
        }

        public override BaseObject Clone()
        {
            Raw_Tag_Block block = new Raw_Tag_Block(base.name, base.HaloVersion, base.rawdatatype, new TagBlockLayout());
            block.ms = base.ms;
            block.rawdatasize = base.rawdatasize;
            block.size = base.size;
            block.unk1 = this.unk1;
            block.unk2 = this.unk2;
            block.unk3 = this.unk3;
            block.tagident = this.tagident;
            block.header = (TagBlock) this.header.Clone();
            block.resourceblock = (TagBlock) this.resourceblock.Clone();
            block.raw_offset = (RawDataOffset) this.raw_offset.Clone();
            block.datasize = this.datasize;
            block.header_Size = this.header_Size;
            block.SetMetaOffset(base.MetaOffset);
            block.SetOffset(base.Offset);
            block.SetMapOffset(base.MapOffset);
            block.Name = base.Name;
            block.SetType(base.Type);
            return block;
        }

        public void CompileRawData()
        {
            base.ms = new MemoryStream();
            EndianIO.EndianWriter bW = new EndianIO.EndianWriter(base.ms, EndianType.LittleEndian);
            int num = 0;
            bW.BaseStream.Position = 0L;
            core.TagInterface.String.WriteString(ref bW, "hklb", 4);
            bW.Write(0);
            this.header.WriteChunks(ref bW, MetaInterface.InterfaceMode.DecompiledTagMode);
            foreach (BaseObject obj2 in this.header.Chunks[0])
            {
                if ((obj2.Type == BaseObjectType.TagBlock) && (((TagBlock) obj2).ChunkCount != 0))
                {
                    Padding.Write(ref bW, (long) ((int) bW.BaseStream.Position), 4);
                    core.TagInterface.String.WriteString(ref bW, "crsr", 4);
                    ((TagBlock) obj2).WriteChunks(ref bW, MetaInterface.InterfaceMode.DecompiledTagMode);
                    if (((TagBlock) obj2).Layout is Vertex_Declaration)
                    {
                        foreach (Vertex_Declaration declaration in ((TagBlock) obj2).Chunks)
                        {
                            core.TagInterface.String.WriteString(ref bW, "crsr", 4);
                            declaration.Tag_Block.WriteChunks(ref bW, MetaInterface.InterfaceMode.DecompiledTagMode);
                            num++;
                        }
                    }
                    num++;
                }
            }
            core.TagInterface.String.WriteString(ref bW, "fklb", 4);
        }

        public void DecompileRawData()
        {
            if (base.ms != null)
            {
                int index = 0;
                int num2 = 0;
                EndianIO.EndianReader bR = new EndianIO.EndianReader(base.ms, EndianType.LittleEndian);
                bR.BaseStream.Position = num2 + 8;
                this.header.ReadChunks(ref bR, MetaInterface.InterfaceMode.DecompiledTagMode, Map.HaloVersionEnum.Halo2);
                foreach (BaseObject obj2 in this.header.Chunks[0])
                {
                    if ((obj2.Type == BaseObjectType.TagBlock) && (((TagBlock) obj2).ChunkCount != 0))
                    {
                        bR.BaseStream.Position = ((num2 + 8) + this.header_Size) + this.GetResource(index).block_offset.Value;
                        ((TagBlock) obj2).ReadChunks(ref bR, MetaInterface.InterfaceMode.DecompiledTagMode, Map.HaloVersionEnum.Halo2);
                        if (((TagBlock) obj2).Layout is Vertex_Declaration)
                        {
                            foreach (Vertex_Declaration declaration in ((TagBlock) obj2).Chunks)
                            {
                                bR.BaseStream.Position = ((num2 + 8) + this.header_Size) + this.GetResource(index + 1).block_offset.Value;
                                declaration.SetUpTagBlock(this.GetResource(index + 1).block_size.Value);
                                declaration.Tag_Block.ReadChunks(ref bR, MetaInterface.InterfaceMode.DecompiledTagMode, Map.HaloVersionEnum.Halo2);
                                index++;
                            }
                        }
                        index++;
                    }
                }
            }
        }

        public Rsrc_Block GetResource(int index)
        {
            return (Rsrc_Block) this.resourceblock.Chunks[index];
        }

        public void Read(ref EndianIO.EndianReader BR, MetaInterface.InterfaceMode mode)
        {
            if (base.HaloVersion == Map.HaloVersionEnum.Halo2)
            {
                switch (base.rawdatatype)
                {
                    case Data.RawDataTypes.H2_mode1:
                    case Data.RawDataTypes.H2_mode2:
                    case Data.RawDataTypes.H2_weat:
                    case Data.RawDataTypes.H2_DECR:
                    case Data.RawDataTypes.H2_PRTM:
                    case Data.RawDataTypes.H2_ugh_coconuts:
                    case Data.RawDataTypes.H2_sbsp_visualmesh:
                    case Data.RawDataTypes.H2_sbsp_water:
                    case Data.RawDataTypes.H2_sbsp_scenery:
                    case Data.RawDataTypes.H2_sbsp_lightmap:
                    case Data.RawDataTypes.H2_sbsp_unknown1:
                        this.raw_offset.Read(ref BR);
                        base.rawdatasize = BR.ReadInt32();
                        this.header_Size = BR.ReadInt32();
                        this.datasize = BR.ReadInt32();
                        this.resourceblock.Read(ref BR, mode);
                        this.tagident.Read(ref BR, mode);
                        this.unk1 = BR.ReadInt16();
                        this.unk2 = BR.ReadInt16();
                        this.unk3 = BR.ReadInt32();
                        break;
                }
            }
        }

        public override void Read_Raw(ref EndianIO.EndianReader BR)
        {
            base.ms = new MemoryStream(base.Total_Raw_Data_Size);
            base.ms.Write(BR.ReadBytes(base.Total_Raw_Data_Size), 0, base.Total_Raw_Data_Size);
        }

        public void Write(ref EndianIO.EndianWriter BW, MetaInterface.InterfaceMode mode)
        {
            switch (base.rawdatatype)
            {
                case Data.RawDataTypes.H2_mode1:
                case Data.RawDataTypes.H2_mode2:
                case Data.RawDataTypes.H2_weat:
                case Data.RawDataTypes.H2_DECR:
                case Data.RawDataTypes.H2_PRTM:
                case Data.RawDataTypes.H2_ugh_coconuts:
                case Data.RawDataTypes.H2_sbsp_visualmesh:
                case Data.RawDataTypes.H2_sbsp_water:
                case Data.RawDataTypes.H2_sbsp_scenery:
                case Data.RawDataTypes.H2_sbsp_lightmap:
                case Data.RawDataTypes.H2_sbsp_unknown1:
                    this.raw_offset.Write(ref BW);
                    BW.Write(base.rawdatasize);
                    BW.Write(this.header_Size);
                    BW.Write(this.datasize);
                    this.resourceblock.Write(ref BW);
                    this.tagident.Write(ref BW, mode);
                    BW.Write(this.unk1);
                    BW.Write(this.unk2);
                    BW.Write(this.unk3);
                    break;
            }
        }

        [Category("Values")]
        public int Data_Size
        {
            get
            {
                return this.datasize;
            }
            set
            {
                this.datasize = value;
            }
        }

        public TagBlockLayout Header
        {
            get
            {
                return this.header.Chunks[0];
            }
        }

        public int Header_Size
        {
            get
            {
                return this.header.ChunkSize;
            }
        }

        [TypeConverter(typeof(MetaInterface.BaseObjectConverter))]
        public RawDataOffset Raw_Offset
        {
            get
            {
                return this.raw_offset;
            }
        }

        [TypeConverter(typeof(MetaInterface.BaseObjectConverter))]
        public TagBlock Resource_Block
        {
            get
            {
                return this.resourceblock;
            }
        }

        public TagIdent Tag_Ident
        {
            get
            {
                return this.tagident;
            }
        }

        public class BVH : TagBlockLayout
        {
            public static readonly Raw_Tag_Block.BVH Layout = new Raw_Tag_Block.BVH();
            public ByteInteger unk1 = new ByteInteger("");

            public BVH()
            {
                base.Add(this.unk1);
            }
        }

        public enum ChunkType : short
        {
            BVH = 40,
            Byte = 100,
            Indice = 0x20,
            Mesh_BVH_Order = 0x30,
            Sub_Cluster = 0,
            Sub_Cluster_Data = 8,
            Unknown_1 = 0x44,
            Unknown_2 = 0x54,
            Unknown_3 = 0x5c,
            Vertex_Declaration_Element = 0x38
        }

        public class Indice : TagBlockLayout
        {
            public static readonly Raw_Tag_Block.Indice Layout = new Raw_Tag_Block.Indice();
            public ShortInteger vertex_index = new ShortInteger("vertex_index");

            public Indice()
            {
                base.Add(this.vertex_index);
            }
        }

        public class Mesh_BVH_Order : TagBlockLayout
        {
            public static readonly Raw_Tag_Block.Mesh_BVH_Order Layout = new Raw_Tag_Block.Mesh_BVH_Order();
            public ShortInteger unk1 = new ShortInteger("");

            public Mesh_BVH_Order()
            {
                base.Add(this.unk1);
            }
        }

        public class Rsrc_Block : TagBlockLayout
        {
            public LongInteger block_offset = new LongInteger("Block Offset");
            public LongInteger block_size = new LongInteger("Block Size");
            public core.TagInterface.Enumerator child_type = new core.TagInterface.Enumerator("Child Type", core.TagInterface.Enumerator.TagEnumeratorIndexType.Byte);
            public ShortInteger chunk_size = new ShortInteger("Chunk Size");
            public core.TagInterface.Enumerator chunk_type = new core.TagInterface.Enumerator("Chunk Type", core.TagInterface.Enumerator.TagEnumeratorIndexType.Word);
            public core.TagInterface.Enumerator chunk_type2 = new core.TagInterface.Enumerator("Chunk Type", core.TagInterface.Enumerator.TagEnumeratorIndexType.Word);
            public static readonly Raw_Tag_Block.Rsrc_Block Layout = new Raw_Tag_Block.Rsrc_Block();
            public ByteInteger unknown = new ByteInteger("Unknown");

            public Rsrc_Block()
            {
                base.Add(this.child_type);
                base.Add(this.unknown);
                base.Add(this.chunk_type);
                base.Add(this.chunk_type2);
                base.Add(this.chunk_size);
                base.Add(this.block_size);
                base.Add(this.block_offset);
            }
        }

        public class Sub_Cluster : TagBlockLayout
        {
            public RealFloat bounding_radius = new RealFloat("Bounding Radius");
            public RealPoint3D centroid = new RealPoint3D("Centroid");
            public ShortInteger constant1 = new ShortInteger("Constant");
            public ShortInteger constant2 = new ShortInteger("Constant");
            public ShortInteger face_count = new ShortInteger("Face Count");
            public ShortInteger face_start = new ShortInteger("Face Start");
            public static readonly Raw_Tag_Block.Sub_Cluster Layout = new Raw_Tag_Block.Sub_Cluster();
            public ShortInteger leaf_count = new ShortInteger("Leaf Count");
            public ShortInteger leaf_start = new ShortInteger("Leaf Start");
            public Padding pad1 = new Padding(2);
            public Padding pad2 = new Padding(0x10);
            public ShortInteger starting_leaf_number = new ShortInteger("Starting Leaf Number #");
            public RealBounds world_bounds_x = new RealBounds("World Bounds X");
            public RealBounds world_bounds_y = new RealBounds("World Bounds Y");
            public RealBounds world_bounds_z = new RealBounds("World Bounds Z");

            public Sub_Cluster()
            {
                base.Add(this.constant1);
                base.Add(this.constant2);
                base.Add(this.starting_leaf_number);
                base.Add(this.face_start);
                base.Add(this.face_count);
                base.Add(this.leaf_start);
                base.Add(this.leaf_count);
                base.Add(this.pad1);
                base.Add(this.centroid);
                base.Add(this.pad2);
                base.Add(this.bounding_radius);
                base.Add(this.world_bounds_x);
                base.Add(this.world_bounds_y);
                base.Add(this.world_bounds_z);
            }
        }

        public class Sub_Cluster_Data : TagBlockLayout
        {
            public ShortInteger count = new ShortInteger("count");
            public static readonly Raw_Tag_Block.Sub_Cluster_Data Layout = new Raw_Tag_Block.Sub_Cluster_Data();
            public ShortInteger nulled = new ShortInteger("null");
            public ShortInteger start_index = new ShortInteger("start_index");
            public ShortInteger sub_cluster_index = new ShortInteger("Sub-Cluster Index");

            public Sub_Cluster_Data()
            {
                base.Add(this.start_index);
                base.Add(this.count);
                base.Add(this.nulled);
                base.Add(this.sub_cluster_index);
            }
        }

        public class Unknown_1 : TagBlockLayout
        {
            public static readonly Raw_Tag_Block.Unknown_1 Layout = new Raw_Tag_Block.Unknown_1();
            public Padding pad2 = new Padding(0x44);

            public Unknown_1()
            {
                base.Add(this.pad2);
            }
        }

        public class Unknown_2 : TagBlockLayout
        {
            public static readonly Raw_Tag_Block.Unknown_2 Layout = new Raw_Tag_Block.Unknown_2();
            public Padding pad2 = new Padding(4);

            public Unknown_2()
            {
                base.Add(this.pad2);
            }
        }

        public class Unknown_3 : TagBlockLayout
        {
            public static readonly Raw_Tag_Block.Unknown_3 Layout = new Raw_Tag_Block.Unknown_3();
            public Padding pad2 = new Padding(2);

            public Unknown_3()
            {
                base.Add(this.pad2);
            }
        }

        public enum UnknownType : byte
        {
            BVH = 1,
            Null = 0,
            Vertex_Declaration_Child = 2
        }

        public class Vertex_Declaration : TagBlockLayout
        {
            public ByteInteger chunksize = new ByteInteger("chunk size");
            public core.TagInterface.Enumerator chunktype = new core.TagInterface.Enumerator("chunktype", core.TagInterface.Enumerator.TagEnumeratorIndexType.Byte);
            public static readonly Raw_Tag_Block.Vertex_Declaration Layout = new Raw_Tag_Block.Vertex_Declaration();
            public Padding nulled = new Padding(30);
            private TagBlock tagblock;

            public Vertex_Declaration()
            {
                base.Add(this.chunktype);
                base.Add(this.chunksize);
                base.Add(this.nulled);
            }

            public void SetUpTagBlock(int totalchunksize)
            {
                Vertex_Chunk_Type type = (Vertex_Chunk_Type) ((short) this.chunktype.Value);
                Vertex_Chunk_Type type2 = type;
                if (type2 <= Vertex_Chunk_Type.Compressed_Normal)
                {
                    switch (type2)
                    {
                        case Vertex_Chunk_Type.Vertex_Chunk_Type_1:
                            this.tagblock = new TagBlock("Vertex Chunk Type 1", Vertex_Chunk_Type_1.Layout, totalchunksize / Vertex_Chunk_Type_1.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;

                        case Vertex_Chunk_Type.Vertex_Chunk_Type_2:
                            this.tagblock = new TagBlock("Vertex Chunk Type 2", Vertex_Chunk_Type_2.Layout, totalchunksize / Vertex_Chunk_Type_2.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;

                        case (Vertex_Chunk_Type.Vertex_Chunk_Type_1 | Vertex_Chunk_Type.Vertex_Chunk_Type_2):
                        case (Vertex_Chunk_Type.Vertex_Chunk_Type_1 | Vertex_Chunk_Type.Vertex_Chunk_Type_3):
                        case (Vertex_Chunk_Type.Vertex_Chunk_Type_1 | Vertex_Chunk_Type.Vertex_Chunk_Type_2 | Vertex_Chunk_Type.Vertex_Chunk_Type_3):
                        case (Vertex_Chunk_Type.Uncompressed_UV | Vertex_Chunk_Type.Vertex_Chunk_Type_2):
                            return;

                        case Vertex_Chunk_Type.Vertex_Chunk_Type_3:
                            this.tagblock = new TagBlock("Vertex Chunk Type 3", Vertex_Chunk_Type_3.Layout, totalchunksize / Vertex_Chunk_Type_3.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;

                        case Vertex_Chunk_Type.Vertex_Chunk_Type_6:
                            this.tagblock = new TagBlock("Vertex Chunk Type 6", Vertex_Chunk_Type_6.Layout, totalchunksize / Vertex_Chunk_Type_6.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;

                        case Vertex_Chunk_Type.Vertex_Chunk_Type_8:
                            this.tagblock = new TagBlock("Vertex Chunk Type 8", Vertex_Chunk_Type_8.Layout, totalchunksize / Vertex_Chunk_Type_8.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;

                        case Vertex_Chunk_Type.Uncompressed_UV:
                            this.tagblock = new TagBlock("Uncompressed UV", Uncompressed_UV.Layout, totalchunksize / Uncompressed_UV.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;

                        case Vertex_Chunk_Type.Compressed_UV:
                            this.tagblock = new TagBlock("Compressed UV", Compressed_UV.Layout, totalchunksize / Compressed_UV.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;

                        case Vertex_Chunk_Type.Compressed_Normal:
                            this.tagblock = new TagBlock("Compressed Normal", Compressed_Normal.Layout, totalchunksize / Compressed_Normal.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;
                    }
                }
                else
                {
                    switch (type2)
                    {
                        case Vertex_Chunk_Type.Compressed_Lightmap_UV:
                            this.tagblock = new TagBlock("Compressed Lightmap UV", Compressed_Lightmap_UV.Layout, totalchunksize / Compressed_Lightmap_UV.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            break;

                        case Vertex_Chunk_Type.Unknown:
                            this.tagblock = new TagBlock("unknown", Unknown.Layout, totalchunksize / Unknown.Layout.Chunk_Size, Map.HaloVersionEnum.Halo2);
                            return;
                    }
                }
            }

            public TagBlock Tag_Block
            {
                get
                {
                    return this.tagblock;
                }
            }

            public class Compressed_Lightmap_UV : TagBlockLayout
            {
                public static readonly Raw_Tag_Block.Vertex_Declaration.Compressed_Lightmap_UV Layout = new Raw_Tag_Block.Vertex_Declaration.Compressed_Lightmap_UV();
                public ShortInteger u = new ShortInteger("u");
                public ShortInteger v = new ShortInteger("v");

                public Compressed_Lightmap_UV()
                {
                    base.Add(this.u);
                    base.Add(this.v);
                }
            }

            public class Compressed_Normal : TagBlockLayout
            {
                public LongInteger binormal = new LongInteger("y");
                public static readonly Raw_Tag_Block.Vertex_Declaration.Compressed_Normal Layout = new Raw_Tag_Block.Vertex_Declaration.Compressed_Normal();
                public LongInteger normal = new LongInteger("x");
                public LongInteger tangent = new LongInteger("z");

                public Compressed_Normal()
                {
                    base.Add(this.normal);
                    base.Add(this.binormal);
                    base.Add(this.tangent);
                }
            }

            public class Compressed_UV : TagBlockLayout
            {
                public static readonly Raw_Tag_Block.Vertex_Declaration.Compressed_UV Layout = new Raw_Tag_Block.Vertex_Declaration.Compressed_UV();
                public ShortInteger u = new ShortInteger("u");
                public ShortInteger v = new ShortInteger("v");

                public Compressed_UV()
                {
                    base.Add(this.u);
                    base.Add(this.v);
                }
            }

            public class Uncompressed_UV : TagBlockLayout
            {
                public static readonly Raw_Tag_Block.Vertex_Declaration.Uncompressed_UV Layout = new Raw_Tag_Block.Vertex_Declaration.Uncompressed_UV();
                public RealFloat u = new RealFloat("x");
                public RealFloat v = new RealFloat("y");

                public Uncompressed_UV()
                {
                    base.Add(this.u);
                    base.Add(this.v);
                }
            }

            public class Unknown : TagBlockLayout
            {
                public static readonly Raw_Tag_Block.Vertex_Declaration.Unknown Layout = new Raw_Tag_Block.Vertex_Declaration.Unknown();
                public ShortInteger u = new ShortInteger("u");
                public ShortInteger v = new ShortInteger("v");

                public Unknown()
                {
                    base.Add(this.u);
                    base.Add(this.v);
                }
            }

            public enum Vertex_Chunk_Type : short
            {
                Compressed_Lightmap_UV = 0x1f,
                Compressed_Normal = 0x1b,
                Compressed_UV = 0x19,
                Uncompressed_UV = 0x18,
                Unknown = 0x30,
                Vertex_Chunk_Type_1 = 1,
                Vertex_Chunk_Type_2 = 2,
                Vertex_Chunk_Type_3 = 4,
                Vertex_Chunk_Type_6 = 6,
                Vertex_Chunk_Type_8 = 8
            }

            public class Vertex_Chunk_Type_1 : TagBlockLayout
            {
                public static readonly Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_1 Layout = new Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_1();
                public RealPoint3D translation = new RealPoint3D("Translation");

                public Vertex_Chunk_Type_1()
                {
                    base.Add(this.translation);
                }
            }

            public class Vertex_Chunk_Type_2 : TagBlockLayout
            {
                public static readonly Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_2 Layout = new Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_2();
                public ShortInteger translation_x = new ShortInteger("x");
                public ShortInteger translation_y = new ShortInteger("y");
                public ShortInteger translation_z = new ShortInteger("z");

                public Vertex_Chunk_Type_2()
                {
                    base.Add(this.translation_x);
                    base.Add(this.translation_y);
                    base.Add(this.translation_z);
                }
            }

            public class Vertex_Chunk_Type_3 : TagBlockLayout
            {
                public ShortInteger bone = new ShortInteger("bone");
                public static readonly Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_3 Layout = new Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_3();
                public ShortInteger translation_x = new ShortInteger("x");
                public ShortInteger translation_y = new ShortInteger("y");
                public ShortInteger translation_z = new ShortInteger("z");

                public Vertex_Chunk_Type_3()
                {
                    base.Add(this.translation_x);
                    base.Add(this.translation_y);
                    base.Add(this.translation_z);
                    base.Add(this.bone);
                }
            }

            public class Vertex_Chunk_Type_6 : TagBlockLayout
            {
                public static readonly Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_6 Layout = new Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_6();
                public ShortInteger translation_x = new ShortInteger("x");
                public ShortInteger translation_y = new ShortInteger("y");
                public ShortInteger translation_z = new ShortInteger("z");
                public Padding unk = new Padding(6);

                public Vertex_Chunk_Type_6()
                {
                    base.Add(this.translation_x);
                    base.Add(this.translation_y);
                    base.Add(this.translation_z);
                    base.Add(this.unk);
                }
            }

            public class Vertex_Chunk_Type_8 : TagBlockLayout
            {
                public static readonly Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_8 Layout = new Raw_Tag_Block.Vertex_Declaration.Vertex_Chunk_Type_8();
                public ShortInteger translation_x = new ShortInteger("x");
                public ShortInteger translation_y = new ShortInteger("y");
                public ShortInteger translation_z = new ShortInteger("z");
                public Padding unk = new Padding(6);

                public Vertex_Chunk_Type_8()
                {
                    base.Add(this.translation_x);
                    base.Add(this.translation_y);
                    base.Add(this.translation_z);
                    base.Add(this.unk);
                }
            }
        }
    }
}

