﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;

    public class StringID : BaseObject
    {
        public short Index;
        public int StringLength;
        public string Value;

        public StringID(string name) : base(name)
        {
            base.size = 4;
            base.type = BaseObjectType.StringID;
        }

        public override BaseObject Clone()
        {
            core.TagInterface.StringID gid = new core.TagInterface.StringID(base.name);
            gid.Value = this.Value;
            gid.StringLength = this.StringLength;
            gid.Index = this.Index;
            gid.SetMetaOffset(base.MetaOffset);
            gid.SetOffset(base.Offset);
            gid.SetMapOffset(base.MapOffset);
            gid.Name = base.Name;
            gid.SetType(base.Type);
            return gid;
        }

        public void Read(ref EndianIO.EndianReader BR, MetaInterface.InterfaceMode mode)
        {
            if (mode == MetaInterface.InterfaceMode.CompiledTagMode)
            {
                this.Index = BR.ReadInt16();
                BR.ReadByte();
                this.StringLength = BR.ReadByte();
            }
            else if (mode == MetaInterface.InterfaceMode.DecompiledTagMode)
            {
                this.Value = core.TagInterface.String.ReadString(ref BR, 0x40);
                this.StringLength = this.Value.Length;
            }
        }

        public void Write(ref EndianIO.EndianWriter BW, MetaInterface.InterfaceMode mode)
        {
            if (mode == MetaInterface.InterfaceMode.CompiledTagMode)
            {
                byte num = 0;
                BW.Write((ushort) this.Index);
                BW.Write(num);
                BW.Write((byte) this.StringLength);
            }
            else if (mode == MetaInterface.InterfaceMode.DecompiledTagMode)
            {
                core.TagInterface.String.WriteString(ref BW, this.Value, 0x40);
            }
        }

        [Category("Values")]
        public string Text
        {
            get
            {
                return this.Value;
            }
            set
            {
                this.Value = value;
            }
        }
    }
}

