﻿namespace core.TagInterface
{
    using core;
    using core.IO;
    using System;
    using System.ComponentModel;

    public class TagReference : BaseObject
    {
        private Map.HaloVersionEnum HaloVersion;
        private TagIdent ident;
        internal int stringlength;
        private Pointer StringTranslation;
        private HaloTagType tagType;

        public TagReference(string name, Map.HaloVersionEnum haloversion) : base(name)
        {
            this.stringlength = 0;
            this.HaloVersion = haloversion;
            this.tagType = new HaloTagType(name);
            this.StringTranslation = new Pointer(name);
            this.ident = new TagIdent(name);
            switch (this.HaloVersion)
            {
                case Map.HaloVersionEnum.Halo2:
                    base.size = 8;
                    break;

                case Map.HaloVersionEnum.HaloCE:
                case Map.HaloVersionEnum.Halo1:
                    base.size = 0x10;
                    break;
            }
            base.type = BaseObjectType.TagReference;
        }

        public override BaseObject Clone()
        {
            TagReference reference = new TagReference(base.name, this.HaloVersion);
            reference.tagType = (HaloTagType) this.tagType.Clone();
            reference.StringTranslation = (Pointer) this.StringTranslation.Clone();
            reference.stringlength = this.stringlength;
            reference.ident = (TagIdent) this.ident.Clone();
            reference.SetMetaOffset(base.MetaOffset);
            reference.SetOffset(base.Offset);
            reference.SetMapOffset(base.MapOffset);
            reference.Name = base.Name;
            reference.SetType(base.Type);
            return reference;
        }

        public void Read(ref EndianIO.EndianReader BR, MetaInterface.InterfaceMode mode)
        {
            switch (this.HaloVersion)
            {
                case Map.HaloVersionEnum.Halo2:
                    this.TagType.Read(ref BR);
                    this.Ident.Read(ref BR, mode);
                    break;

                case Map.HaloVersionEnum.HaloCE:
                case Map.HaloVersionEnum.Halo1:
                    this.tagType.Read(ref BR);
                    this.StringTranslation.Read(ref BR);
                    this.stringlength = BR.ReadInt32();
                    this.Ident.Read(ref BR, mode);
                    break;
            }
        }

        public void Write(ref EndianIO.EndianWriter BW, MetaInterface.InterfaceMode mode)
        {
            switch (this.HaloVersion)
            {
                case Map.HaloVersionEnum.Halo2:
                    this.TagType.Write(ref BW);
                    this.Ident.Write(ref BW, mode);
                    break;

                case Map.HaloVersionEnum.HaloCE:
                case Map.HaloVersionEnum.Halo1:
                    this.tagType.Write(ref BW);
                    this.StringTranslation.Write(0, 0, ref BW);
                    if (mode == MetaInterface.InterfaceMode.DecompiledTagMode)
                    {
                        this.stringlength = (this.ident.TagName != "null") ? this.ident.TagName.Length : 0;
                    }
                    BW.Write(this.stringlength);
                    this.Ident.Write(ref BW, mode);
                    break;
            }
        }

        [Category("Values")]
        public TagIdent Ident
        {
            get
            {
                return this.ident;
            }
        }

        [Category("Values")]
        public HaloTagType TagType
        {
            get
            {
                return this.tagType;
            }
        }
    }
}

