﻿namespace core
{
    using core.H1;
    using core.H2;
    using System;
    using System.Collections.Generic;

    public sealed class Tag_Layouts : List<Tag_Category>
    {
        public static H1TagCategory h1tagdefs = new H1TagCategory();
        public static H2TagCategory h2tagdefs = new H2TagCategory();
        public static Tag_Layouts Layouts;

        public Tag_Layouts()
        {
            base.Add(h1tagdefs);
            base.Add(h2tagdefs);
        }

        public static string GetFullTagPath(string tagname, string tagtype, Map.HaloVersionEnum haloversion)
        {
            return (GetTagPath(haloversion) + GetMetaPath(tagname, tagtype, haloversion));
        }

        public static string GetMetaPath(string tagname, string tagtype, Map.HaloVersionEnum haloversion)
        {
            int num = tagname.LastIndexOf('\\');
            return (tagname.Substring(0, num + 1) + tagname.Substring(num + 1) + "." + GetTagCategory(haloversion).GetTagDefinitionByTagType(tagtype).Extension);
        }

        public static Tag_Category GetTagCategory(Map.HaloVersionEnum version)
        {
            foreach (Tag_Category category in Layouts)
            {
                if (category.HaloVersion == version)
                {
                    return category;
                }
            }
            return null;
        }

        public static string GetTagPath(Map.HaloVersionEnum haloversion)
        {
            string decompiledTagsPath = ProgramSettings.DecompiledTagsPath;
            if (decompiledTagsPath[decompiledTagsPath.Length - 1] != '\\')
            {
                decompiledTagsPath = decompiledTagsPath + @"\";
            }
            switch (haloversion)
            {
                case Map.HaloVersionEnum.Halo2:
                    return (decompiledTagsPath + @"H2\");

                case Map.HaloVersionEnum.HaloCE:
                    return (decompiledTagsPath + @"H1\");
            }
            return decompiledTagsPath;
        }
    }
}

